@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
-* Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+* Copyright 2018 Pelican Mapping
 * http://osgearth.org
 *
 * osgEarth is free software; you can redistribute it and/or modify
@@ -85,7 +85,7 @@ void printFeature( Feature* feature )
 
 void printAllFeatures(FeatureSource* features)
 {
-    osg::ref_ptr< FeatureCursor > cursor = features->createFeatureCursor();
+    osg::ref_ptr< FeatureCursor > cursor = features->createFeatureCursor(0L);
     while (cursor.valid() && cursor->hasMore())
     {
         osg::ref_ptr< Feature > feature = cursor->nextFeature();
