@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
-* Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+* Copyright 2018 Pelican Mapping
 * http://osgearth.org
 *
 * osgEarth is free software; you can redistribute it and/or modify
@@ -41,6 +41,7 @@
 #include <osgEarth/Capabilities>
 #include <osgEarth/ShaderUtils>
 #include <osgEarth/FileUtils>
+#include <osgEarth/GLUtils>
 #include <osgEarthUtil/Controls>
 
 using namespace osgEarth;
@@ -79,10 +80,9 @@ osg::Geode* makeGeom( float v )
     geom->setVertexArray( verts );
     geom->setUseDisplayList(false);
     geom->setUseVertexBufferObjects(true);
-    osg::Vec4Array* colors = new osg::Vec4Array();
+    osg::Vec4Array* colors = new osg::Vec4Array(osg::Array::BIND_OVERALL);
     colors->push_back( osg::Vec4(0,0,1,1) );
     geom->setColorArray(colors);
-    geom->setColorBinding(osg::Geometry::BIND_OVERALL);
     geom->addPrimitiveSet(new osg::DrawArrays(GL_TRIANGLES,0,3));
     geode->addDrawable(geom);
     return geode;
@@ -296,7 +296,7 @@ namespace TEST_5
 
         osg::Group* root = new osg::Group();
         root->getOrCreateStateSet()->setRenderBinDetails( 0, "TraversalOrderBin" );
-        root->getOrCreateStateSet()->setMode(GL_LIGHTING,0);
+        GLUtils::setLighting(root->getOrCreateStateSet(), 0);
 
         root->addChild( n1 );
         root->addChild( n2 );
