@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
-* Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+* Copyright 2018 Pelican Mapping
 * http://osgearth.org
 *
 * osgEarth is free software; you can redistribute it and/or modify
@@ -20,7 +20,6 @@
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 */
 #include <osgEarth/Map>
-#include <osgEarth/MapFrame>
 #include <osgEarth/MapNode>
 #include <osgEarth/MapModelChange>
 #include <osgEarth/ElevationPool>
@@ -37,11 +36,10 @@ using namespace osgEarth;
 using namespace osgEarth::Util;
 using namespace osgEarth::Util::Controls;
 
-void createControlPanel( osgViewer::View* );
+void createControlPanel(Container*);
 void updateControlPanel();
 
 static osg::ref_ptr<Map> s_activeMap;
-static Grid* s_masterGrid;
 static Grid* s_activeBox;
 static Grid* s_inactiveBox;
 static bool s_updateRequired = true;
@@ -145,7 +143,11 @@ main( int argc, char** argv )
     viewer.getCamera()->setSmallFeatureCullingPixelSize(-1.0f);
 
     // Load an earth file 
-    osg::Node* loaded = osgEarth::Util::MapNodeHelper().load(arguments, &viewer);
+    Container* uiRoot = new VBox();
+    uiRoot->setAbsorbEvents(false);
+    createControlPanel(uiRoot);
+
+    osg::Node* loaded = osgEarth::Util::MapNodeHelper().load(arguments, &viewer, uiRoot);
     osgEarth::MapNode* mapNode = osgEarth::MapNode::get(loaded);
     if ( !mapNode ) {
         OE_WARN << "No osgEarth MapNode found in the loaded file(s)." << std::endl;
@@ -156,16 +158,13 @@ main( int argc, char** argv )
     s_activeMap = mapNode->getMap();
     s_activeMap->addMapCallback( new MyMapListener() );
 
-    osg::Group* root = new osg::Group();
-
-    // install the control panel
-    createControlPanel( &viewer );
-    root->addChild( loaded );
+    //osg::Group* root = new osg::Group();
+    //root->addChild( loaded );
 
     // update the control panel with the two Maps:
     updateControlPanel();
 
-    viewer.setSceneData( root );
+    viewer.setSceneData( loaded );
 
     // install our control panel updater
     viewer.addUpdateOperation( new UpdateOperation() );
@@ -177,6 +176,17 @@ main( int argc, char** argv )
 
 //------------------------------------------------------------------------
 
+struct EnableDisableHandler : public ControlEventHandler
+{
+    EnableDisableHandler( Layer* layer ) : _layer(layer) { }
+    void onClick( Control* control )
+    {
+        _layer->setEnabled( !_layer->getEnabled() );
+        updateControlPanel();
+    }
+    Layer* _layer;
+};
+
 struct ToggleLayerVisibility : public ControlEventHandler
 {
     ToggleLayerVisibility( VisibleLayer* layer ) : _layer(layer) { }
@@ -245,16 +255,43 @@ struct ZoomLayerHandler : public ControlEventHandler
         const GeoExtent& extent = _layer->getExtent();
         if (extent.isValid())
         {
-            ViewFitter fitter(s_activeMap->getSRS(), s_view->getCamera());
             std::vector<GeoPoint> points;
             points.push_back(GeoPoint(extent.getSRS(), extent.west(), extent.south()));
             points.push_back(GeoPoint(extent.getSRS(), extent.east(), extent.north()));
+            
+            ViewFitter fitter(s_activeMap->getSRS(), s_view->getCamera());
             Viewpoint vp;
             if (fitter.createViewpoint(points, vp))
             {
                 s_manip->setViewpoint(vp, 2.0);
             }
         }
+        else if (_layer->getNode())
+        {
+            const osg::BoundingSphere& bs = _layer->getNode()->getBound();
+            if (bs.valid())
+            {
+                osg::Vec3d c = bs.center();
+                double r = bs.radius();
+                const SpatialReference* mapSRS = s_activeMap->getSRS();
+
+                std::vector<GeoPoint> points;
+                GeoPoint p;
+                p.fromWorld(mapSRS, osg::Vec3d(c.x()+r, c.y(), c.z())); points.push_back(p);
+                p.fromWorld(mapSRS, osg::Vec3d(c.x()-r, c.y(), c.z())); points.push_back(p);
+                p.fromWorld(mapSRS, osg::Vec3d(c.x(), c.y()+r, c.z())); points.push_back(p);
+                p.fromWorld(mapSRS, osg::Vec3d(c.x(), c.y()-r, c.z())); points.push_back(p);
+                p.fromWorld(mapSRS, osg::Vec3d(c.x(), c.y(), c.z()+r)); points.push_back(p);
+                p.fromWorld(mapSRS, osg::Vec3d(c.x(), c.y(), c.z()-r)); points.push_back(p);
+
+                ViewFitter fitter(s_activeMap->getSRS(), s_view->getCamera());
+                Viewpoint vp;
+                if (fitter.createViewpoint(points, vp))
+                {
+                    s_manip->setViewpoint(vp, 2.0);
+                }
+            }
+        }
     }
     Layer* _layer;
 };
@@ -263,41 +300,25 @@ struct ZoomLayerHandler : public ControlEventHandler
 
 
 void
-createControlPanel( osgViewer::View* view )
+createControlPanel(Container* container)
 {
-    ControlCanvas* canvas = ControlCanvas::getOrCreate( view );
-
-    s_masterGrid = new Grid();
-    s_masterGrid->setMargin( 5 );
-    s_masterGrid->setPadding( 5 );
-    s_masterGrid->setChildSpacing( 10 );
-    s_masterGrid->setChildVertAlign( Control::ALIGN_CENTER );
-    s_masterGrid->setAbsorbEvents( true );
-    s_masterGrid->setVertAlign( Control::ALIGN_TOP );
-
     //The Map layers
     s_activeBox = new Grid();
-    s_activeBox->setBackColor(0,0,0,0.5);
-    s_activeBox->setMargin( 10 );
+    s_activeBox->setBackColor(0,0,0,0.1);
     s_activeBox->setPadding( 10 );
     s_activeBox->setChildSpacing( 10 );
     s_activeBox->setChildVertAlign( Control::ALIGN_CENTER );
     s_activeBox->setAbsorbEvents( true );
-    s_activeBox->setVertAlign( Control::ALIGN_TOP );
-    s_masterGrid->setControl( 0, 0, s_activeBox );
+    container->addControl(s_activeBox);
 
     //the removed layers
     s_inactiveBox = new Grid();
-    s_inactiveBox->setBackColor(0,0,0,0.5);
-    s_inactiveBox->setMargin( 10 );
+    s_inactiveBox->setBackColor(0,0,0,0.1);
     s_inactiveBox->setPadding( 10 );
     s_inactiveBox->setChildSpacing( 10 );
     s_inactiveBox->setChildVertAlign( Control::ALIGN_CENTER );
     s_inactiveBox->setAbsorbEvents( true );
-    s_inactiveBox->setVertAlign( Control::ALIGN_TOP );
-    s_masterGrid->setControl( 0, 1, s_inactiveBox );
-
-    canvas->addControl( s_masterGrid );
+    container->addControl(s_inactiveBox);
 }
 
 void
@@ -320,20 +341,32 @@ addLayerItem( Grid* grid, int layerIndex, int numLayers, Layer* layer, bool isAc
     
     ElevationLayer* elevationLayer = dynamic_cast<ElevationLayer*>(layer);
 
-    // a checkbox to enable/disable the layer:
+    // a checkbox to toggle the layer's visibility:
     if (visibleLayer && layer->getEnabled() && !(imageLayer && imageLayer->isCoverage()))
     {
-        CheckBoxControl* enabled = new CheckBoxControl( visibleLayer->getVisible() );
-        enabled->addEventHandler( new ToggleLayerVisibility(visibleLayer) );
-        grid->setControl( gridCol, gridRow, enabled );
+        CheckBoxControl* visibility = new CheckBoxControl( visibleLayer->getVisible() );
+        visibility->addEventHandler( new ToggleLayerVisibility(visibleLayer) );
+        grid->setControl( gridCol, gridRow, visibility );
     }
     gridCol++;
 
     // the layer name
-    LabelControl* name = new LabelControl( layer->getName() );
-    if (!layer->getEnabled())
-        name->setForeColor(osg::Vec4f(1,1,1,0.35));
-    grid->setControl( gridCol, gridRow, name );
+    if (layer->getEnabled() && (layer->getExtent().isValid() || layer->getNode()))
+    {
+        ButtonControl* name = new ButtonControl(layer->getName());
+        name->clearBackColor();
+        name->setPadding(4);
+        name->addEventHandler( new ZoomLayerHandler(layer) );
+        grid->setControl( gridCol, gridRow, name );
+    }
+    else
+    {
+        LabelControl* name = new LabelControl( layer->getName() );
+        name->setPadding(4);
+        if (!layer->getEnabled())
+            name->setForeColor(osg::Vec4f(1,1,1,0.35));
+        grid->setControl( gridCol, gridRow, name );
+    }
     gridCol++;
 
     // layer type
@@ -362,16 +395,16 @@ addLayerItem( Grid* grid, int layerIndex, int numLayers, Layer* layer, bool isAc
     }
     gridCol++;
 
-    // zoom button
-    if (layer->getExtent().isValid())
-    {
-        LabelControl* zoomButton = new LabelControl("GO", 14);
-        zoomButton->setBackColor( .4,.4,.4,1 );
-        zoomButton->setActiveColor( .8,0,0,1 );
-        zoomButton->addEventHandler( new ZoomLayerHandler(layer) );
-        grid->setControl( gridCol, gridRow, zoomButton );
-    }
-    gridCol++;
+    //// zoom button
+    //if (layer->getExtent().isValid() || layer->getNode())
+    //{
+    //    LabelControl* zoomButton = new LabelControl("GO", 14);
+    //    zoomButton->setBackColor( .4,.4,.4,1 );
+    //    zoomButton->setActiveColor( .8,0,0,1 );
+    //    zoomButton->addEventHandler( new ZoomLayerHandler(layer) );
+    //    grid->setControl( gridCol, gridRow, zoomButton );
+    //}
+    //gridCol++;
 
     // move buttons
     if ( layerIndex < numLayers-1 && isActive )
@@ -400,10 +433,18 @@ addLayerItem( Grid* grid, int layerIndex, int numLayers, Layer* layer, bool isAc
     addRemove->setBackColor( .4,.4,.4,1 );
     addRemove->setActiveColor( .8,0,0,1 );
     addRemove->addEventHandler( new RemoveLayerHandler(layer) );
-
     grid->setControl( gridCol, gridRow, addRemove );
     gridCol++;
 
+    // enable/disable button
+    LabelControl* enableDisable = new LabelControl(layer->getEnabled() ? "DISABLE" : "ENABLE", 14);
+    enableDisable->setHorizAlign( Control::ALIGN_CENTER );
+    enableDisable->setBackColor( .4,.4,.4,1 );
+    enableDisable->setActiveColor( .8,0,0,1 );
+    enableDisable->addEventHandler( new EnableDisableHandler(layer) );
+    grid->setControl( gridCol, gridRow, enableDisable );
+    gridCol++;
+
     if (layer->getStatus().isError())
     {
         grid->setControl(gridCol, gridRow, new LabelControl(layer->getStatus().message(), osg::Vec4(1,.2,.2,1)));
@@ -438,13 +479,14 @@ updateControlPanel()
 
     int row = 0;
 
-    LabelControl* activeLabel = new LabelControl( "Map Layers", 20, osg::Vec4f(1,1,0,1) );
+    LabelControl* activeLabel = new LabelControl( "Map Layers" );
+    activeLabel->setForeColor(osg::Vec4f(1,1,0,1));
     s_activeBox->setControl( 1, row++, activeLabel );
 
     // the active map layers:
-    MapFrame mapf( s_activeMap.get() );
+    LayerVector layers;
+    s_activeMap->getLayers(layers);
 
-    const LayerVector& layers = mapf.layers();
     for (int i = layers.size()-1; i >= 0; --i)
     {
         Layer* layer = layers[i].get();
