@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
- * Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+ * Copyright 2018 Pelican Mapping
  * http://osgearth.org
  *
  * osgEarth is free software; you can redistribute it and/or modify
@@ -18,10 +18,6 @@
  */
 
 #include <osgEarth/Cube>
-#include <osg/Math>
-#include <osg/Notify>
-#include <sstream>
-#include <algorithm>
 
 using namespace osgEarth;
 
@@ -425,8 +421,7 @@ CubeSpatialReference::_init()
 }
 
 GeoLocator*
-CubeSpatialReference::createLocator(double xmin, double ymin, double xmax, double ymax,
-                                    bool plate_carre) const
+CubeSpatialReference::createLocator(double xmin, double ymin, double xmax, double ymax) const
 {
     int face;
     CubeUtils::cubeToFace( xmin, ymin, xmax, ymax, face );
