@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
- * Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+ * Copyright 2018 Pelican Mapping
  * http://osgearth.org
  *
  * osgEarth is free software; you can redistribute it and/or modify
@@ -19,17 +19,9 @@
 #include <osgEarth/CullingUtils>
 #include <osgEarth/LineFunctor>
 #include <osgEarth/VirtualProgram>
-#include <osgUtil/LineSegmentIntersector>
-#include <osgEarth/GeoData>
 #include <osgEarth/Utils>
-#include <osg/ClusterCullingCallback>
-#include <osg/PrimitiveSet>
-#include <osg/Geode>
 #include <osg/TemplatePrimitiveFunctor>
-#include <osgGA/GUIActionAdapter>
-#include <osgUtil/CullVisitor>
-#include <osgUtil/IntersectionVisitor>
-#include <osgUtil/LineSegmentIntersector>
+#include <osgDB/ObjectWrapper>
 
 using namespace osgEarth;
 
@@ -160,8 +152,8 @@ namespace
             node->accept( *this );
         }
 
-        void apply( osg::Geode& geode )
-        {            
+        void apply(osg::Drawable& drawable)
+        {
             if ( _pass == 1 )
             {
                 osg::Matrixd local2world;
@@ -171,8 +163,7 @@ namespace
                 osg::TemplatePrimitiveFunctor<ComputeMaxNormalLength> pass1;
                 pass1.set( _normalECEF, local2world, &_maxNormalLen );
 
-                for( unsigned i=0; i<geode.getNumDrawables(); ++i )
-                    geode.getDrawable(i)->accept( pass1 );
+                drawable.accept(pass1);
             }
 
             else // if ( _pass == 2 )
@@ -181,8 +172,8 @@ namespace
 
                 osg::TemplatePrimitiveFunctor<ComputeMaxRadius2> pass2;
                 pass2.set( center, &_maxRadius2 );
-                for( unsigned i=0; i<geode.getNumDrawables(); ++i )
-                    geode.getDrawable(i)->accept( pass2 );
+
+                drawable.accept(pass2);
             }
         }
 
@@ -261,17 +252,12 @@ namespace
             _matrixStack.push_back( osg::Matrixd::identity() );
         }
 
-        void apply( osg::Geode& geode )
+        void apply(osg::Drawable& drawable)
         {
             LineFunctor<ComputeMinDeviation> functor;
             functor.set( _matrixStack.back(), _ecefNormal, &_minDeviation );
-
-             for( unsigned i=0; i<geode.getNumDrawables(); ++i )
-             {
-                 geode.getDrawable(i)->accept( functor );
-             }
-
-             traverse(geode);
+            drawable.accept(functor);
+            traverse(drawable);
         }
 
         void apply( osg::Transform& xform )
@@ -313,26 +299,6 @@ Culling::asCullVisitor(osg::NodeVisitor* nv)
     return 0L;
 }
 
-//------------------------------------------------------------------------
-
-void
-DoNotComputeNearFarCullCallback::operator()(osg::Node* node, osg::NodeVisitor* nv)
-{
-    osgUtil::CullVisitor* cv = static_cast< osgUtil::CullVisitor*>( nv );
-    osg::CullSettings::ComputeNearFarMode oldMode;
-    if( cv )
-    {
-        oldMode = cv->getComputeNearFarMode();
-        cv->setComputeNearFarMode( osg::CullSettings::DO_NOT_COMPUTE_NEAR_FAR );
-    }
-    traverse(node, nv);
-    if( cv )
-    {
-        cv->setComputeNearFarMode(oldMode);
-    }
-}
-
-
 //----------------------------------------------------------------------------
 
 bool 
@@ -539,59 +505,12 @@ ClusterCullingFactory::create(const GeoExtent& extent)
         maxRadius);
 }
 
-//------------------------------------------------------------------------
-
-CullNodeByNormal::CullNodeByNormal( const osg::Vec3d& normal )
-{
-    _normal = normal;
-    //_normal.normalize();
-}
-
-void
-CullNodeByNormal::operator()(osg::Node* node, osg::NodeVisitor* nv)
-{
-    osg::Vec3d eye, center, up;
-    osgUtil::CullVisitor* cv = Culling::asCullVisitor(nv);
-
-    cv->getCurrentCamera()->getViewMatrixAsLookAt(eye,center,up);
-
-    eye.normalize();
-    osg::Vec3d normal = _normal;
-    normal.normalize();
-
-    double dotProduct = eye * normal;
-    if ( dotProduct > 0.0 )
-    {
-        traverse(node, nv);
-    }
-}
-
-//------------------------------------------------------------------------
-
-void
-DisableSubgraphCulling::operator()(osg::Node* n, osg::NodeVisitor* v)
-{
-    osgUtil::CullVisitor* cv = Culling::asCullVisitor(v);
-    cv->getCurrentCullingSet().setCullingMask( osg::CullSettings::NO_CULLING );
-    traverse(n, v);
-}
-
 
 //------------------------------------------------------------------------
 
 // The max frame time in ms
 double OcclusionCullingCallback::_maxFrameTime = 10.0;
 
-//OcclusionCullingCallback::OcclusionCullingCallback(const osgEarth::SpatialReference *srs, const osg::Vec3d& world, osg::Node* node):
-//_srs        ( srs ),
-//_world      ( world ),
-//_node       ( node ),
-//_visible    ( true ),
-//_maxAltitude( 200000 )
-//{
-//    //nop
-//}
-//
 OcclusionCullingCallback::OcclusionCullingCallback(GeoTransform* xform) :
 _xform      ( xform ),
 _visible    ( true ),
@@ -865,12 +784,6 @@ ProxyCullVisitor::apply(osg::Transform& node)
     _cv->popFromNodePath();
 }
 
-void 
-ProxyCullVisitor::apply(osg::Geode& node)
-{
-    ProxyCullVisitor::apply(static_cast<osg::Node&>(node));
-}
-
 void 
 ProxyCullVisitor::apply(osg::LOD& node)
 {
@@ -890,7 +803,7 @@ ProxyCullVisitor::apply(osg::Drawable& drawable)
     if (node_state) _cv->pushStateSet(node_state);
 
     osg::RefMatrix& matrix = *_cv->getModelViewMatrix();
-    const osg::BoundingBox& bb = Utils::getBoundingBox(&drawable);
+    const osg::BoundingBox& bb = drawable.getBoundingBox();
 
     bool culledOut = false;
 
@@ -1014,6 +927,7 @@ HorizonCullingProgram::install(osg::StateSet* stateset)
     if ( stateset )
     {
         VirtualProgram* vp = VirtualProgram::getOrCreate(stateset);
+        vp->setName("HorizonCullingProgram");
         vp->setFunction( "oe_horizon_vertex",   horizon_vs, ShaderComp::LOCATION_VERTEX_VIEW );
         vp->setFunction( "oe_horizon_fragment", horizon_fs, ShaderComp::LOCATION_FRAGMENT_COLORING );
     }
@@ -1095,22 +1009,75 @@ ClipToGeocentricHorizon::operator()(osg::Node* node, osg::NodeVisitor* nv)
     traverse(node, nv);
 }
 
+//..........................................................................
+
+void
+AltitudeCullCallback::operator()(osg::Node* node, osg::NodeVisitor* nv)
+{
+    bool visible = true;
+
+    if (_maxAltitude.isSet())
+    {
+        Horizon* horizon = Horizon::get(*nv);
+        if (horizon)
+        {
+            visible = nv->getDistanceToViewPoint(osg::Vec3(0, 0, 0), true) <
+                _maxAltitude.get() + horizon->getRadius();
+        }
+        else if (_srs.valid())
+        {
+            if (_srs->isGeographic())
+            {
+                visible = nv->getDistanceToViewPoint(osg::Vec3(0, 0, 0), true) <
+                    _maxAltitude.get() + _srs->getEllipsoid()->getRadiusEquator();
+            }
+            else
+            {
+                visible = nv->getViewPoint().z() <= _maxAltitude.get();
+            }
+        }
+    }
+
+    if (visible)
+    {
+        traverse(node, nv);
+    }
+}
+
 //......................................................................
 
 namespace
 {
+    Config dumpStateSet(const osg::StateSet* ss)
+    {
+        Config conf("StateSetAttrs");
+        for (osg::StateSet::AttributeList::const_iterator i = ss->getAttributeList().begin(); i != ss->getAttributeList().end(); ++i)
+        {
+            osg::StateAttribute* sa = i->second.first.get();
+            Config saconf(sa->className());
+            conf.add(saconf);
+        }
+        return conf;
+    }
+
+    Config dumpRenderLeaf(osgUtil::RenderLeaf* leaf)
+    {
+        Config conf("Leaf");
+        conf.add("Name", leaf->getDrawable()->getName());
+        conf.add("Depth", leaf->_depth);
+        if (leaf->getDrawable()->getStateSet())
+            conf.add(dumpStateSet(leaf->getDrawable()->getStateSet()));
+        return conf;
+    }
+
     Config dumpStateGraph(osgUtil::StateGraph* sg)
     {
         Config conf("StateGraph");
 
         Config leaves("Leaves");
         for(osgUtil::StateGraph::LeafList::const_iterator i = sg->_leaves.begin(); i != sg->_leaves.end(); ++i)
-        {
-            Config leaf("Leaf");
-            leaf.add("Name", i->get()->getDrawable()->getName());
-            leaf.add("Depth", i->get()->_depth);
-            leaves.add( leaf );
-        }
+            leaves.add(dumpRenderLeaf(i->get()));
+
         if ( !leaves.empty() )
             conf.add(leaves);
 
@@ -1132,8 +1099,18 @@ CullDebugger::dumpRenderBin(osgUtil::RenderBin* bin) const
     Config conf("RenderBin");
     if ( !bin->getName().empty() )
         conf.set("Name", bin->getName());
+    if (bin->getStateSet())
+        conf.set("StateSet", dumpStateSet(bin->getStateSet()));
+
     conf.set("BinNum", bin->getBinNum());
-    
+
+    Config leaves("Leaves");
+    for (osgUtil::RenderBin::RenderLeafList::const_iterator i = bin->getRenderLeafList().begin(); i != bin->getRenderLeafList().end(); ++i)
+        leaves.add(dumpRenderLeaf(*i));
+
+    if (!leaves.empty())
+        conf.add(leaves);
+
     Config sg("StateGraphList");
     sg.add("NumChildren", bin->getStateGraphList().size());
 
@@ -1156,3 +1133,41 @@ CullDebugger::dumpRenderBin(osgUtil::RenderBin* bin) const
 
     return conf;
 }
+
+//...................................................................
+
+void
+InstallViewportSizeUniform::operator()(osg::Node* node, osg::NodeVisitor* nv)
+{
+    osgUtil::CullVisitor* cv = dynamic_cast<osgUtil::CullVisitor*>(nv);
+    const osg::Camera* camera = cv->getCurrentCamera();
+    
+    osg::ref_ptr<osg::StateSet> ss;
+
+    if (camera && camera->getViewport())
+    {
+        ss = new osg::StateSet();
+        ss->addUniform(new osg::Uniform("oe_ViewportSize", osg::Vec2f(
+            camera->getViewport()->width(),
+            camera->getViewport()->height())));
+        cv->pushStateSet(ss.get());
+    }
+
+    traverse(node, nv);
+
+    if (ss.valid())
+        cv->popStateSet();
+}
+
+namespace osgEarth { namespace Serializers { namespace InstallViewportSizeUniform
+{
+    REGISTER_OBJECT_WRAPPER(
+        InstallViewportSizeUniform,
+        new osgEarth::InstallViewportSizeUniform,
+        osgEarth::InstallViewportSizeUniform,
+        "osg::Object osg::NodeCallback osgEarth::InstallViewportSizeUniform")
+    {
+        // no properties
+    }
+} } }
+
