@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
- * Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+ * Copyright 2018 Pelican Mapping
  * http://osgearth.org
  *
  * osgEarth is free software; you can redistribute it and/or modify
@@ -17,14 +17,10 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>
  */
 #include <osgEarth/ElevationLayer>
-#include <osgEarth/VerticalDatum>
 #include <osgEarth/HeightFieldUtils>
 #include <osgEarth/Progress>
-#include <osgEarth/MemCache>
 #include <osgEarth/Metrics>
-#include <osgEarth/ImageUtils>
-#include <osg/Version>
-#include <iterator>
+#include <osgEarth/URI>
 
 using namespace osgEarth;
 using namespace OpenThreads;
@@ -35,6 +31,8 @@ namespace osgEarth {
     REGISTER_OSGEARTH_LAYER(elevation, osgEarth::ElevationLayer);
 }
 
+//#define ANALYZE
+
 //------------------------------------------------------------------------
 
 ElevationLayerOptions::ElevationLayerOptions() :
@@ -76,26 +74,20 @@ Config
 ElevationLayerOptions::getConfig() const
 {
     Config conf = TerrainLayerOptions::getConfig();
-    conf.key() = "elevation";
-
     conf.set("offset", _offset);
     conf.set("nodata_policy", "default",     _noDataPolicy, NODATA_INTERPOLATE );
     conf.set("nodata_policy", "interpolate", _noDataPolicy, NODATA_INTERPOLATE );
     conf.set("nodata_policy", "msl",         _noDataPolicy, NODATA_MSL );
-
-    //if (driver().isSet())
-    //    conf.set("driver", driver()->getDriver());
-
     return conf;
 }
 
 void
 ElevationLayerOptions::fromConfig( const Config& conf )
 {
-    conf.getIfSet("offset", _offset );
-    conf.getIfSet("nodata_policy", "default",     _noDataPolicy, NODATA_INTERPOLATE );
-    conf.getIfSet("nodata_policy", "interpolate", _noDataPolicy, NODATA_INTERPOLATE );
-    conf.getIfSet("nodata_policy", "msl",         _noDataPolicy, NODATA_MSL );
+    conf.get("offset", _offset );
+    conf.get("nodata_policy", "default",     _noDataPolicy, NODATA_INTERPOLATE );
+    conf.get("nodata_policy", "interpolate", _noDataPolicy, NODATA_INTERPOLATE );
+    conf.get("nodata_policy", "msl",         _noDataPolicy, NODATA_MSL );
 }
 
 void
@@ -245,14 +237,23 @@ ElevationLayer::createHeightFieldFromTileSource(const TileKey&    key,
 {
     osg::ref_ptr<osg::HeightField> result;
 
+    if (progress && progress->isCanceled())
+    {
+        return 0L;
+    }
+
     TileSource* source = getTileSource();
     if ( !source )
+    {
+        if (progress) progress->message() = "no tile source";
         return 0L;
+    }
 
     // If the key is blacklisted, fail.
     if ( source->getBlacklist()->contains( key ))
     {
         OE_DEBUG << LC << "Tile " << key.str() << " is blacklisted " << std::endl;
+        if (progress) progress->message() = "blacklisted";
         return 0L;
     }
 
@@ -264,12 +265,13 @@ ElevationLayer::createHeightFieldFromTileSource(const TileKey&    key,
         if (!mayHaveData(key))
         {
             OE_DEBUG << LC << "Source for layer has no data at " << key.str() << std::endl;
+            //if (progress) progress->message() = "mayHaveData=false";
             return 0L;
         }
 
         // Make it from the source:
         result = source->createHeightField( key, getOrCreatePreCacheOp(), progress );
-   
+
         // If the result is good, we how have a heightfield but it's vertical values
         // are still relative to the tile source's vertical datum. Convert them.
         if (result.valid())
@@ -288,8 +290,7 @@ ElevationLayer::createHeightFieldFromTileSource(const TileKey&    key,
         // we can't get it and it wasn't cancelled
         if (!result.valid())
         {
-            if ( progress == 0L ||
-                 ( !progress->isCanceled() && !progress->needsRetry() ) )
+            if ( progress == 0L || !progress->isCanceled() )
             {
                 source->getBlacklist()->add( key );
             }
@@ -304,6 +305,11 @@ ElevationLayer::createHeightFieldFromTileSource(const TileKey&    key,
         assembleHeightField( key, result, dummyNormalMap, progress );
     }
 
+    if (progress && progress->isCanceled())
+    {
+        return 0L;
+    }
+
     return result.release();
 }
 
@@ -314,14 +320,44 @@ ElevationLayer::assembleHeightField(const TileKey& key,
                                     osg::ref_ptr<NormalMap>& out_normalMap,
                                     ProgressCallback* progress)
 {			
-    //osg::HeightField* result = 0L;
-
     // Collect the heightfields for each of the intersecting tiles.
     GeoHeightFieldVector heightFields;
 
     //Determine the intersecting keys
     std::vector< TileKey > intersectingTiles;
-    getProfile()->getIntersectingTiles( key, intersectingTiles );
+    
+    if (key.getLOD() > 0u)
+    {
+        getProfile()->getIntersectingTiles(key, intersectingTiles);
+    }
+
+    else
+    {
+        // LOD is zero - check whether the LOD mapping went out of range, and if so,
+        // fall back until we get valid tiles. This can happen when you have two
+        // profiles with very different tile schemes, and the "equivalent LOD" 
+        // surpasses the max data LOD of the tile source.
+        unsigned numTilesThatMayHaveData = 0u;
+
+        int intersectionLOD = getProfile()->getEquivalentLOD(key.getProfile(), key.getLOD());
+
+        while (numTilesThatMayHaveData == 0u && intersectionLOD >= 0)
+        {
+            intersectingTiles.clear();
+            getProfile()->getIntersectingTiles(key.getExtent(), intersectionLOD, intersectingTiles);
+
+            for (unsigned int i = 0; i < intersectingTiles.size(); ++i)
+            {
+                const TileKey& layerKey = intersectingTiles[i];
+                if (mayHaveData(layerKey) == true)
+                {
+                    ++numTilesThatMayHaveData;
+                }
+            }
+
+            --intersectionLOD;
+        }
+    }
 
     // collect heightfield for each intersecting key. Note, we're hitting the
     // underlying tile source here, so there's no vetical datum shifts happening yet.
@@ -337,72 +373,89 @@ ElevationLayer::assembleHeightField(const TileKey& key,
                 osg::ref_ptr<osg::HeightField> hf;
                 osg::ref_ptr<NormalMap> normalMap;
                 createImplementation(layerKey, hf, normalMap, progress);
-                //osg::HeightField* hf = createHeightFieldImplementation( layerKey, progress );
                 if (hf.valid())
                 {
                     heightFields.push_back( GeoHeightField(hf.get(), normalMap.get(), layerKey.getExtent()) );
                 }
             }
         }
-    }
 
-    // If we actually got a HeightField, resample/reproject it to match the incoming TileKey's extents.
-    if (heightFields.size() > 0)
-    {		
-        unsigned int width = 0;
-        unsigned int height = 0;
+        // If we actually got a HeightField, resample/reproject it to match the incoming TileKey's extents.
+        if (heightFields.size() > 0)
+        {		
+            unsigned int width = 0;
+            unsigned int height = 0;
 
-        for (GeoHeightFieldVector::iterator itr = heightFields.begin(); itr != heightFields.end(); ++itr)
-        {
-            if (itr->getHeightField()->getNumColumns() > width)
-                width = itr->getHeightField()->getNumColumns();
-            if (itr->getHeightField()->getNumRows() > height) 
-                height = itr->getHeightField()->getNumRows();                        
-        }
-
-        //Now sort the heightfields by resolution to make sure we're sampling the highest resolution one first.
-        std::sort( heightFields.begin(), heightFields.end(), GeoHeightField::SortByResolutionFunctor());        
-
-        out_hf = new osg::HeightField();
-        out_hf->allocate(width, height);
-
-        out_normalMap = new NormalMap(width, height);
-
-        //Go ahead and set up the heightfield so we don't have to worry about it later
-        double minx, miny, maxx, maxy;
-        key.getExtent().getBounds(minx, miny, maxx, maxy);
-        double dx = (maxx - minx)/(double)(width-1);
-        double dy = (maxy - miny)/(double)(height-1);
-
-        //Create the new heightfield by sampling all of them.
-        for (unsigned int c = 0; c < width; ++c)
-        {
-            double x = minx + (dx * (double)c);
-            for (unsigned r = 0; r < height; ++r)
+            for (GeoHeightFieldVector::iterator itr = heightFields.begin(); itr != heightFields.end(); ++itr)
             {
-                double y = miny + (dy * (double)r);
+                if (itr->getHeightField()->getNumColumns() > width)
+                    width = itr->getHeightField()->getNumColumns();
+                if (itr->getHeightField()->getNumRows() > height) 
+                    height = itr->getHeightField()->getNumRows();                        
+            }
 
-                //For each sample point, try each heightfield.  The first one with a valid elevation wins.
-                float elevation = NO_DATA_VALUE;
-                osg::Vec3 normal(0,0,1);
+            //Now sort the heightfields by resolution to make sure we're sampling the highest resolution one first.
+            std::sort( heightFields.begin(), heightFields.end(), GeoHeightField::SortByResolutionFunctor());        
 
-                for (GeoHeightFieldVector::iterator itr = heightFields.begin(); itr != heightFields.end(); ++itr)
+            out_hf = new osg::HeightField();
+            out_hf->allocate(width, height);
+
+            out_normalMap = new NormalMap(width, height);
+
+            //Go ahead and set up the heightfield so we don't have to worry about it later
+            double minx, miny, maxx, maxy;
+            key.getExtent().getBounds(minx, miny, maxx, maxy);
+            double dx = (maxx - minx)/(double)(width-1);
+            double dy = (maxy - miny)/(double)(height-1);
+
+            //Create the new heightfield by sampling all of them.
+            for (unsigned int c = 0; c < width; ++c)
+            {
+                double x = minx + (dx * (double)c);
+                for (unsigned r = 0; r < height; ++r)
                 {
-                    // get the elevation value, at the same time transforming it vertically into the 
-                    // requesting key's vertical datum.
-                    float e = 0.0;
-                    osg::Vec3 n;
-                    if (itr->getElevationAndNormal(key.getExtent().getSRS(), x, y, INTERP_BILINEAR, key.getExtent().getSRS(), e, n))
+                    double y = miny + (dy * (double)r);
+
+                    //For each sample point, try each heightfield.  The first one with a valid elevation wins.
+                    float elevation = NO_DATA_VALUE;
+                    osg::Vec3 normal(0,0,1);
+
+                    for (GeoHeightFieldVector::iterator itr = heightFields.begin(); itr != heightFields.end(); ++itr)
                     {
-                        elevation = e;
-                        normal = n;
-                        break;
+                        // get the elevation value, at the same time transforming it vertically into the 
+                        // requesting key's vertical datum.
+                        float e = 0.0;
+                        osg::Vec3 n;
+                        if (itr->getElevationAndNormal(key.getExtent().getSRS(), x, y, INTERP_BILINEAR, key.getExtent().getSRS(), e, n))
+                        {
+                            elevation = e;
+                            normal = n;
+                            break;
+                        }
                     }
+                    out_hf->setHeight( c, r, elevation );   
+                    out_normalMap->set( c, r, normal );
                 }
-                out_hf->setHeight( c, r, elevation );   
-                out_normalMap->set( c, r, normal );
             }
         }
+        else
+        {
+            //if (progress && progress->message().empty())
+            //    progress->message() = "assemble yielded no heightfields";
+        }
+    }
+    else
+    {
+        //if (progress && progress->message().empty())
+        //    progress->message() = "assemble yielded no intersecting tiles";
+    }
+
+
+    // If the progress was cancelled clear out any of the output data.
+    if (progress && progress->isCanceled())
+    {
+        out_hf = 0;
+        out_normalMap = 0;
     }
 }
 
@@ -439,7 +492,10 @@ ElevationLayer::createHeightField(const TileKey&    key,
     bool fromMemCache = false;
 
     // cache key combines the key with the full signature (incl vdatum)
-    std::string cacheKey = Stringify() << key.str() << "_" << key.getProfile()->getFullSignature();
+    // the cache key combines the Key and the horizontal profile.
+    std::string cacheKey = Cache::makeCacheKey(
+        Stringify() << key.str() << "-" << key.getProfile()->getHorizSignature(),
+        "elevation");
     const CachePolicy& policy = getCacheSettings()->cachePolicy().get();
 
     if ( _memCache.valid() )
@@ -537,6 +593,12 @@ ElevationLayer::createHeightField(const TileKey&    key,
                 createImplementation(key, hf, normalMap, progress);
             }
 
+            // Check for cancelation before writing to a cache
+            if (progress && progress->isCanceled())
+            {
+                return GeoHeightField::INVALID;
+            }
+
             // validate it to make sure it's legal.
             if ( hf.valid() && !validateHeightField(hf.get()) )
             {
@@ -582,14 +644,13 @@ ElevationLayer::createHeightField(const TileKey&    key,
         }
     }
 
-    // write to mem cache if needed:
-    if ( result.valid() && !fromMemCache && _memCache.valid() )
+    // Check for cancelation before writing to a cache:
+    if ( progress && progress->isCanceled() )
     {
-        CacheBin* bin = _memCache->getOrCreateDefaultBin();
-        bin->write(cacheKey, result.getHeightField(), 0L);
+        return GeoHeightField::INVALID;
     }
 
-    // post-processing:
+    // post-processing -- must be done before caching because it may alter the heightfield data
     if ( result.valid() )
     {
         if ( options().noDataPolicy() == NODATA_MSL )
@@ -609,13 +670,20 @@ ElevationLayer::createHeightField(const TileKey&    key,
             }
 
             HeightFieldUtils::resolveInvalidHeights(
-                result.getHeightField(),
+                hf.get(),
                 result.getExtent(),
                 NO_DATA_VALUE,
                 geoid );
         }
     }
 
+    // write to mem cache if needed:
+    if ( result.valid() && !fromMemCache && _memCache.valid() )
+    {
+        CacheBin* bin = _memCache->getOrCreateDefaultBin();
+        bin->write(cacheKey, result.getHeightField(), 0L);
+    }
+
     return result;
 }
 
@@ -721,10 +789,10 @@ namespace
                 }
                 else
                 {
-                    int s0 = std::max(s - (s % step), 0);
-                    int s1 = (s%step == 0)? s0 : std::min(s0+step, w-1);
-                    int t0 = std::max(t - (t % step), 0);
-                    int t1 = (t%step == 0)? t0 : std::min(t0+step, h-1);
+                    int s0 = osg::maximum(s - (s % step), 0);
+                    int s1 = (s%step == 0)? s0 : osg::minimum(s0+step, w-1);
+                    int t0 = osg::maximum(t - (t % step), 0);
+                    int t1 = (t%step == 0)? t0 : osg::minimum(t0+step, h-1);
                     
                     if (s0 == s1 && t0 == t1)
                     {
@@ -795,6 +863,14 @@ ElevationLayerVector::populateHeightFieldAndNormalMap(osg::HeightField*      hf,
     LayerDataVector contenders;
     LayerDataVector offsets;
 
+#ifdef ANALYZE
+    struct LayerAnalysis {
+        LayerAnalysis() : samples(0), used(false), failed(false), fallback(false), actualKeyValid(true) { }
+        int samples; bool used; bool failed; bool fallback; bool actualKeyValid; std::string message;
+    };
+    std::map<ElevationLayer*, LayerAnalysis> layerAnalysis;
+#endif
+
     // Track the number of layers that would return fallback data.
     unsigned numFallbackLayers = 0;
 
@@ -857,6 +933,10 @@ ElevationLayerVector::populateHeightFieldAndNormalMap(osg::HeightField*      hf,
                     ld.key = bestKey;
                     ld.index = i;
                 }
+
+#ifdef ANALYZE
+                layerAnalysis[layer].used = true;
+#endif
             }
         }
     }
@@ -880,47 +960,16 @@ ElevationLayerVector::populateHeightFieldAndNormalMap(osg::HeightField*      hf,
     double   ymin       = key.getExtent().yMin();
     double   dx         = key.getExtent().width() / (double)(numColumns-1);
     double   dy         = key.getExtent().height() / (double)(numRows-1);
-
-#if 0
-    // If the incoming heightfield requests a positive border width, 
-    // we need to adjust the extents so that we request data outside the
-    // extent of the tile key:
-    unsigned border = hf->getBorderWidth();
-    if (border > 0u)
-    {
-        dx = key.getExtent().width() / (double)(numColumns - (border*2+1));
-        dy = key.getExtent().height() / (double)(numRows - (border*2+1));
-        xmin -= dx * (double)border;
-        ymin -= dy * (double)border;
-    }
-#endif
-    
+   
     // We will load the actual heightfields on demand. We might not need them all.
-#if 0
     GeoHeightFieldVector heightFields(contenders.size());
     GeoHeightFieldVector offsetFields(offsets.size());
     std::vector<bool>    heightFallback(contenders.size(), false);
     std::vector<bool>    heightFailed(contenders.size(), false);
     std::vector<bool>    offsetFailed(offsets.size(), false);
-#else
-    GeoHeightFieldVector heightFields[9];
-    GeoHeightFieldVector offsetFields[9]; //(offsets.size());
-    std::vector<bool>    heightFallback[9]; //(contenders.size(), false);
-    std::vector<bool>    heightFailed[9]; //(contenders.size(), false);
-    std::vector<bool>    offsetFailed[9]; //(offsets.size(), false);
-
-    for (int n = 0; n < 9; ++n)
-    {
-        heightFields[n].resize(contenders.size());
-        offsetFields[n].resize(offsets.size());
-        heightFallback[n].assign(9, false);
-        heightFailed[n].assign(9, false);
-        offsetFailed[n].assign(9, false);
-    }
-#endif
 
     // The maximum number of heightfields to keep in this local cache
-    unsigned int maxHeightFields = 50;
+    const unsigned maxHeightFields = 50;
     unsigned numHeightFieldsInCache = 0;
 
     const SpatialReference* keySRS = keyToUse.getProfile()->getSRS();
@@ -936,182 +985,198 @@ ElevationLayerVector::populateHeightFieldAndNormalMap(osg::HeightField*      hf,
 
     TileKey scratchKey; // Storage if a new key needs to be constructed
 
-    for (unsigned c = 0; c < numColumns; ++c)
+    bool requiresResample = true;
+
+    // If we only have a single contender layer, and the tile is the same size as the requested 
+    // heightfield then we just use it directly and avoid having to resample it
+    if (contenders.size() == 1 && offsets.empty())
     {
-        double x = xmin + (dx * (double)c);
-        for (unsigned r = 0; r < numRows; ++r)
+        ElevationLayer* layer = contenders[0].layer.get();
+        TileKey& contenderKey = contenders[0].key;
+
+        GeoHeightField layerHF = layer->createHeightField(contenderKey, 0);
+        if (layerHF.valid())
         {
-            double y = ymin + (dy * (double)r);
-
-            // Collect elevations from each layer as necessary.
-            int resolvedIndex = -1;
-
-            osg::Vec3 normal_sum(0,0,0);
-
-            for(int i=0; i<contenders.size() && resolvedIndex<0; ++i)
+            if (layerHF.getHeightField()->getNumColumns() == hf->getNumColumns() &&
+                layerHF.getHeightField()->getNumRows() == hf->getNumRows())
             {
-                ElevationLayer* layer = contenders[i].layer.get();                
-                TileKey& contenderKey = contenders[i].key;
-                int index = contenders[i].index;
+                requiresResample = false;
+                memcpy(hf->getFloatArray()->asVector().data(),
+                    layerHF.getHeightField()->getFloatArray()->asVector().data(),
+                    sizeof(float) * hf->getFloatArray()->size()
+                );
+                deltaLOD->resize(hf->getFloatArray()->size(), 0);
+                realData = true;
+            }
+        }
+    }
 
-                // If there is a border, the edge points may not fall within the key extents 
-                // and we may need to fetch a neighboring key.
+    // If we need to mosaic multiple layers or resample it to a new output tilesize go through a resampling loop.
+    if (requiresResample)
+    {
+        for (unsigned c = 0; c < numColumns; ++c)
+        {
+            double x = xmin + (dx * (double)c);
 
-                int n = 4; // index 4 is the center/default tile
+            // periodically check for cancelation
+            if (progress && progress->isCanceled())
+            {
+                return false;
+            }
 
-#if 0
-                if (border > 0u && !contenderKey.getExtent().contains(x, y))
+            for (unsigned r = 0; r < numRows; ++r)
+            {
+                double y = ymin + (dy * (double)r);
+
+                // Collect elevations from each layer as necessary.
+                int resolvedIndex = -1;
+
+                osg::Vec3 normal_sum(0, 0, 0);
+
+                for (int i = 0; i < contenders.size() && resolvedIndex < 0; ++i)
                 {
-                    int dTx = x < contenderKey.getExtent().xMin() ? -1 : x > contenderKey.getExtent().xMax() ? +1 : 0;
-                    int dTy = y < contenderKey.getExtent().yMin() ? +1 : y > contenderKey.getExtent().yMax() ? -1 : 0;
-                    contenderKey = contenderKey.createNeighborKey(dTx, dTy);
-                    n = (dTy+1)*3 + (dTx+1);
-                }
-#endif
+                    ElevationLayer* layer = contenders[i].layer.get();
+                    TileKey& contenderKey = contenders[i].key;
+                    int index = contenders[i].index;
 
-                if ( heightFailed[n][i] )
-                    continue;
-
-                TileKey* actualKey = &contenderKey;
-
-                GeoHeightField& layerHF = heightFields[n][i];
-
-                if (!layerHF.valid())
-                {
-                    // We couldn't get the heightfield from the cache, so try to create it.
-                    // We also fallback on parent layers to make sure that we have data at the location even if it's fallback.
-                    while (!layerHF.valid() && actualKey->valid() && layer->isKeyInLegalRange(*actualKey))
-                    {
-                        layerHF = layer->createHeightField(*actualKey, progress);
-                        if (!layerHF.valid())
-                        {
-                            if (actualKey != &scratchKey)
-                            {
-                                scratchKey = *actualKey;
-                                actualKey = &scratchKey;
-                            }
-                            *actualKey = actualKey->createParentKey();
-                        }
-                    }
-
-                    // Mark this layer as fallback if necessary.
-                    if (layerHF.valid())
-                    {
-                        heightFallback[n][i] = (*actualKey != contenderKey); // actualKey != contenders[i].second;
-                        numHeightFieldsInCache++;
-                    }
-                    else
-                    {
-                        heightFailed[n][i] = true;
+                    if (heightFailed[i])
                         continue;
-                    }
-                }
 
-                if (layerHF.valid())
-                {
-                    bool isFallback = heightFallback[n][i];
+                    TileKey* actualKey = &contenderKey;
 
-                    // We only have real data if this is not a fallback heightfield.
-                    if (!isFallback)
+                    GeoHeightField& layerHF = heightFields[i];
+
+                    if (!layerHF.valid())
                     {
-                        realData = true;
-                    }
-                    
-                    float elevation;
-                    if (layerHF.getElevation(keySRS, x, y, interpolation, keySRS, elevation))
-                    {
-                        if ( elevation != NO_DATA_VALUE )
+                        // We couldn't get the heightfield from the cache, so try to create it.
+                        // We also fallback on parent layers to make sure that we have data at the location even if it's fallback.
+                        while (!layerHF.valid() && actualKey->valid() && layer->isKeyInLegalRange(*actualKey))
                         {
-                            // remember the index so we can only apply offset layers that
-                            // sit on TOP of this layer.
-                            resolvedIndex = index;
-
-                            hf->setHeight(c, r, elevation);
-
-                            if (deltaLOD)
+                            layerHF = layer->createHeightField(*actualKey, progress);
+                            if (!layerHF.valid())
                             {
-                                (*deltaLOD)[r*numColumns + c] = key.getLOD() - actualKey->getLOD();
+                                if (actualKey != &scratchKey)
+                                {
+                                    scratchKey = *actualKey;
+                                    actualKey = &scratchKey;
+                                }
+                                *actualKey = actualKey->createParentKey();
                             }
                         }
+
+                        // Mark this layer as fallback if necessary.
+                        if (layerHF.valid())
+                        {
+                            heightFallback[i] = (*actualKey != contenderKey); // actualKey != contenders[i].second;
+                            numHeightFieldsInCache++;
+                        }
                         else
                         {
-                            ++nodataCount;
-                        }
-                    }                    
-                }
-
-
-                // Clear the heightfield cache if we have too many heightfields in the cache.
-                if (numHeightFieldsInCache >= maxHeightFields)
-                {
-                    //OE_NOTICE << "Clearing cache" << std::endl;
-                    for (unsigned int j = 0; j < 9; ++j)
-                    {
-                        for (unsigned int k = 0; k < heightFields[j].size(); k++)
-                        {
-                            heightFields[j][k] = GeoHeightField::INVALID;
-                            heightFallback[j][k] = false;
-                        }
-                    }
-                    numHeightFieldsInCache = 0;
-                }
-            }
-
-            for(int i=offsets.size()-1; i>=0; --i)
-            {
-                // Only apply an offset layer if it sits on top of the resolved layer
-                // (or if there was no resolved layer).
-                if (resolvedIndex >= 0 && offsets[i].index < resolvedIndex)
-                    continue;
-
-                TileKey &contenderKey = offsets[i].key;
-
-                // If there is a border, the edge points may not fall within the key extents 
-                // and we may need to fetch a neighboring key.
-
-                int n = 4; // index 4 is the center/default tile
-
-#if 0
-                if (border > 0u && !contenderKey.getExtent().contains(x, y))
-                {
-                    int dTx = x < contenderKey.getExtent().xMin() ? -1 : x > contenderKey.getExtent().xMax() ? +1 : 0;
-                    int dTy = y < contenderKey.getExtent().yMin() ? +1 : x > contenderKey.getExtent().yMax() ? -1 : 0;
-                    contenderKey = contenderKey.createNeighborKey(dTx, dTy);
-                    n = (dTy+1)*3 + (dTx+1);
-                }
+                            heightFailed[i] = true;
+#ifdef ANALYZE
+                            layerAnalysis[layer].failed = true;
+                            layerAnalysis[layer].actualKeyValid = actualKey->valid();
+                            if (progress) layerAnalysis[layer].message = progress->message();
 #endif
-                
-                if ( offsetFailed[n][i] == true )
-                    continue;
+                            continue;
+                        }
+                    }
 
-                GeoHeightField& layerHF = offsetFields[n][i];
-                if ( !layerHF.valid() )
-                {
-                    ElevationLayer* offset = offsets[i].layer.get();
-
-                    layerHF = offset->createHeightField(contenderKey, progress);
-                    if ( !layerHF.valid() )
+                    if (layerHF.valid())
                     {
-                        offsetFailed[n][i] = true;
-                        continue;
+                        bool isFallback = heightFallback[i];
+#ifdef ANALYZE
+                        layerAnalysis[layer].fallback = isFallback;
+#endif
+
+                        // We only have real data if this is not a fallback heightfield.
+                        if (!isFallback)
+                        {
+                            realData = true;
+                        }
+
+                        float elevation;
+                        if (layerHF.getElevation(keySRS, x, y, interpolation, keySRS, elevation))
+                        {
+                            if (elevation != NO_DATA_VALUE)
+                            {
+                                // remember the index so we can only apply offset layers that
+                                // sit on TOP of this layer.
+                                resolvedIndex = index;
+
+                                hf->setHeight(c, r, elevation);
+
+#ifdef ANALYZE
+                                layerAnalysis[layer].samples++;
+#endif
+
+                                if (deltaLOD)
+                                {
+                                    (*deltaLOD)[r*numColumns + c] = key.getLOD() - actualKey->getLOD();
+                                }
+                            }
+                            else
+                            {
+                                ++nodataCount;
+                            }
+                        }
+                    }
+
+
+                    // Clear the heightfield cache if we have too many heightfields in the cache.
+                    if (numHeightFieldsInCache >= maxHeightFields)
+                    {
+                        //OE_NOTICE << "Clearing cache" << std::endl;
+                        for (unsigned int k = 0; k < heightFields.size(); k++)
+                        {
+                            heightFields[k] = GeoHeightField::INVALID;
+                            heightFallback[k] = false;
+                        }
+                        numHeightFieldsInCache = 0;
                     }
                 }
 
-                // If we actually got a layer then we have real data
-                realData = true;
+                for (int i = offsets.size() - 1; i >= 0; --i)
+                {
+                    // Only apply an offset layer if it sits on top of the resolved layer
+                    // (or if there was no resolved layer).
+                    if (resolvedIndex >= 0 && offsets[i].index < resolvedIndex)
+                        continue;
 
-                float elevation = 0.0f;
-                if (layerHF.getElevation(keySRS, x, y, interpolation, keySRS, elevation) &&
-                    elevation != NO_DATA_VALUE)
-                {                    
-                    hf->getHeight(c, r) += elevation;
+                    TileKey &contenderKey = offsets[i].key;
 
-                    // Update the resolution tracker to account for the offset. Sadly this
-                    // will wipe out the resolution of the actual data, and might result in 
-                    // normal faceting. See the comments on "createNormalMap" for more info
-                    if (deltaLOD)
+                    if (offsetFailed[i] == true)
+                        continue;
+
+                    GeoHeightField& layerHF = offsetFields[i];
+                    if (!layerHF.valid())
                     {
-                        (*deltaLOD)[r*numColumns + c] = key.getLOD() - contenderKey.getLOD();
+                        ElevationLayer* offset = offsets[i].layer.get();
+
+                        layerHF = offset->createHeightField(contenderKey, progress);
+                        if (!layerHF.valid())
+                        {
+                            offsetFailed[i] = true;
+                            continue;
+                        }
+                    }
+
+                    // If we actually got a layer then we have real data
+                    realData = true;
+
+                    float elevation = 0.0f;
+                    if (layerHF.getElevation(keySRS, x, y, interpolation, keySRS, elevation) &&
+                        elevation != NO_DATA_VALUE)
+                    {
+                        hf->getHeight(c, r) += elevation;
+
+                        // Update the resolution tracker to account for the offset. Sadly this
+                        // will wipe out the resolution of the actual data, and might result in 
+                        // normal faceting. See the comments on "createNormalMap" for more info
+                        if (deltaLOD)
+                        {
+                            (*deltaLOD)[r*numColumns + c] = key.getLOD() - contenderKey.getLOD();
+                        }
                     }
                 }
             }
@@ -1120,9 +1185,41 @@ ElevationLayerVector::populateHeightFieldAndNormalMap(osg::HeightField*      hf,
 
     if (normalMap)
     {
+        // periodically check for cancelation
+        if (progress && progress->isCanceled())
+        {
+            return false;
+        }
+
         createNormalMap(key.getExtent(), hf, deltaLOD.get(), normalMap);
     }
 
+#ifdef ANALYZE
+    {
+        static Threading::Mutex m;
+        Threading::ScopedMutexLock lock(m);
+        std::cout << key.str() << ": ";
+        for (std::map<ElevationLayer*, LayerAnalysis>::const_iterator i = layerAnalysis.begin();
+            i != layerAnalysis.end(); ++i)
+        {
+            std::cout << i->first->getName() 
+                << " used=" << i->second.used
+                << " failed=" << i->second.failed
+                << " akv=" << i->second.actualKeyValid
+                << " fallback=" << i->second.fallback
+                << " samples=" << i->second.samples
+                << " msg=" << i->second.message
+                << "; ";
+        }
+        std::cout << std::endl;
+    }
+#endif
+
+    if (progress && progress->isCanceled())
+    {
+        return false;
+    }
+
     // Return whether or not we actually read any real data
     return realData;
 }
