@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
- * Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+ * Copyright 2018 Pelican Mapping
  * http://osgearth.org
  *
  * osgEarth is free software; you can redistribute it and/or modify
@@ -17,22 +17,11 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>
  */
 #include <osgEarth/ImageLayer>
-#include <osgEarth/ColorFilter>
-#include <osgEarth/TileSource>
 #include <osgEarth/ImageMosaic>
-#include <osgEarth/ImageUtils>
 #include <osgEarth/Registry>
-#include <osgEarth/StringUtils>
 #include <osgEarth/Progress>
-#include <osgEarth/URI>
-#include <osgEarth/MemCache>
-#include <osgEarth/Registry>
 #include <osgEarth/Capabilities>
 #include <osgEarth/Metrics>
-#include <osg/Version>
-#include <osgDB/WriteFile>
-#include <memory.h>
-#include <limits.h>
 
 using namespace osgEarth;
 using namespace OpenThreads;
@@ -81,8 +70,6 @@ void
 ImageLayerOptions::setDefaults()
 {
     _transparentColor.init( osg::Vec4ub(0,0,0,0) );
-    _minRange.init( 0.0 );
-    _maxRange.init( FLT_MAX );
     _featherPixels.init( false );
     _minFilter.init( osg::Texture::LINEAR_MIPMAP_LINEAR );
     _magFilter.init( osg::Texture::LINEAR );
@@ -101,12 +88,11 @@ ImageLayerOptions::mergeConfig( const Config& conf )
 void
 ImageLayerOptions::fromConfig(const Config& conf)
 {
-    conf.getIfSet( "nodata_image",   _noDataImageFilename );
-    conf.getIfSet( "min_range",      _minRange );
-    conf.getIfSet( "max_range",      _maxRange );
-    conf.getIfSet( "shared",         _shared );
-    conf.getIfSet( "coverage",       _coverage );
-    conf.getIfSet( "feather_pixels", _featherPixels);
+    conf.get( "nodata_image",   _noDataImageFilename );
+    conf.get( "shared",         _shared );
+    conf.get( "coverage",       _coverage );
+    conf.get( "feather_pixels", _featherPixels);
+    conf.get( "altitude",       _altitude );
 
     if ( conf.hasValue( "transparent_color" ) )
         _transparentColor = stringToColor( conf.value( "transparent_color" ), osg::Vec4ub(0,0,0,0));
@@ -117,41 +103,39 @@ ImageLayerOptions::fromConfig(const Config& conf)
         ColorFilterRegistry::instance()->readChain( conf.child("color_filters"), _colorFilters );
     }
 
-    conf.getIfSet("mag_filter","LINEAR",                _magFilter,osg::Texture::LINEAR);
-    conf.getIfSet("mag_filter","LINEAR_MIPMAP_LINEAR",  _magFilter,osg::Texture::LINEAR_MIPMAP_LINEAR);
-    conf.getIfSet("mag_filter","LINEAR_MIPMAP_NEAREST", _magFilter,osg::Texture::LINEAR_MIPMAP_NEAREST);
-    conf.getIfSet("mag_filter","NEAREST",               _magFilter,osg::Texture::NEAREST);
-    conf.getIfSet("mag_filter","NEAREST_MIPMAP_LINEAR", _magFilter,osg::Texture::NEAREST_MIPMAP_LINEAR);
-    conf.getIfSet("mag_filter","NEAREST_MIPMAP_NEAREST",_magFilter,osg::Texture::NEAREST_MIPMAP_NEAREST);
-    conf.getIfSet("min_filter","LINEAR",                _minFilter,osg::Texture::LINEAR);
-    conf.getIfSet("min_filter","LINEAR_MIPMAP_LINEAR",  _minFilter,osg::Texture::LINEAR_MIPMAP_LINEAR);
-    conf.getIfSet("min_filter","LINEAR_MIPMAP_NEAREST", _minFilter,osg::Texture::LINEAR_MIPMAP_NEAREST);
-    conf.getIfSet("min_filter","NEAREST",               _minFilter,osg::Texture::NEAREST);
-    conf.getIfSet("min_filter","NEAREST_MIPMAP_LINEAR", _minFilter,osg::Texture::NEAREST_MIPMAP_LINEAR);
-    conf.getIfSet("min_filter","NEAREST_MIPMAP_NEAREST",_minFilter,osg::Texture::NEAREST_MIPMAP_NEAREST);
+    conf.get("mag_filter","LINEAR",                _magFilter,osg::Texture::LINEAR);
+    conf.get("mag_filter","LINEAR_MIPMAP_LINEAR",  _magFilter,osg::Texture::LINEAR_MIPMAP_LINEAR);
+    conf.get("mag_filter","LINEAR_MIPMAP_NEAREST", _magFilter,osg::Texture::LINEAR_MIPMAP_NEAREST);
+    conf.get("mag_filter","NEAREST",               _magFilter,osg::Texture::NEAREST);
+    conf.get("mag_filter","NEAREST_MIPMAP_LINEAR", _magFilter,osg::Texture::NEAREST_MIPMAP_LINEAR);
+    conf.get("mag_filter","NEAREST_MIPMAP_NEAREST",_magFilter,osg::Texture::NEAREST_MIPMAP_NEAREST);
+    conf.get("min_filter","LINEAR",                _minFilter,osg::Texture::LINEAR);
+    conf.get("min_filter","LINEAR_MIPMAP_LINEAR",  _minFilter,osg::Texture::LINEAR_MIPMAP_LINEAR);
+    conf.get("min_filter","LINEAR_MIPMAP_NEAREST", _minFilter,osg::Texture::LINEAR_MIPMAP_NEAREST);
+    conf.get("min_filter","NEAREST",               _minFilter,osg::Texture::NEAREST);
+    conf.get("min_filter","NEAREST_MIPMAP_LINEAR", _minFilter,osg::Texture::NEAREST_MIPMAP_LINEAR);
+    conf.get("min_filter","NEAREST_MIPMAP_NEAREST",_minFilter,osg::Texture::NEAREST_MIPMAP_NEAREST);
 
-    conf.getIfSet("texture_compression", "none", _texcomp, osg::Texture::USE_IMAGE_DATA_FORMAT);
-    conf.getIfSet("texture_compression", "auto", _texcomp, (osg::Texture::InternalFormatMode)~0);
-    conf.getIfSet("texture_compression", "fastdxt", _texcomp, (osg::Texture::InternalFormatMode)(~0 - 1));
+    conf.get("texture_compression", "none", _texcomp, osg::Texture::USE_IMAGE_DATA_FORMAT);
+    conf.get("texture_compression", "auto", _texcomp, (osg::Texture::InternalFormatMode)~0);
+    conf.get("texture_compression", "fastdxt", _texcomp, (osg::Texture::InternalFormatMode)(~0 - 1));
     //TODO add all the enums
 
     // uniform names
-    conf.getIfSet("shared_sampler", _shareTexUniformName);
-    conf.getIfSet("shared_matrix",  _shareTexMatUniformName);
+    conf.get("shared_sampler", _shareTexUniformName);
+    conf.get("shared_matrix",  _shareTexMatUniformName);
 }
 
 Config
 ImageLayerOptions::getConfig() const
 {
     Config conf = TerrainLayerOptions::getConfig();
-    conf.key() = "image";
 
     conf.set( "nodata_image",   _noDataImageFilename );
-    conf.set( "min_range",      _minRange );
-    conf.set( "max_range",      _maxRange );
     conf.set( "shared",         _shared );
     conf.set( "coverage",       _coverage );
     conf.set( "feather_pixels", _featherPixels );
+    conf.set( "altitude",       _altitude );
 
     if (_transparentColor.isSet())
         conf.set("transparent_color", colorToString( _transparentColor.value()));
@@ -161,7 +145,7 @@ ImageLayerOptions::getConfig() const
         Config filtersConf("color_filters");
         if ( ColorFilterRegistry::instance()->writeChain( _colorFilters, filtersConf ) )
         {
-            conf.update( filtersConf );
+            conf.set( filtersConf );
         }
     }
 
@@ -351,7 +335,7 @@ ImageLayer::open()
 
     // If we are using createTexture to make image tiles,
     // we don't need to load a tile source plugin.
-    if (createTextureSupported())
+    if (useCreateTexture())
     {
         setTileSourceExpected(false);
     }
@@ -364,8 +348,38 @@ ImageLayer::init()
 {
     TerrainLayer::init();
 
+    _useCreateTexture = false;
+
     // image layers render as a terrain texture.
-    setRenderType(RENDERTYPE_TILE);
+    setRenderType(RENDERTYPE_TERRAIN_SURFACE);
+
+    if (options().altitude().isSet())
+    {
+        setAltitude(options().altitude().get());
+    }
+}
+
+void
+ImageLayer::setAltitude(const Distance& value)
+{
+    options().altitude() = value;
+
+    if (value != 0.0)
+    {
+        osg::StateSet* stateSet = getOrCreateStateSet();
+
+        stateSet->addUniform(
+            new osg::Uniform("oe_terrain_altitude", (float)options().altitude()->as(Units::METERS)),
+            osg::StateAttribute::ON | osg::StateAttribute::OVERRIDE);
+
+        stateSet->setMode(GL_CULL_FACE, 0);
+    }
+    else
+    {
+        osg::StateSet* stateSet = getOrCreateStateSet();
+        getOrCreateStateSet()->removeUniform("oe_terrain_altitude");
+        stateSet->removeMode(GL_CULL_FACE);
+    }
 }
 
 void
@@ -379,29 +393,10 @@ ImageLayer::fireCallback(ImageLayerCallback::MethodPtr method)
 }
 
 void
-ImageLayer::setMinVisibleRange( float minVisibleRange )
+ImageLayer::setUseCreateTexture()
 {
-    options().minVisibleRange() = minVisibleRange;
-    fireCallback( &ImageLayerCallback::onVisibleRangeChanged );
-}
-
-float
-ImageLayer::getMinVisibleRange() const
-{
-    return options().minVisibleRange().get();
-}
-
-void
-ImageLayer::setMaxVisibleRange( float maxVisibleRange )
-{
-    options().maxVisibleRange() = maxVisibleRange;
-    fireCallback( &ImageLayerCallback::onVisibleRangeChanged );
-}
-
-float
-ImageLayer::getMaxVisibleRange() const
-{
-    return options().maxVisibleRange().get();
+    _useCreateTexture = true;
+    setTileSourceExpected(false);
 }
 
 bool
@@ -590,7 +585,10 @@ ImageLayer::createImageInKeyProfile(const TileKey&    key,
         << key.getExtent().toString() << std::endl;
 
     // the cache key combines the Key and the horizontal profile.
-    std::string cacheKey = Stringify() << key.str() << "_" << key.getProfile()->getHorizSignature();
+    std::string cacheKey = Cache::makeCacheKey(
+        Stringify() << key.str() << "-" << key.getProfile()->getHorizSignature(),
+        "image");
+
     const CachePolicy& policy = getCacheSettings()->cachePolicy().get();
     
     // Check the layer L2 cache first
@@ -683,6 +681,12 @@ ImageLayer::createImageInKeyProfile(const TileKey&    key,
         ImageUtils::fixInternalFormat( result.getImage() );
     }
 
+    // Check for cancelation before writing to a cache:
+    if (progress && progress->isCanceled())
+    {
+        return GeoImage::INVALID;
+    }
+
     // memory cache first:
     if ( result.valid() && _memCache.valid() )
     {
@@ -775,13 +779,17 @@ ImageLayer::createImageFromTileSource(const TileKey&    key,
     // blacklist this tile for future requests.
     if (result == 0L)
     {
-        if ( progress == 0L ||
-             ( !progress->isCanceled() && !progress->needsRetry() ) )
+        if ( progress == 0L || !progress->isCanceled() )
         {
             source->getBlacklist()->add( key );
         }
     }
 
+    if (progress && progress->isCanceled())
+    {
+        return GeoImage::INVALID;
+    }
+
     return GeoImage(result.get(), key.getExtent());
 }
 
@@ -858,7 +866,7 @@ ImageLayer::assembleImage(const TileKey& key, ProgressCallback* progress)
                 // the tile source did not return a tile, so make a note of it.
                 failedKeys.push_back( *k );
 
-                if (progress && (progress->isCanceled() || progress->needsRetry()))
+                if (progress && progress->isCanceled())
                 {
                     retry = true;
                     break;
@@ -962,6 +970,11 @@ ImageLayer::assembleImage(const TileKey& key, ProgressCallback* progress)
         ImageUtils::featherAlphaRegions( result.getImage() );
     }
 
+    if (progress && progress->isCanceled())
+    {
+        return GeoImage::INVALID;
+    }
+
     return result;
 }
 
@@ -1042,3 +1055,17 @@ ImageLayer::applyTextureCompressionMode(osg::Texture* tex) const
         tex->setInternalFormatMode(options().textureCompression().get());
     }
 }
+
+
+void
+ImageLayer::modifyTileBoundingBox(const TileKey& key, osg::BoundingBox& box) const
+{
+    if (options().altitude().isSet())
+    {
+        if (options().altitude()->as(Units::METERS) > box.zMax())
+        {
+            box.zMax() = options().altitude()->as(Units::METERS);
+        }
+    }
+    TerrainLayer::modifyTileBoundingBox(key, box);
+}
\ No newline at end of file
