@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
- * Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+ * Copyright 2018 Pelican Mapping
  * http://osgearth.org
  *
  * osgEarth is free software; you can redistribute it and/or modify
@@ -18,11 +18,6 @@
  */
 
 #include <osgEarth/ImageMosaic>
-#include <osgEarth/ImageUtils>
-#include <osgEarth/HeightFieldUtils>
-#include <osg/Notify>
-#include <osg/Timer>
-#include <osg/io_utils>
 
 #define LC "[ImageMosaic] "
 
