@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
- * Copyright 2015 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+ * Copyright 2018 Pelican Mapping
  * http://osgearth.org
  *
  * osgEarth is free software; you can redistribute it and/or modify
@@ -25,12 +25,11 @@
  */
 
 #include <osgEarth/JsonUtils>
-#include <osg/Notify>
+
+#include <sstream>
 
 #include <string.h>
-#include <stdio.h>
 #include <stdlib.h>
-#include <sstream>
 
 #if _MSC_VER >= 1400 // VC++ 8.0
 #pragma warning( disable : 4996 )   // disable warning about strdup being deprecated.
@@ -572,6 +571,7 @@ Value::Value( ValueType type )
 
 Value::Value( Int value )
    : type_( intValue )
+   , allocated_( 0 )
    , comments_( 0 )
 # ifdef JSON_VALUE_USE_INTERNAL_MAP
    , itemIsUsed_( 0 )
@@ -583,6 +583,7 @@ Value::Value( Int value )
 
 Value::Value( UInt value )
    : type_( uintValue )
+   , allocated_( 0 )
    , comments_( 0 )
 # ifdef JSON_VALUE_USE_INTERNAL_MAP
    , itemIsUsed_( 0 )
@@ -593,6 +594,7 @@ Value::Value( UInt value )
 
 Value::Value( double value )
    : type_( realValue )
+   , allocated_( 0 )
    , comments_( 0 )
 # ifdef JSON_VALUE_USE_INTERNAL_MAP
    , itemIsUsed_( 0 )
@@ -652,6 +654,7 @@ Value::Value( const CppTL::ConstString &value )
 
 Value::Value( bool value )
    : type_( booleanValue )
+   , allocated_(false)
    , comments_( 0 )
 # ifdef JSON_VALUE_USE_INTERNAL_MAP
    , itemIsUsed_( 0 )
@@ -663,6 +666,7 @@ Value::Value( bool value )
 
 Value::Value( const Value &other )
    : type_( other.type_ )
+   , allocated_(false)
    , comments_( 0 )
 # ifdef JSON_VALUE_USE_INTERNAL_MAP
    , itemIsUsed_( 0 )
@@ -1776,7 +1780,8 @@ Value::end()
 // //////////////////////////////////////////////////////////////////
 
 PathArgument::PathArgument()
-   : kind_( kindNone )
+   : index_(0)
+   , kind_( kindNone )
 {
 }
 
@@ -1791,6 +1796,7 @@ PathArgument::PathArgument( Value::UInt index )
 PathArgument::PathArgument( const char *key )
    : key_( key )
    , kind_( kindKey )
+   , index_(0)
 {
 }
 
@@ -1798,6 +1804,7 @@ PathArgument::PathArgument( const char *key )
 PathArgument::PathArgument( const std::string &key )
    : key_( key.c_str() )
    , kind_( kindKey )
+   , index_(0)
 {
 }
 
@@ -2016,8 +2023,15 @@ namespace osgEarth {
 // Class Reader
 // //////////////////////////////////////////////////////////////////
 
-Reader::Reader()
+Reader::Reader() :
+begin_(0),
+end_(0),
+current_(0),
+lastValueEnd_(0),
+lastValue_(0),
+collectComments_(false)
 {
+    //nop
 }
 
 bool
@@ -2497,7 +2511,7 @@ Reader::decodeDouble( Token &token )
    int length = int(token.end_ - token.start_);
    if ( length <= bufferSize )
    {
-      Char buffer[bufferSize];
+      Char buffer[bufferSize+1];
       memcpy( buffer, token.start_, length );
       buffer[length] = 0;
       count = sscanf( buffer, "%lf", &value );
@@ -2959,6 +2973,7 @@ FastWriter::writeValue( const Value &value )
 StyledWriter::StyledWriter()
    : rightMargin_( 74 )
    , indentSize_( 3 )
+   , addChildValues_(false)
 {
 }
 
@@ -3235,6 +3250,7 @@ StyledStreamWriter::StyledStreamWriter( std::string indentation )
    : document_(NULL)
    , rightMargin_( 74 )
    , indentation_( indentation )
+   , addChildValues_(false)
 {
 }
 
