@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
- * Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+ * Copyright 2018 Pelican Mapping
  * http://osgearth.org
  *
  * osgEarth is free software; you can redistribute it and/or modify
@@ -17,15 +17,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>
  */
 #include <osgEarth/Map>
-#include <osgEarth/MapFrame>
 #include <osgEarth/MapModelChange>
 #include <osgEarth/Registry>
-#include <osgEarth/TileSource>
-#include <osgEarth/HeightFieldUtils>
-#include <osgEarth/URI>
-#include <osgEarth/ElevationPool>
 #include <osgEarth/Utils>
-#include <iterator>
 
 using namespace osgEarth;
 
@@ -33,20 +27,20 @@ using namespace osgEarth;
 
 //------------------------------------------------------------------------
 
-Map::ElevationLayerCB::ElevationLayerCB(Map* map) :
-_map(map)
-{
-    //nop
-}
+Map::VisibleLayerCB::VisibleLayerCB(Map* map) : _map(map) { }
 
-void
-Map::ElevationLayerCB::onVisibleChanged(VisibleLayer* layer)
-{
+void Map::VisibleLayerCB::onVisibleChanged(VisibleLayer* layer) {
     osg::ref_ptr<Map> map;
     if ( _map.lock(map) )
-    {
-        _map->notifyElevationLayerVisibleChanged(layer);
-    }
+        _map->notifyLayerVisibleChanged(layer);
+}
+
+Map::LayerCB::LayerCB(Map* map) : _map(map) { }
+
+void Map::LayerCB::onEnabledChanged(Layer* layer) {
+    osg::ref_ptr<Map> map;
+    if (_map.lock(map))
+        map->notifyOnLayerEnabledChanged(layer);
 }
 
 //------------------------------------------------------------------------
@@ -121,9 +115,11 @@ Map::ctor()
     // encode this map in the read options.
     OptionsData<const Map>::set(_readOptions.get(), "osgEarth.Map", this);
 
-    // set up a callback that the Map will use to detect Elevation Layer
-    // visibility changes
-    _elevationLayerCB = new ElevationLayerCB(this);
+    // set up a callback that the Map will use to detect Layer visibility changes
+    _visibleLayerCB = new VisibleLayerCB(this);
+
+    // create a callback that the Map will use to detect setEnabled calls
+    _layerCB = new LayerCB(this);
 
     // elevation sampling
     _elevationPool = new ElevationPool();
@@ -133,7 +129,6 @@ Map::ctor()
 Map::~Map()
 {
     OE_DEBUG << LC << "~Map" << std::endl;
-    clear();
 }
 
 ElevationPool*
@@ -143,23 +138,66 @@ Map::getElevationPool() const
 }
 
 void
-Map::notifyElevationLayerVisibleChanged(VisibleLayer* layer)
+Map::notifyLayerVisibleChanged(VisibleLayer* layer)
 {
     // bump the revision safely:
     Revision newRevision;
     {
-        Threading::ScopedWriteLock lock( const_cast<Map*>(this)->_mapDataMutex );
+        Threading::ScopedWriteLock lock(_mapDataMutex);
+        newRevision = ++_dataModelRevision;
+    }
+
+    ElevationLayer* elevationLayer = dynamic_cast<ElevationLayer*>(layer);
+    if (elevationLayer)
+    {
+        // reinitialize the elevation pool:
+        _elevationPool->clear();
+    }
+
+    MapModelChange change(
+        MapModelChange::TOGGLE_LAYER,
+        newRevision,
+        layer);
+
+    for( MapCallbackList::iterator i = _mapCallbacks.begin(); i != _mapCallbacks.end(); i++ )
+    {
+        i->get()->onMapModelChanged(change);
+    }
+}
+
+void
+Map::notifyOnLayerEnabledChanged(Layer* layer)
+{
+    // bump the revision safely:
+    Revision newRevision;
+    {
+        Threading::ScopedWriteLock lock(_mapDataMutex);
         newRevision = ++_dataModelRevision;
     }
 
     // reinitialize the elevation pool:
-    _elevationPool->clear();
+    if (dynamic_cast<ElevationLayer*>(layer))
+    {
+        _elevationPool->clear();
+    }
+
+    if (layer->getEnabled())
+    {
+        openLayer(layer);
+    }
+    else
+    {
+        closeLayer(layer);
+    }
+
+    MapModelChange change(
+        layer->getEnabled() ? MapModelChange::ENABLE_LAYER : MapModelChange::DISABLE_LAYER,
+        newRevision,
+        layer);
 
-    // a separate block b/c we don't need the mutex
     for( MapCallbackList::iterator i = _mapCallbacks.begin(); i != _mapCallbacks.end(); i++ )
     {
-        i->get()->onMapModelChanged( MapModelChange(
-            MapModelChange::TOGGLE_ELEVATION_LAYER, newRevision, layer) );
+        i->get()->onMapModelChanged(change);
     }
 }
 
@@ -219,6 +257,29 @@ Map::setCache(Cache* cache)
         cacheSettings->setCache(cache);
 }
 
+void
+Map::getAttributions(StringSet& attributions) const
+{
+    LayerVector layers;
+    getLayers(layers);
+
+    for (LayerVector::const_iterator itr = layers.begin(); itr != layers.end(); ++itr)
+    {
+        if (itr->get()->getEnabled())
+        {
+            VisibleLayer* visibleLayer = dynamic_cast<VisibleLayer*>(itr->get());
+            if (!visibleLayer || visibleLayer->getVisible())
+            {
+                std::string attribution = itr->get()->getAttribution();
+                if (!attribution.empty())
+                {
+                    attributions.insert(attribution);
+                }
+            }
+        }
+    }
+}
+
 MapCallback*
 Map::addMapCallback(MapCallback* cb) const
 {
@@ -262,40 +323,23 @@ Map::endUpdate()
 void
 Map::addLayer(Layer* layer)
 {
+    // Store in a ref_ptr for scope to ensure callbacks don't accidentally delete while adding
+    osg::ref_ptr<Layer> layerRef( layer );
     osgEarth::Registry::instance()->clearBlacklist();
     if ( layer )
     {
+        // Set up callbacks
+        installLayerCallbacks(layer);
+
+        // Open the layer if it's enabled:
         if (layer->getEnabled())
         {
-            // Pass along the Read Options (including the cache settings, etc.) to the layer:
-            layer->setReadOptions(_readOptions.get());
-            
-            // If this is a terrain layer, tell it about the Map profile.
-            TerrainLayer* terrainLayer = dynamic_cast<TerrainLayer*>(layer);
-            if (terrainLayer && _profile.valid())
-            {
-                terrainLayer->setTargetProfileHint( _profile.get() );
-            }            
-
-            // Attempt to open the layer. Don't check the status here.
-            layer->open();
-
-            // If this is an elevation layer, install a callback so we know when
-            // it's visibility changes:
-            ElevationLayer* elevationLayer = dynamic_cast<ElevationLayer*>(layer);
-            if (elevationLayer)
-            {
-                elevationLayer->addCallback(_elevationLayerCB.get());
-
-                // invalidate the elevation pool
-                getElevationPool()->clear();
-            }
+            openLayer(layer);
         }
 
+        // Add the layer to our stack.
         int newRevision;
         unsigned index = -1;
-
-        // Add the layer to our stack.
         {
             Threading::ScopedWriteLock lock( _mapDataMutex );
 
@@ -305,7 +349,7 @@ Map::addLayer(Layer* layer)
         }
 
         // tell the layer it was just added.
-        layer->addedToMap(this);
+        //layer->addedToMap(this);
 
         // a separate block b/c we don't need the mutex
         for( MapCallbackList::iterator i = _mapCallbacks.begin(); i != _mapCallbacks.end(); i++ )
@@ -322,36 +366,17 @@ Map::insertLayer(Layer* layer, unsigned index)
     osgEarth::Registry::instance()->clearBlacklist();
     if ( layer )
     {
+        // Set up callbacks
+        installLayerCallbacks(layer);
+
+        // Open the layer if it's enabled:
         if (layer->getEnabled())
         {
-            // Pass along the Read Options (including the cache settings, etc.) to the layer:
-            layer->setReadOptions(_readOptions.get());
-            
-            // If this is a terrain layer, tell it about the Map profile.
-            TerrainLayer* terrainLayer = dynamic_cast<TerrainLayer*>(layer);
-            if (terrainLayer && _profile.valid())
-            {
-                terrainLayer->setTargetProfileHint( _profile.get() );
-            }            
-
-            // Attempt to open the layer. Don't check the status here.
-            layer->open();
-
-            // If this is an elevation layer, install a callback so we know when
-            // it's visibility changes:
-            ElevationLayer* elevationLayer = dynamic_cast<ElevationLayer*>(layer);
-            if (elevationLayer)
-            {
-                elevationLayer->addCallback(_elevationLayerCB.get());
-
-                // invalidate the elevation pool
-                getElevationPool()->clear();
-            }
+            openLayer(layer);
         }
 
-        int newRevision;
-
         // Add the layer to our stack.
+        int newRevision;
         {
             Threading::ScopedWriteLock lock( _mapDataMutex );
 
@@ -364,7 +389,7 @@ Map::insertLayer(Layer* layer, unsigned index)
         }
 
         // tell the layer it was just added.
-        layer->addedToMap(this);
+        //layer->addedToMap(this);
 
         // a separate block b/c we don't need the mutex
         for( MapCallbackList::iterator i = _mapCallbacks.begin(); i != _mapCallbacks.end(); i++ )
@@ -387,6 +412,8 @@ Map::removeLayer(Layer* layer)
     osg::ref_ptr<Layer> layerToRemove = layer;
     Revision newRevision;
 
+    closeLayer(layer);
+
     if ( layerToRemove.get() )
     {
         Threading::ScopedWriteLock lock( _mapDataMutex );
@@ -402,20 +429,13 @@ Map::removeLayer(Layer* layer)
         }
 
         // tell the layer it was just removed.
-        layerToRemove->removedFromMap(this);
+        //layerToRemove->removedFromMap(this);
     }
 
-    ElevationLayer* elevationLayer = dynamic_cast<ElevationLayer*>(layerToRemove.get());
-    if (elevationLayer)
-    {
-        elevationLayer->removeCallback(_elevationLayerCB.get());
-
-        // invalidate the pool
-        getElevationPool()->clear();
-    }
+    uninstallLayerCallbacks(layerToRemove.get());
 
     // a separate block b/c we don't need the mutex
-    if ( newRevision >= 0 ) // layerToRemove.get() )
+    if ( newRevision >= 0 )
     {
         for( MapCallbackList::iterator i = _mapCallbacks.begin(); i != _mapCallbacks.end(); i++ )
         {
@@ -478,12 +498,83 @@ Map::moveLayer(Layer* layer, unsigned newIndex)
     }
 }
 
+void
+Map::installLayerCallbacks(Layer* layer)
+{
+    VisibleLayer* visibleLayer = dynamic_cast<VisibleLayer*>(layer);
+    if (visibleLayer)
+    {
+        visibleLayer->addCallback(_visibleLayerCB.get());
+    }
+
+    // If this is an elevation layer, install a callback so we know when
+    // it's visibility changes:
+    ElevationLayer* elevationLayer = dynamic_cast<ElevationLayer*>(layer);
+    if (elevationLayer)
+    {
+        // invalidate the elevation pool
+        getElevationPool()->clear();
+    }
+
+    // Callback to detect changes in "enabled"
+    layer->addCallback(_layerCB.get());
+}
+
+void
+Map::uninstallLayerCallbacks(Layer* layer)
+{
+    VisibleLayer* visibleLayer = dynamic_cast<VisibleLayer*>(layer);
+    if (visibleLayer)
+    {
+        visibleLayer->removeCallback(_visibleLayerCB.get());
+    }
+
+    // undo the things we did in prepareLayer:
+    ElevationLayer* elevationLayer = dynamic_cast<ElevationLayer*>(layer);
+    if (elevationLayer)
+    {
+        // invalidate the pool
+        getElevationPool()->clear();
+    }
+
+    layer->removeCallback(_layerCB.get());
+}
+
+void
+Map::openLayer(Layer* layer)
+{
+    // Pass along the Read Options (including the cache settings, etc.) to the layer:
+    layer->setReadOptions(_readOptions.get());
+
+    // If this is a terrain layer, tell it about the Map profile.
+    TerrainLayer* terrainLayer = dynamic_cast<TerrainLayer*>(layer);
+    if (terrainLayer && _profile.valid())
+    {
+        terrainLayer->setTargetProfileHint(_profile.get());
+    }
+
+    // Attempt to open the layer. Don't check the status here.
+    if (layer->open().isOK())
+    {
+        layer->addedToMap(this);
+    }
+}
+
+void
+Map::closeLayer(Layer* layer)
+{
+    if (layer)
+    {
+        layer->removedFromMap(this);
+    }
+}
+
 Revision
 Map::getLayers(LayerVector& out_list) const
 {
     out_list.reserve( _layers.size() );
 
-    Threading::ScopedReadLock lock( const_cast<Map*>(this)->_mapDataMutex );
+    Threading::ScopedReadLock lock(_mapDataMutex);
     for( LayerVector::const_iterator i = _layers.begin(); i != _layers.end(); ++i )
         out_list.push_back( i->get() );
 
@@ -493,14 +584,14 @@ Map::getLayers(LayerVector& out_list) const
 unsigned
 Map::getNumLayers() const
 {
-    Threading::ScopedReadLock lock( const_cast<Map*>(this)->_mapDataMutex );
+    Threading::ScopedReadLock lock( _mapDataMutex );
     return _layers.size();
 }
 
 Layer*
 Map::getLayerByName(const std::string& name) const
 {
-    Threading::ScopedReadLock( const_cast<Map*>(this)->_mapDataMutex );
+    Threading::ScopedReadLock lock( _mapDataMutex );
     for(LayerVector::const_iterator i = _layers.begin(); i != _layers.end(); ++i)
         if ( i->get()->getName() == name )
             return i->get();
@@ -510,7 +601,7 @@ Map::getLayerByName(const std::string& name) const
 Layer*
 Map::getLayerByUID(UID layerUID) const
 {
-    Threading::ScopedReadLock( const_cast<Map*>(this)->_mapDataMutex );
+    Threading::ScopedReadLock lock( _mapDataMutex );
     for( LayerVector::const_iterator i = _layers.begin(); i != _layers.end(); ++i )
         if ( i->get()->getUID() == layerUID )
             return i->get();
@@ -520,7 +611,7 @@ Map::getLayerByUID(UID layerUID) const
 Layer*
 Map::getLayerAt(unsigned index) const
 {
-    Threading::ScopedReadLock( const_cast<Map*>(this)->_mapDataMutex );
+    Threading::ScopedReadLock lock( _mapDataMutex );
     if ( index >= 0 && index < (int)_layers.size() )
         return _layers[index].get();
     else
@@ -530,7 +621,7 @@ Map::getLayerAt(unsigned index) const
 unsigned
 Map::getIndexOfLayer(const Layer* layer) const
 {
-    Threading::ScopedReadLock( const_cast<Map*>(this)->_mapDataMutex );
+    Threading::ScopedReadLock lock( _mapDataMutex );
     unsigned index = 0;
     for (; index < _layers.size(); ++index)
     {
@@ -558,12 +649,16 @@ Map::clear()
     // a separate block b/c we don't need the mutex
     for( MapCallbackList::iterator i = _mapCallbacks.begin(); i != _mapCallbacks.end(); i++ )
     {
+        i->get()->onBeginUpdate();
+
         for(LayerVector::iterator layer = layersRemoved.begin();
             layer != layersRemoved.end();
             ++layer)
         {
             i->get()->onMapModelChanged(MapModelChange(MapModelChange::REMOVE_LAYER, newRevision, layer->get()));
         }
+
+        i->get()->onEndUpdate();
     }
 
     // Invalidate the elevation pool.
@@ -647,13 +742,13 @@ Map::calculateProfile()
 
         // Finally, if there is still no profile, default to global geodetic.
         if (!profile.valid())
-        {            
+        {
             profile = Registry::instance()->getGlobalGeodeticProfile();
         }
 
 
         // Set the map's profile!
-        _profile = profile.release();        
+        _profile = profile.release();
 
         // create a "proxy" profile to use when querying elevation layers with a vertical datum
         if ( _profile->getSRS()->getVerticalDatum() != 0L )
@@ -672,7 +767,7 @@ Map::calculateProfile()
 
         for( MapCallbackList::iterator i = _mapCallbacks.begin(); i != _mapCallbacks.end(); i++ )
         {
-            i->get()->onMapInfoEstablished( MapInfo(this) );
+            //i->get()->onMapInfoEstablished( MapInfo(this) );
         }
     }
 
@@ -689,118 +784,46 @@ Map::calculateProfile()
 const SpatialReference*
 Map::getWorldSRS() const
 {
-    return getSRS() && getSRS()->isGeographic() ? getSRS()->getECEF() : getSRS();
+    return getSRS() && getSRS()->isGeographic() ? getSRS()->getGeocentricSRS() : getSRS();
 }
 
 bool
-Map::sync(MapFrame& frame) const
+Map::isFast(const TileKey& key, const LayerVector& layers) const
 {
-    bool result = false;
+    if (getCache() == NULL)
+        return false;
 
-    if ( frame._mapDataModelRevision != _dataModelRevision || !frame._initialized )
+    for (LayerVector::const_iterator i = layers.begin(); i != layers.end(); ++i)
     {
-        // hold the read lock while copying the layer lists.
-        Threading::ScopedReadLock lock( const_cast<Map*>(this)->_mapDataMutex );
+        Layer* layer = i->get();
+        if (!layer)
+            continue;
 
-        if (!frame._initialized)
-            frame._layers.reserve(_layers.size());
+        if (!layer->getEnabled())
+            continue;
 
-        frame._layers.clear();
+        TerrainLayer* terrainlayer = dynamic_cast<TerrainLayer*>(layer);
+        if (terrainlayer)
+        {
+            if (terrainlayer->getCacheSettings()->cachePolicy()->isCacheDisabled())
+              return false;
 
-        std::copy(_layers.begin(), _layers.end(), std::back_inserter(frame._layers));
+            //If no data is available on this tile, we'll be fast
+            if (!terrainlayer->mayHaveData(key))
+                continue;
 
-        // sync the revision numbers.
-        frame._initialized = true;
-        frame._mapDataModelRevision = _dataModelRevision;
+            // No tile source? skip it
+            osg::ref_ptr< TileSource > source = terrainlayer->getTileSource();
+            if (!source.valid())
+                continue;
 
-        result = true;
+            //If the tile is blacklisted, it should also be fast.
+            if (source->getBlacklist()->contains(key))
+                continue;
+
+            if (!terrainlayer->isCached(key))
+                return false;
+        }
     }
-    return result;
-}
-
-
-//......................................................................
-// @deprecated functions
-
-#include <osgEarth/ImageLayer>
-#include <osgEarth/ElevationLayer>
-#include <osgEarth/ModelLayer>
-#include <osgEarth/MaskLayer>
-
-void 
-Map::addImageLayer(class ImageLayer* layer) {
-    OE_DEPRECATED(Map::addImageLayer, Map::addLayer);
-    addLayer(layer);
-}
-
-void 
-Map::insertImageLayer(class ImageLayer* layer, unsigned index) {
-    OE_DEPRECATED(Map::insertImageLayer, Map::insertLayer);
-    insertLayer(layer, index);
-}
-
-void 
-Map::removeImageLayer(class ImageLayer* layer) {
-    OE_DEPRECATED(Map::removeImageLayer, Map::removeLayer);
-    removeLayer(layer);
-}
-
-void 
-Map::moveImageLayer(class ImageLayer* layer, unsigned newIndex) {
-    OE_DEPRECATED(Map::moveImageLayer, Map::moveLayer);
-    moveLayer(layer, newIndex);
-}
-
-void 
-Map::addElevationLayer(class ElevationLayer* layer) {
-    OE_DEPRECATED(Map::addElevationLayer, Map::addLayer);
-    addLayer(layer);
-}
-
-void 
-Map::removeElevationLayer(class ElevationLayer* layer) {
-    OE_DEPRECATED(Map::removeElevationLayer, Map::removeLayer);
-    removeLayer(layer);
-}
-
-void 
-Map::moveElevationLayer(class ElevationLayer* layer, unsigned newIndex) {
-    OE_DEPRECATED(Map::moveElevationLayer, Map::moveLayer);
-    moveLayer(layer, newIndex);
-}
-
-void 
-Map::addModelLayer(class ModelLayer* layer) {
-    OE_DEPRECATED(Map::addModelLayer, Map::addLayer);
-    addLayer(layer);
-}
-
-void 
-Map::insertModelLayer(class ModelLayer* layer, unsigned index) {
-    OE_DEPRECATED(Map::insertModelLayer, Map::insertLayer);
-    insertLayer(layer, index);
-}
-
-void 
-Map::removeModelLayer(class ModelLayer* layer) {
-    OE_DEPRECATED(Map::removeModelLayer, Map::removeLayer);
-    removeLayer(layer);
-}
-
-void 
-Map::moveModelLayer(class ModelLayer* layer, unsigned newIndex) {
-    OE_DEPRECATED(Map::moveModelLayer, Map::moveLayer);
-    moveLayer(layer, newIndex);
-}
-
-void 
-Map::addTerrainMaskLayer(class MaskLayer* layer) {
-    OE_DEPRECATED(Map::addTerrainMaskLayer, Map::addLayer);
-    addLayer(layer);
-}
-
-void 
-Map::removeTerrainMaskLayer(class MaskLayer* layer) {
-    OE_DEPRECATED(Map::removeTerrainMaskLayer, Map::removeLayer);
-    removeLayer(layer);
+    return true;
 }
