@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
-* Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+* Copyright 2018 Pelican Mapping
 * http://osgearth.org
 *
 * osgEarth is free software; you can redistribute it and/or modify
@@ -20,32 +20,16 @@
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 */
 #include <osgEarth/MapNode>
-#include <osgEarth/Capabilities>
-#include <osgEarth/ClampableNode>
 #include <osgEarth/ClampingTechnique>
 #include <osgEarth/CullingUtils>
-#include <osgEarth/DrapeableNode>
 #include <osgEarth/DrapingTechnique>
-#include <osgEarth/MapNodeObserver>
-#include <osgEarth/MaskNode>
 #include <osgEarth/NodeUtils>
 #include <osgEarth/Registry>
-#include <osgEarth/ShaderFactory>
-#include <osgEarth/TraversalData>
-#include <osgEarth/VirtualProgram>
-#include <osgEarth/OverlayDecorator>
-#include <osgEarth/TerrainEngineNode>
-#include <osgEarth/TerrainResources>
-#include <osgEarth/ShaderGenerator>
-#include <osgEarth/SpatialReference>
-#include <osgEarth/MapModelChange>
-#include <osgEarth/Lighting>
 #include <osgEarth/ResourceReleaser>
-#include <osgEarth/URI>
-#include <osg/ArgumentParser>
-#include <osg/PagedLOD>
+#include <osgEarth/Lighting>
+#include <osgEarth/GLUtils>
+#include <osgEarth/HorizonClipPlane>
 #include <osgUtil/Optimizer>
-#include <typeinfo>
 
 using namespace osgEarth;
 
@@ -67,17 +51,18 @@ namespace
 
         void onLayerAdded(Layer* layer, unsigned index) {
             _node->onLayerAdded(layer, index);
-            // for backwards compat until we refactor ModelLayer to use Layer::getNode
-            MapCallback::onLayerAdded(layer, index);
         }
         void onLayerRemoved(Layer* layer, unsigned index) {
             _node->onLayerRemoved(layer, index);
-            // for backwards compat until we refactor ModelLayer to use Layer::getNode
-            MapCallback::onLayerRemoved(layer, index);
         }
         void onLayerMoved(Layer* layer, unsigned oldIndex, unsigned newIndex) {
             _node->onLayerMoved(layer, oldIndex, newIndex);
-            MapCallback::onLayerMoved(layer, oldIndex, newIndex);
+        }
+        void onLayerEnabled(Layer* layer) {
+            _node->onLayerAdded(layer, _node->getMap()->getIndexOfLayer(layer));
+        }
+        void onLayerDisabled(Layer* layer) {
+            _node->onLayerRemoved(layer, _node->getMap()->getIndexOfLayer(layer));
         }
 
         osg::observer_ptr<MapNode> _node;
@@ -89,7 +74,7 @@ namespace
     {
         MapNodeObserverInstaller( MapNode* mapNode ) : _mapNode( mapNode ) { }
 
-        virtual void onPostMergeNode(osg::Node* node)
+        virtual void onPostMergeNode(osg::Node* node, osg::Object* sender)
         {
             if ( _mapNode.valid() && node )
             {
@@ -101,6 +86,30 @@ namespace
         osg::observer_ptr<MapNode> _mapNode;
     };
 
+    // proxy cull callback for layers that have their own cull callback
+    struct LayerCullCallbackDispatch : public osg::NodeCallback
+    {
+        Layer* _layer;
+
+        LayerCullCallbackDispatch(Layer* layer) : _layer(layer) { }
+
+        virtual void operator()(osg::Node* node, osg::NodeVisitor* nv)
+        {
+            if (_layer->getNode())
+            {
+                _layer->apply(_layer->getNode(), nv);
+            }
+            //if (_layer->getCullCallback() && _layer->getNode())
+            //{
+            //    _layer->apply(_layer->getNode(), nv);
+            //}
+            else
+            {
+                traverse(node, nv);
+            }
+        };
+    };
+
     typedef std::vector< osg::ref_ptr<Extension> > Extensions;
 }
 
@@ -307,10 +316,8 @@ MapNode::init()
     // initialize terrain-level lighting:
     if ( terrainOptions.enableLighting().isSet() )
     {
-        _terrainEngineContainer->getOrCreateStateSet()->setDefine(OE_LIGHTING_DEFINE, terrainOptions.enableLighting().get());
-
-        _terrainEngineContainer->getOrCreateStateSet()->setMode(
-            GL_LIGHTING,
+        GLUtils::setLighting(
+            _terrainEngineContainer->getOrCreateStateSet(),
             terrainOptions.enableLighting().value() ? 1 : 0 );
     }
 
@@ -366,36 +373,15 @@ MapNode::init()
 
     if ( _mapNodeOptions.enableLighting().isSet() )
     {
-        stateset->setDefine(OE_LIGHTING_DEFINE, terrainOptions.enableLighting().get());
-
-        stateset->setMode(
-            GL_LIGHTING,
+        GLUtils::setLighting(
+            stateset,
             _mapNodeOptions.enableLighting().value() ? 1 : 0);
     }
 
     // Add in some global uniforms
-    stateset->addUniform( new osg::Uniform("oe_isGeocentric", _map->isGeocentric()) );
     if ( _map->isGeocentric() )
     {
-        OE_INFO << LC << "Adding ellipsoid uniforms.\n";
-
-        // for a geocentric map, use an ellipsoid unit-frame transform and its inverse:
-        if (_map->getSRS() != NULL && _map->getSRS()->getEllipsoid() != NULL)
-        {
-            osg::Vec3d ellipFrameInverse(
-                _map->getSRS()->getEllipsoid()->getRadiusEquator(),
-                _map->getSRS()->getEllipsoid()->getRadiusEquator(),
-                _map->getSRS()->getEllipsoid()->getRadiusPolar());
-            stateset->addUniform( new osg::Uniform("oe_ellipsoidFrameInverse", osg::Vec3f(ellipFrameInverse)) );
-
-            osg::Vec3d ellipFrame = osg::componentDivide(osg::Vec3d(1.0,1.0,1.0), ellipFrameInverse);
-            stateset->addUniform( new osg::Uniform("oe_ellipsoidFrame", osg::Vec3f(ellipFrame)) );
-        }
-        else
-        {
-            stateset->addUniform( new osg::Uniform("oe_ellipsoidFrameInverse", osg::Vec3f()) );
-            stateset->addUniform( new osg::Uniform("oe_ellipsoidFrame", osg::Vec3f()) );
-        }
+        stateset->setDefine("OE_IS_GEOCENTRIC");
     }
 
     // install a default material for everything in the map
@@ -410,6 +396,12 @@ MapNode::init()
     // install a callback that sets the viewport size uniform:
     this->addCullCallback(new InstallViewportSizeUniform());
 
+    // install a callback that updates a horizon object and installs a clipping plane
+    if (getMapSRS()->isGeographic())
+    {
+        this->addCullCallback(new HorizonClipPlane(getMapSRS()->getEllipsoid()));
+    }
+
     // register for event traversals so we can deal with blacklisted filenames
     ADJUST_EVENT_TRAV_COUNT( this, 1 );
 
@@ -419,18 +411,11 @@ MapNode::init()
 
 MapNode::~MapNode()
 {
+    // Remove this node's map callback first:
     _map->removeMapCallback( _mapCallback.get() );
 
+    // Then invoke "removed" on all the layers in a batch.
     _mapCallback->invokeOnLayerRemoved(_map.get());
-    //ModelLayerVector modelLayers;
-    //_map->getLayers( modelLayers );
-    ////Remove our model callback from any of the model layers in the map
-    //for (osgEarth::ModelLayerVector::iterator itr = modelLayers.begin(); itr != modelLayers.end(); ++itr)
-    //{
-    //    this->onModelLayerRemoved( itr->get() );
-    //}
-
-    _map->clear();
 
     this->clearExtensions();
 
@@ -447,8 +432,6 @@ MapNode::getConfig() const
     Config mapConf("map");
     mapConf.set("version", "2");
 
-    MapFrame mapf( _map.get() );
-
     // the map and node options:
     Config optionsConf = _map->getInitialMapOptions().getConfig();
     optionsConf.merge( getMapNodeOptions().getConfig() );
@@ -456,7 +439,7 @@ MapNode::getConfig() const
 
     // the layers
     LayerVector layers;
-    mapf.getLayers(layers);
+    _map->getLayers(layers);
 
     for (LayerVector::const_iterator i = layers.begin(); i != layers.end(); ++i)
     {
@@ -474,10 +457,8 @@ MapNode::getConfig() const
     {
         Extension* e = i->get();
         Config conf = e->getConfigOptions().getConfig();
-        if ( !conf.key().empty() )
-        {
-            mapConf.add( conf );
-        }
+        conf.key() = e->getConfigKey();
+        mapConf.add( conf );
     }
 
     Config ext = externalConfig();
@@ -615,7 +596,7 @@ MapNode::getLayerNodeGroup() const
 osg::Node*
 MapNode::getLayerNode(Layer* layer) const
 {
-    return layer ? layer->getOrCreateNode() : 0L;
+    return layer ? layer->getNode() : 0L;
 }
 
 
@@ -648,14 +629,18 @@ namespace
         for (LayerVector::iterator i = layers.begin(); i != layers.end(); ++i)
         {
             Layer* layer = i->get();
-            osg::Node* node = layer->getOrCreateNode();
-            if (node)
+            if (layer->getEnabled())
             {
-                osg::Group* container = new osg::Group();
-                container->setName(layer->getName());
-                container->addChild(node);
-                container->setStateSet(layer->getStateSet());
-                layerNodes->addChild(container);
+                osg::Node* node = layer->getNode();
+                if (node)
+                {
+                    osg::Group* container = new osg::Group();
+                    container->setName(layer->getName());
+                    container->addChild(node);
+                    container->setStateSet(layer->getOrCreateStateSet());
+                    container->setCullCallback(new LayerCullCallbackDispatch(layer));
+                    layerNodes->addChild(container);
+                }
             }
         }
     }
@@ -670,90 +655,59 @@ MapNode::onLayerAdded(Layer* layer, unsigned index)
     // Communicate terrain resources to the layer:
     layer->setTerrainResources(getTerrainEngine()->getResources());
 
-    // Compatibility, until we refactor things.
-    ModelLayer* modelLayer = dynamic_cast<ModelLayer*>(layer);
-    if (modelLayer)
-    {
-        // TODO:  Why go through all the MapNodeObserver stuff when we can just pass in the MapNode here?
-        modelLayer->getOrCreateSceneGraph(_map.get(), _map->getReadOptions(), 0L);
-        // Install the MapNodeObserverInstaller so that MapNodeObservers will be notified of the MapNode.
-        modelLayer->getSceneGraphCallbacks()->add(new MapNodeObserverInstaller(this));
-    }
+    // Each layer gets a callback to change the MapNode if necessary
+    layer->getSceneGraphCallbacks()->add(new MapNodeObserverInstaller(this));
 
     // Create the layer's node, if it has one:
-    osg::Node* node = layer->getOrCreateNode();
+    osg::Node* node = layer->getNode();
     if (node)
     {
-        // Call setMapNode on any MapNodeObservers on this initial creation.
-        MapNodeReplacer replacer( this );
-        //node->accept( replacer );
-
-        rebuildLayerNodes(_map.get(), _layerNodes);
-
         OE_DEBUG << LC << "Adding node from layer \"" << layer->getName() << "\" to the scene graph\n";
 
-        // TODO: move this logic into ModelLayer.
-        if (modelLayer)
-        {
-            ModelSource* ms = modelLayer->getModelSource();
-            if (ms)
-            {
-                // enfore a rendering bin if necessary:
-                if (ms->getOptions().renderOrder().isSet())
-                {
-                    osg::StateSet* mss = node->getOrCreateStateSet();
-                    mss->setRenderBinDetails(
-                        ms->getOptions().renderOrder().value(),
-                        mss->getBinName().empty() ? "DepthSortedBin" : mss->getBinName());
-                }
-                if (ms->getOptions().renderBin().isSet())
-                {
-                    osg::StateSet* mss = node->getOrCreateStateSet();
-                    mss->setRenderBinDetails(
-                        mss->getBinNumber(),
-                        ms->getOptions().renderBin().get());
-                }
-            }
-        }
+        // notify before adding it to the graph:
+        layer->getSceneGraphCallbacks()->firePreMergeNode(node);
+
+        // update the layer-to-node table (adds the node to the graph)
+        rebuildLayerNodes(_map.get(), _layerNodes);
+
+        // after putting it in the graph:
+        layer->getSceneGraphCallbacks()->firePostMergeNode(node);
     }
 }
 
 void
 MapNode::onLayerRemoved(Layer* layer, unsigned index)
 {
-    if (layer && layer->getOrCreateNode())
+    if (layer)
     {
-        rebuildLayerNodes(_map.get(), _layerNodes);
+        osg::Node* node = layer->getNode();
+        if (node)
+        {
+            layer->getSceneGraphCallbacks()->fireRemoveNode(node);
+            rebuildLayerNodes(_map.get(), _layerNodes);
+        }
     }
 }
 
 void
 MapNode::onLayerMoved(Layer* layer, unsigned oldIndex, unsigned newIndex)
 {
-    if (layer && layer->getOrCreateNode())
+    if (layer && layer->getNode())
     {
         rebuildLayerNodes(_map.get(), _layerNodes);
     }
 }
 
-namespace
+void
+MapNode::openMapLayers()
 {
-    struct MaskNodeFinder : public osg::NodeVisitor {
-        MaskNodeFinder() : osg::NodeVisitor( osg::NodeVisitor::TRAVERSE_ALL_CHILDREN ) { }
-        void apply( osg::Group& group ) {
-            if ( dynamic_cast<MaskNode*>( &group ) ) {
-                _groups.push_back( &group );
-            }
-            traverse(group);
-        }
-        std::list< osg::Group* > _groups;
-    };
-}
+    LayerVector layers;
+    _map->getLayers(layers);
 
-namespace
-{
-    template<typename T> void tryOpenLayer(T* layer)
+    for (LayerVector::const_iterator i = layers.begin(); i != layers.end(); ++i)
     {
+        Layer* layer = i->get();
+
         if (!layer->getStatus().isError())
         {
             const Status& status = layer->open();
@@ -765,19 +719,6 @@ namespace
     }
 }
 
-void
-MapNode::openMapLayers()
-{
-    MapFrame frame(_map.get());
-
-    for (LayerVector::const_iterator i = frame.layers().begin();
-        i != frame.layers().end();
-        ++i)
-    {
-        tryOpenLayer(i->get());
-    }
-}
-
 void
 MapNode::traverse( osg::NodeVisitor& nv )
 {
@@ -812,9 +753,7 @@ MapNode::resizeGLObjectBuffers(unsigned maxSize)
     getMap()->getLayers(layers);
     for (LayerVector::const_iterator i = layers.begin(); i != layers.end(); ++i)
     {
-        if ((*i)->getStateSet()) {
-            (*i)->getStateSet()->resizeGLObjectBuffers(maxSize);
-        }
+        i->get()->resizeGLObjectBuffers(maxSize);
     }
 }
 
@@ -827,10 +766,16 @@ MapNode::releaseGLObjects(osg::State* state) const
     getMap()->getLayers(layers);
     for (LayerVector::const_iterator i = layers.begin(); i != layers.end(); ++i)
     {
-        if ((*i)->getStateSet()) {
-            (*i)->getStateSet()->releaseGLObjects(state);
-        }
+        i->get()->releaseGLObjects(state);
     }
+
+    // osg::Node doesn't release nested callbacks, oops
+    for(const osg::Callback* cc = getCullCallback(); cc; cc = cc->getNestedCallback())
+        cc->releaseGLObjects(state);
+    for(const osg::Callback* uc = getUpdateCallback(); uc; uc = uc->getNestedCallback())
+        uc->releaseGLObjects(state);
+    for(const osg::Callback* ec = getEventCallback(); ec; ec = ec->getNestedCallback())
+        ec->releaseGLObjects(state);
 }
 
 DrapingManager*
