@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
- * Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+ * Copyright 2018 Pelican Mapping
  * http://osgearth.org
  *
  * osgEarth is free software; you can redistribute it and/or modify
@@ -36,7 +36,6 @@ _options(optionsPtr ? optionsPtr : &_optionsConcrete)
 void
 PatchLayer::init()
 {
-    Layer::init();
-    
-    setRenderType(RENDERTYPE_PATCH);
+    Layer::init();    
+    setRenderType(RENDERTYPE_TERRAIN_PATCH);
 }
