@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
- * Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+ * Copyright 2018 Pelican Mapping
  * http://osgearth.org
  *
  * osgEarth is free software; you can redistribute it and/or modify
@@ -18,16 +18,10 @@
  */
 #include <osgEarth/ShaderUtils>
 #include <osgEarth/ShaderFactory>
-#include <osgEarth/VirtualProgram>
 #include <osgEarth/Registry>
 #include <osgEarth/Capabilities>
 #include <osgEarth/CullingUtils>
-#include <osgEarth/URI>
 #include <osgEarth/GLSLChunker>
-#include <osg/ComputeBoundsVisitor>
-#include <osg/LightSource>
-#include <osgDB/FileUtils>
-#include <list>
 
 using namespace osgEarth;
 
@@ -43,68 +37,6 @@ namespace
 
 
     typedef std::list<const osg::StateSet*> StateSetStack;
-
-#if 0
-
-    static osg::StateAttribute::GLModeValue 
-    getModeValue(const StateSetStack& statesetStack, osg::StateAttribute::GLMode mode)
-    {
-        osg::StateAttribute::GLModeValue base_val = osg::StateAttribute::ON;
-
-        for(StateSetStack::const_iterator itr = statesetStack.begin();
-            itr != statesetStack.end();
-            ++itr)
-        {
-            osg::StateAttribute::GLModeValue val = (*itr)->getMode(mode);
-
-            if ( (val & osg::StateAttribute::INHERIT) == 0 )
-            {
-
-                if ((val & osg::StateAttribute::PROTECTED)!=0 ||
-                    (base_val & osg::StateAttribute::OVERRIDE)==0)
-                {
-                    base_val = val;
-                }
-            }
-        }
-        return base_val;
-    }
-#endif
-    
-#if 0
-    static const osg::Light*
-    getLightByID(const StateSetStack& statesetStack, int id)
-    {
-        const osg::Light* base_light = NULL;
-        osg::StateAttribute::GLModeValue base_val = osg::StateAttribute::ON;
-        
-        for(StateSetStack::const_iterator itr = statesetStack.begin();
-            itr != statesetStack.end();
-            ++itr)
-        {
-            
-            osg::StateAttribute::GLModeValue val = (*itr)->getMode(GL_LIGHT0+id);
-
-            //if ( (val & osg::StateAttribute::INHERIT) == 0 )
-            {
-            //    if ((val & osg::StateAttribute::PROTECTED)!=0 ||
-            //        (base_val & osg::StateAttribute::OVERRIDE)==0)
-                {
-                    base_val = val;
-                    const osg::StateAttribute* lightAtt = (*itr)->getAttribute(osg::StateAttribute::LIGHT, id);
-                    if(lightAtt){
-                        const osg::Light* asLight = dynamic_cast<const osg::Light*>(lightAtt);
-                        if(val){
-                            base_light = asLight;
-                        }
-                    }
-                }
-            }
-            
-        }
-        return base_light;
-    }
-#endif
     
     static const osg::Material*
     getFrontMaterial(const StateSetStack& statesetStack)
@@ -296,6 +228,7 @@ namespace
     }
 }
 
+#if 0
 void
 ShaderPreProcessor::applySupportForNoFFP(osg::Shader* shader)
 {
@@ -316,6 +249,7 @@ ShaderPreProcessor::applySupportForNoFFP(osg::Shader* shader)
 
 #endif // !defined(OSG_GL_FIXED_FUNCTION_AVAILABLE)
 }
+#endif
 
 void
 ShaderPreProcessor::run(osg::Shader* shader)
@@ -338,7 +272,11 @@ ShaderPreProcessor::run(osg::Shader* shader)
         GLSLChunker::Chunks chunks;
         chunker.read( source, chunks );
 
-        applySupportForNoFFPImpl(chunks);
+
+        if (shader->getType() != osg::Shader::FRAGMENT)
+        {
+            //applySupportForNoFFPImpl(chunks);
+        }
 
         // Replace varyings with directives that the ShaderFactory can interpret
         // when creating interface blocks.
@@ -604,6 +542,8 @@ DiscardAlphaFragments::install(osg::StateSet* ss, float minAlpha) const
         VirtualProgram* vp = VirtualProgram::getOrCreate(ss);
         if ( vp )
         {
+            vp->setName("Discard Alpha");
+
             std::string code = Stringify()
                 << "#version " << GLSL_VERSION_STR << "\n"
                 << GLSL_DEFAULT_PRECISION_FLOAT << "\n"
