@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
-* Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+* Copyright 2018 Pelican Mapping
 * http://osgearth.org
 *
 * osgEarth is free software; you can redistribute it and/or modify
@@ -20,7 +20,6 @@
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 */
 #include <osgEarth/Shadowing>
-#include <osg/ValueObject>
 
 #define LC "[Shadowing] "
 
@@ -30,7 +29,9 @@ using namespace osgEarth;
 void
 Shadowing::setIsShadowCamera(osg::Camera* camera)
 {
-    camera->getOrCreateStateSet()->setDefine("OE_IS_SHADOW_CAMERA");
+    osg::StateSet* ss = camera->getOrCreateStateSet();
+    ss->setDefine("OE_IS_SHADOW_CAMERA");
+    ss->setDefine("OE_IS_DEPTH_CAMERA");
 }
 
 bool
