@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
- * Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+ * Copyright 2018 Pelican Mapping
  * http://osgearth.org
  *
  * osgEarth is free software; you can redistribute it and/or modify
@@ -19,7 +19,6 @@
 
 #include <osgEarth/SimplexNoise>
 #include <osgEarth/ImageUtils>
-#include <osg/Image>
 #include <algorithm>
 
 #define POW2(x) ((double)(x==0 ? 1 : (2 << (x-1))))
@@ -163,7 +162,7 @@ double SimplexNoise::getTiledValue(double x, double y) const
 {
     const double TwoPI = 2.0 * osg::PI;
     double freq = _freq;
-    double o = std::max(1u, _octaves);
+    double o = osg::maximum(1u, _octaves);
     double amp = 1.0;
     double maxamp = 0.0;
     double n = 0.0;
@@ -196,7 +195,7 @@ double SimplexNoise::getTiledValueWithTurbulence(double x, double y, double F) c
 {
     const double TwoPI = 2.0 * osg::PI;
     double freq = _freq;
-    double o = std::max(1u, _octaves);
+    double o = osg::maximum(1u, _octaves);
     double amp = 1.0;
     double maxamp = 0.0;
     double n = 0.0;
@@ -231,7 +230,7 @@ double SimplexNoise::getTiledValueWithTurbulence(double x, double y, double F) c
 double SimplexNoise::getValue(double xin, double yin) const
 {
     double freq = _freq;
-    double o = std::max(1u, _octaves);
+    double o = osg::maximum(1u, _octaves);
     double amp = 1.0;
     double maxamp = 0.0;
     double n = 0.0;
@@ -254,7 +253,7 @@ double SimplexNoise::getValue(double xin, double yin) const
 double SimplexNoise::getValue(double xin, double yin, double zin) const
 {
     double freq = _freq;
-    double o = std::max(1u, _octaves);
+    double o = osg::maximum(1u, _octaves);
     double amp = 1.0;
     double maxamp = 0.0;
     double n = 0.0;
@@ -278,7 +277,7 @@ double SimplexNoise::getValue(double xin, double yin, double zin) const
 double SimplexNoise::getValue(double xin, double yin, double zin, double win) const
 {
     double freq = _freq;
-    double o = std::max(1u, _octaves);
+    double o = osg::maximum(1u, _octaves);
     double amp = 1.0;
     double maxamp = 0.0;
     double n = 0.0;
@@ -681,8 +680,8 @@ SimplexNoise::createSeamlessImage(unsigned dim) const
         {
             double v = (double)t / (double)dim;
             value.r() = noise.getTiledValue(u, v);
-            minN = std::min(minN, value.r());
-            maxN = std::max(maxN, value.r());
+            minN = osg::minimum(minN, value.r());
+            maxN = osg::maximum(maxN, value.r());
             write(value, s, t);
         }        
     }
