@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
- * Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+ * Copyright 2018 Pelican Mapping
  * http://osgearth.org
  *
  * osgEarth is free software; you can redistribute it and/or modify
@@ -18,7 +18,6 @@
  */
 #include <osgEarth/StateSetCache>
 #include <osg/NodeVisitor>
-#include <osg/Geode>
 #include <osg/BufferIndexBinding>
 
 #define LC "[StateSetCache] "
@@ -120,27 +119,6 @@ namespace
             traverse(node);
         }
 
-        void apply(osg::Geode& geode)
-        {
-            unsigned numDrawables = geode.getNumDrawables();
-            for( unsigned i=0; i<numDrawables; ++i )
-            {
-                osg::Drawable* d = geode.getDrawable(i);
-                if (d && d->getStateSet() )
-                {
-                    // ref for thread-safety; another thread might replace this stateset
-                    // during optimization while we're looking at it.
-                    osg::ref_ptr<osg::StateSet> stateset = d->getStateSet();
-                    if (stateset.valid() &&
-                        stateset->getDataVariance() != osg::Object::DYNAMIC)
-                    {
-                        applyStateSet( stateset.get() );
-                    }
-                }
-            }
-            apply((osg::Node&)geode);
-        }
-
         // assume: stateSet is safely referenced by caller
         void applyStateSet(osg::StateSet* stateSet)
         {
@@ -214,31 +192,6 @@ namespace
             }
             traverse(node);
         }
-
-        void apply(osg::Geode& geode)
-        {
-            unsigned numDrawables = geode.getNumDrawables();
-            for( unsigned i=0; i<numDrawables; ++i )
-            {
-                osg::Drawable* d = geode.getDrawable(i);
-                if ( d && d->getStateSet())
-                {
-                    osg::ref_ptr<osg::StateSet> stateset = d->getStateSet();
-                    if (stateset.valid() && isEligible(stateset.get()))
-                    {
-                        _stateSets++;
-                        osg::ref_ptr<osg::StateSet> shared;
-                        if ( _cache->share(stateset, shared) )
-                        {
-                            d->setStateSet( shared.get() );
-                            _shares++;
-                        }
-                        //else _misses.push_back(in.get());
-                    }
-                }
-            }
-            apply((osg::Node&)geode);
-        }
     };
 }
 
@@ -261,6 +214,17 @@ StateSetCache::~StateSetCache()
     prune();
 }
 
+void
+StateSetCache::releaseGLObjects(osg::State* state) const
+{
+    Threading::ScopedMutexLock lock( _mutex );
+    for(StateSetSet::const_iterator i = _stateSetCache.begin(); i != _stateSetCache.end(); ++i)
+    {
+        i->get()->releaseGLObjects(state);
+    }
+    
+}
+
 void
 StateSetCache::setMaxSize(unsigned value)
 {
