@@ -1,5 +1,5 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
+/* osgEarth - Geospatial SDK for OpenSceneGraph
 * Copyright 2008-2012 Pelican Mapping
 * http://osgearth.org
 *
@@ -28,7 +28,7 @@
 #include <osgDB/InputStream>
 #include <osgDB/OutputStream>
 
-namespace
+namespace osgEarth { namespace Serializers { namespace TextureBuffer
 {
     REGISTER_OBJECT_WRAPPER( TextureBuffer,
                              new osg::TextureBuffer,
@@ -38,7 +38,7 @@ namespace
         ADD_IMAGE_SERIALIZER( Image, osg::Image, NULL );  // _image
         ADD_INT_SERIALIZER( TextureWidth, 0 );  // _textureWidth
     }
-}
+} } }
 
 #elif OSG_VERSION_LESS_THAN(3,5,6)
 
@@ -47,13 +47,16 @@ namespace
 #include <osgDB/InputStream>
 #include <osgDB/OutputStream>
 
-REGISTER_OBJECT_WRAPPER( TextureBuffer,
-                         new osg::TextureBuffer,
-                         osg::TextureBuffer,
-                         "osg::Object osg::StateAttribute osg::Texture osg::TextureBuffer" )
+namespace osgEarth { namespace Serializers { namespace TextureBuffer
 {
-    ADD_INT_SERIALIZER( TextureWidth, 0 );                       // _textureWidth
-    ADD_OBJECT_SERIALIZER( BufferData, osg::BufferData, NULL );  // _bufferData
-}
+    REGISTER_OBJECT_WRAPPER( TextureBuffer,
+                            new osg::TextureBuffer,
+                            osg::TextureBuffer,
+                            "osg::Object osg::StateAttribute osg::Texture osg::TextureBuffer" )
+    {
+        ADD_INT_SERIALIZER( TextureWidth, 0 );                       // _textureWidth
+        ADD_OBJECT_SERIALIZER( BufferData, osg::BufferData, NULL );  // _bufferData
+    }
+} } }
 
 #endif
