@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
-* Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+* Copyright 2018 Pelican Mapping
 * http://osgearth.org
 *
 * osgEarth is free software; you can redistribute it and/or modify
@@ -27,7 +27,4 @@ using namespace osgEarth::Annotation;
 //---------------------------------------------------------------------------
 
 // static defaults
-bool AnnotationSettings::_continuousClamping = true;
-bool AnnotationSettings::_autoDepthOffset = true;
 double AnnotationSettings::_occlusionCullingMaxAltitude = 200000.0;
-double AnnotationSettings::_occlusionCullingHeightAdjustment = 5.0;
