@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
-* Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+* Copyright 2018 Pelican Mapping
 * http://osgearth.org
 *
 * osgEarth is free software; you can redistribute it and/or modify
@@ -28,6 +28,7 @@
 #include <osgEarth/CullingUtils>
 #include <osgEarth/MapNode>
 #include <osgEarth/TerrainEngineNode>
+#include <osgEarth/GLUtils>
 
 #include <osgText/Text>
 #include <osg/ComputeBoundsVisitor>
@@ -46,51 +47,27 @@ using namespace osgEarth::Annotation;
 #define DEFAULT_HORIZON_CULLING true
 
 GeoPositionNode::GeoPositionNode() :
-AnnotationNode(),
-_geoxform(0L),
-_paxform(0L),
-_occlusionCullingRequested( DEFAULT_OCCLUSION_CULLING ),
-_horizonCullingRequested( DEFAULT_HORIZON_CULLING )
+AnnotationNode()
 {
-    init();
+    construct();
 }
 
-GeoPositionNode::GeoPositionNode(MapNode* mapNode) :
-AnnotationNode(),
-_geoxform(0L),
-_paxform(0L),
-_occlusionCullingRequested( DEFAULT_OCCLUSION_CULLING ),
-_horizonCullingRequested( DEFAULT_HORIZON_CULLING )
+GeoPositionNode::GeoPositionNode(const Config& conf, const osgDB::Options* options) :
+AnnotationNode( conf, options )
 {
-    init();
-    GeoPositionNode::setMapNode( mapNode );
-}
-
-GeoPositionNode::GeoPositionNode(MapNode* mapNode, const GeoPoint& position ) :
-AnnotationNode(),
-_geoxform(0L),
-_paxform(0L),
-_occlusionCullingRequested( DEFAULT_OCCLUSION_CULLING ),
-_horizonCullingRequested( DEFAULT_HORIZON_CULLING )
-{
-    init();
-    GeoPositionNode::setMapNode( mapNode );
-    _geoxform->setPosition( position );
-}
-
-GeoPositionNode::GeoPositionNode(const GeoPositionNode& rhs, const osg::CopyOp& copy) :
-AnnotationNode(rhs, copy),
-_geoxform(0L),
-_paxform(0L),
-_occlusionCullingRequested( DEFAULT_OCCLUSION_CULLING ),
-_horizonCullingRequested( DEFAULT_HORIZON_CULLING )
-{
-    //nop - UNUSED
+    construct();
+    setConfig(conf);
 }
 
 void
-GeoPositionNode::init()
+GeoPositionNode::construct()
 {
+    _geoxform = 0L;
+    _paxform = 0L;
+
+    _occlusionCullingRequested = DEFAULT_OCCLUSION_CULLING;
+    _horizonCullingRequested = DEFAULT_HORIZON_CULLING;
+
     this->removeChildren(0, this->getNumChildren());
 
     _geoxform = new GeoTransform();
@@ -98,15 +75,12 @@ GeoPositionNode::init()
 
     _paxform = new osg::PositionAttitudeTransform();
     _geoxform->addChild( _paxform );
-    
-    this->getOrCreateStateSet()->setMode( GL_LIGHTING, 0 );
 }
 
 void
 GeoPositionNode::setMapNode( MapNode* mapNode )
 {
-    MapNode* oldMapNode = getMapNode();
-    if ( oldMapNode != mapNode )
+    if (mapNode != getMapNode())
     {
         AnnotationNode::setMapNode( mapNode );
 
@@ -173,28 +147,7 @@ GeoPositionNode::applyStyle(const Style& style)
 void
 GeoPositionNode::setPosition(const GeoPoint& pos)
 {
-    GeoPoint pos2 = pos;
-
-    // The altitude symbol, if there is one, overrides the incoming GeoPoint:
-    const AltitudeSymbol* alt = getStyle().get<AltitudeSymbol>();
-    if (alt)
-    {
-        if (alt->clamping() == alt->CLAMP_TO_TERRAIN)
-        {
-            pos2.z() = 0;
-            pos2.altitudeMode() = ALTMODE_RELATIVE;
-        }
-        else if (alt->clamping() == alt->CLAMP_RELATIVE_TO_TERRAIN)
-        {
-            pos2.altitudeMode() = ALTMODE_RELATIVE;
-        }
-        else if (alt->clamping() == alt->CLAMP_NONE)
-        {
-            pos2.altitudeMode() = ALTMODE_ABSOLUTE;
-        }
-    }
-    _geoxform->setPosition( pos2 );
-    dirty();
+    _geoxform->setPosition(pos);
 }
 
 bool
@@ -243,21 +196,11 @@ void GeoPositionNode::setOcclusionCullingMaxAltitude( double occlusionCullingMax
     }
 }
 
-
-GeoPositionNode::GeoPositionNode(MapNode* mapNode, const Config& conf) :
-AnnotationNode            ( conf ),
-_occlusionCullingRequested( DEFAULT_OCCLUSION_CULLING ),
-_horizonCullingRequested  ( DEFAULT_HORIZON_CULLING )
-{
-    init();
-    GeoPositionNode::setMapNode( mapNode );
-    setConfig(conf);
-}
-
 void
 GeoPositionNode::setConfig(const Config& conf)
 {
     //AnnotationNode::setConfig(conf);
+
     if (conf.hasValue("name"))
     {
         setName(conf.value("name"));
@@ -303,16 +246,16 @@ GeoPositionNode::getConfig() const
 {
     Config conf = AnnotationNode::getConfig();
 
-    conf.addObj( "position", getGeoTransform()->getPosition() );
+    conf.set( "position", getGeoTransform()->getPosition() );
 
     const osg::Vec3d& scale = getPositionAttitudeTransform()->getScale();
     if ( scale.x() != 1.0f || scale.y() != 1.0f || scale.z() != 1.0f )
     {
         Config c( "scale" );
-        c.add( "x", scale.x() );
-        c.add( "y", scale.y() );
-        c.add( "z", scale.z() );
-        conf.add( c );
+        c.set( "x", scale.x() );
+        c.set( "y", scale.y() );
+        c.set( "z", scale.z() );
+        conf.set( c );
     }
 
     const osg::Vec3d& offset = getPositionAttitudeTransform()->getPosition();
@@ -322,7 +265,7 @@ GeoPositionNode::getConfig() const
         c.set( "x", offset.x() );
         c.set( "y", offset.y() );
         c.set( "z", offset.z() );
-        conf.add( c );
+        conf.set( c );
     }
 
     const osg::Quat& rot = getPositionAttitudeTransform()->getAttitude();
@@ -333,7 +276,7 @@ GeoPositionNode::getConfig() const
         c.set( "y", rot.y() );
         c.set( "z", rot.z() );
         c.set( "w", rot.w() );
-        conf.add( c );
+        conf.set( c );
     }
 
     return conf;
