@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
- * Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+ * Copyright 2018 Pelican Mapping
  * http://osgearth.org
  *
  * osgEarth is free software; you can redistribute it and/or modify
@@ -18,7 +18,6 @@
  */
 #include "EarthFileSerializer"
 #include <osgEarth/FileUtils>
-#include <osgEarth/MapFrame>
 #include <osgEarth/Extension>
 #include <osgEarth/StringUtils>
 #include <osgEarth/FileUtils>
@@ -181,7 +180,7 @@ namespace
         {
             _rewriteAbsolutePaths = false;
             _newReferrerAbsPath = osgDB::convertFileNameToUnixStyle( osgDB::getRealPath(referrer) );
-            _newReferrerFolder  = osgDB::getFilePath( osgDB::findDataFile(_newReferrerAbsPath) );
+            _newReferrerFolder  = osgDB::getFilePath( _newReferrerAbsPath );
         }
 
         /** Whether to make absolute paths into relative paths if possible */
@@ -275,7 +274,7 @@ namespace
 
 			// Definition: an "element" is a part between slashes. Ex: "/a/b" has two elements ("a" and "b").
 			// Algorithm:
-			// 1. If paths are neither both absolute nor both relative, then we cannot do anything (we need to make them absolute, but need additionnal info on how to make it). Return.
+			// 1. If paths are neither both absolute nor both relative, then we cannot do anything (we need to make them absolute, but need additional info on how to make it). Return.
 			// 2. If both paths are absolute and root isn't the same (for Windows only, as roots are of the type "C:", "D:"), then the operation is impossible. Return.
 			// 3. Iterate over two paths elements until elements are equal
 			// 4. For each remaining element in "from", add ".." to result
@@ -333,8 +332,7 @@ namespace
 
     bool addLayer(const Config& conf, Map* map)
     {
-        std::string name = conf.key();
-        Layer* layer = Layer::create(name, conf);
+        Layer* layer = Layer::create(conf);
         if (layer)
         {
             map->addLayer(layer);
@@ -438,23 +436,6 @@ EarthFileSerializer2::deserialize( const Config& conf, const std::string& referr
             // nop - handled earlier
         }
 
-#if 0
-        else if ( i->key() == "image" )
-        {
-            addImageLayer( *i, map );
-        }
-
-        else */if ( i->key() == "model" )
-        {
-            addModelLayer( *i, map );
-        }
-
-        else if ( i->key() == "mask" )
-        {
-            addMaskLayer( *i, map );
-        }
-#endif
-
         else if ( i->key() == "external" || i->key() == "extensions" )
         {
             externalConfig = *i;
@@ -464,7 +445,6 @@ EarthFileSerializer2::deserialize( const Config& conf, const std::string& referr
                 Extension* extension = loadExtension(*e);
                 if (extension)
                     extensions.push_back(extension);
-                //addExtension( *e, mapNode.get() );
             }
         }
 
