@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
- * Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+ * Copyright 2018 Pelican Mapping
  * http://osgearth.org
  *
  * osgEarth is free software; you can redistribute it and/or modify
@@ -37,13 +37,8 @@ using namespace osgEarth;
 #define Q2(x) #x
 #define Q(x)  Q2(x)
 
-#if (defined(_DEBUG) || defined(QT_DEBUG)) && defined(OSGEARTH_DEBUG_POSTFIX)
-#   define LIBNAME_UTIL_POSTFIX Q(OSGEARTH_DEBUG_POSTFIX)
-#elif defined(OSGEARTH_RELEASE_POSTFIX)
-#   define LIBNAME_UTIL_POSTFIX Q(OSGEARTH_RELEASE_POSTFIX)
-#else
-#   define LIBNAME_UTIL_POSTFIX ""
-#endif
+#define LIBNAME_UTIL_POSTFIX Q(OSGEARTH_LIBRARY_POSTFIX)
+
 
 #if defined(WIN32)
 #   define LIBNAME_UTIL "osgEarthUtil"
