@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
-* Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+* Copyright 2018 Pelican Mapping
 * http://osgearth.org
 *
 * osgEarth is free software; you can redistribute it and/or modify
@@ -81,7 +81,7 @@ HeightFieldCache::getOrCreateHeightField(const MapFrame&                 frame,
         // Elevation "smoothing" uses the parent HF as the starting point for building
         // a new tile. This will cause lower-resolution data to propagate down the tree
         // and fill in any gaps in higher-resolution data. The result will be an elevation
-        // grid that is "smoother" but not neccessarily as accurate.
+        // grid that is "smoother" but not necessarily as accurate.
         if ( _useParentAsReferenceHF && parent_hf && parentKey.valid() )
         {
             out_hf = HeightFieldUtils::createSubSample(
@@ -106,6 +106,16 @@ HeightFieldCache::getOrCreateHeightField(const MapFrame&                 frame,
             true, // convertToHAE
             progress );
 
+        // Check for cancelation before writing to a cache
+        if (progress && progress->isCanceled())
+        {
+            if (out_hf.valid())
+            {
+                OE_DEBUG << LC << "Cancelation with a valid HF; this would cache bad data." << std::endl;
+            }
+            return false;
+        }
+
         // If the map failed to provide any suitable data sources at all, replace the
         // heightfield with data from its parent (if available). 
         if ( !populated )
