@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
-* Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+* Copyright 2018 Pelican Mapping
 * http://osgearth.org
 *
 * osgEarth is free software; you can redistribute it and/or modify
@@ -145,7 +145,7 @@ MPTerrainEngineNode::registerEngine(MPTerrainEngineNode* engineNode)
 }
 
 // since this method is called in a database pager thread, we use a ref_ptr output
-// parameter to avoid the engine node being destructed between the time we 
+// parameter to avoid the engine node being destructed between the time we
 // return it and the time it's accessed; this could happen if the user removed the
 // MapNode from the scene during paging.
 void
@@ -312,7 +312,7 @@ MPTerrainEngineNode::includeShaderLibrary(VirtualProgram* vp)
         osg::Shader* VS = new osg::Shader(osg::Shader::VERTEX, libVS);
         VS->setName( "oe_terrain_SDK_mp_VS" );
         vp->setShader( VS );
-        
+
         osg::Shader* FS = new osg::Shader(osg::Shader::FRAGMENT, libFS);
         FS->setName( "oe_terrain_SDK_mp_FS" );
         vp->setShader( FS );
@@ -343,7 +343,7 @@ MPTerrainEngineNode::setMap(const Map* map, const TerrainOptions& options)
     // live tiles of the current map revision so they can inrementally update
     // themselves if necessary.
     _liveTiles = new TileNodeRegistry("live", this->getTerrain());
-    _liveTiles->setRevisioningEnabled( _terrainOptions.incrementalUpdate() == true );
+    _liveTiles->setRevisioningEnabled(false);
     _liveTiles->setMapRevision( _update_mapf->getRevision() );
 
     // Facility to properly release GL objects
@@ -362,7 +362,7 @@ MPTerrainEngineNode::setMap(const Map* map, const TerrainOptions& options)
     {
         getResources()->reserveTextureImageUnit( _secondaryUnit, "MP Engine Secondary" );
     }
-    
+
     // initialize the model factory:
     _tileModelFactory = new TileModelFactory(_liveTiles.get(), _terrainOptions, this);
 
@@ -409,20 +409,18 @@ MPTerrainEngineNode::setMap(const Map* map, const TerrainOptions& options)
     dirtyBound();
 
     OE_INFO << LC << "Edge normalization is " << (_terrainOptions.normalizeEdges() == true? "ON" : "OFF") << std::endl;
+
+    if (_terrainOptions.enableMercatorFastPath().isSetTo(true))        
+    {
+        OE_NOTICE << LC << "Mercator fast path is enabled - warning: it is incompatible with caching" << std::endl;
+    }
 }
 
 
 osg::BoundingSphere
 MPTerrainEngineNode::computeBound() const
 {
-    //if ( _terrain && _terrain->getNumChildren() > 0 )
-    //{
-    //    return _terrain->getBound();
-    //}
-    //else
-    {
-        return TerrainEngineNode::computeBound();
-    }
+    return TerrainEngineNode::computeBound();
 }
 
 void
@@ -430,16 +428,7 @@ MPTerrainEngineNode::invalidateRegion(const GeoExtent& extent,
                                       unsigned         minLevel,
                                       unsigned         maxLevel)
 {
-    if (_terrainOptions.incrementalUpdate() == true && _liveTiles.valid())
-    {
-        GeoExtent extentLocal = extent;
-        if ( !extent.getSRS()->isEquivalentTo(this->getMap()->getSRS()) )
-        {
-            extent.transform(this->getMap()->getSRS(), extentLocal);
-        }
-        
-        _liveTiles->setDirty(extentLocal, minLevel, maxLevel);
-    }
+    //NOP
 }
 
 void
@@ -453,17 +442,7 @@ MPTerrainEngineNode::refresh(bool forceDirty)
     }
     else
     {
-        if ( _terrainOptions.incrementalUpdate() == true )
-        {
-            // run an atomic "dirty" operation:
-            //_update_mapf->sync();
-            //_liveTiles->setMapRevision( _update_mapf->getRevision(), forceDirty );
-        }
-        else
-        {
-            dirtyTerrain();
-        }
-
+        dirtyTerrain();
         _refreshRequired = false;
     }
 }
@@ -548,7 +527,7 @@ MPTerrainEngineNode::dirtyTerrain()
     {
         // Factory to create the root keys:
         KeyNodeFactory* factory = getKeyNodeFactory();
-    
+
         std::vector< TileKey > keys;
         _update_mapf->getProfile()->getAllKeysAtLOD( *_terrainOptions.firstLOD(), keys );
 
@@ -556,7 +535,7 @@ MPTerrainEngineNode::dirtyTerrain()
         OE_DEBUG << LC << "Creating " << keys.size() << " root keys.." << std::endl;
 
         osg::Group* root = new osg::Group;
-        _terrain->addChild( root );        
+        _terrain->addChild( root );
 
 
         osg::ref_ptr<osgDB::Options> dbOptions = Registry::instance()->cloneOrCreateOptions();
@@ -637,7 +616,7 @@ MPTerrainEngineNode::getKeyNodeFactory()
     if ( !knf.valid() )
     {
         // create a compiler for compiling tile models into geometry
-        bool optimizeTriangleOrientation = 
+        bool optimizeTriangleOrientation =
             getMap()->getMapOptions().elevationInterpolation() != INTERP_TRIANGULATE;
 
         MaskLayerVector maskLayers;
@@ -679,12 +658,17 @@ MPTerrainEngineNode::createNode(const TileKey&    key,
 
     OE_DEBUG << LC << "Create node for \"" << key.str() << "\"" << std::endl;
 
-    bool accumulate    = true;  // use parent data to help build tiles if neccesary
+    bool accumulate    = true;  // use parent data to help build tiles if necessary
     bool setupChildren = true;  // prepare the tile for subdivision
 
     // create the node:
     osg::ref_ptr<osg::Node> node = getKeyNodeFactory()->createNode(key, accumulate, setupChildren, progress);
 
+    if (progress && progress->isCanceled())
+    {
+        return 0;
+    }
+
     // release the reference and return it.
     return node.release();
 }
@@ -717,7 +701,7 @@ MPTerrainEngineNode::createTile( const TileKey& key )
     const osgEarth::ElevationInterpolation& interp = _update_mapf->getMapOptions().elevationInterpolation().get();
 
     // Request a heightfield from the map, falling back on lower resolution tiles
-    int tileSize = _terrainOptions.tileSize().get();    
+    int tileSize = _terrainOptions.tileSize().get();
     osg::ref_ptr<osg::HeightField> hf = HeightFieldUtils::createReferenceHeightField( key.getExtent(), tileSize, tileSize, 0u );
 
     TileKey sampleKey = key;
@@ -736,7 +720,7 @@ MPTerrainEngineNode::createTile( const TileKey& key )
                     return 0;
                 }
             }
-        }       
+        }
     }
 
     if (!populated)
@@ -750,7 +734,7 @@ MPTerrainEngineNode::createTile( const TileKey& key )
     model->_elevationData = TileModel::ElevationData(
         hf.get(),
         GeoLocator::createForKey( sampleKey, mapInfo ),
-        false );        
+        false );
 
     bool optimizeTriangleOrientation = getMap()->getMapOptions().elevationInterpolation() != INTERP_TRIANGULATE;
 
@@ -805,6 +789,7 @@ MPTerrainEngineNode::onMapModelChanged( const MapModelChange& change )
             switch( change.getAction() )
             {
             case MapModelChange::ADD_LAYER:
+            case MapModelChange::ENABLE_LAYER:
                 if (change.getImageLayer())
                     addImageLayer(change.getImageLayer());
                 else if (change.getElevationLayer())
@@ -812,6 +797,7 @@ MPTerrainEngineNode::onMapModelChanged( const MapModelChange& change )
                 break;
 
             case MapModelChange::REMOVE_LAYER:
+            case MapModelChange::DISABLE_LAYER:
                 if (change.getImageLayer())
                     removeImageLayer(change.getImageLayer());
                 else if (change.getElevationLayer())
@@ -825,11 +811,17 @@ MPTerrainEngineNode::onMapModelChanged( const MapModelChange& change )
                     moveElevationLayer(change.getElevationLayer());
                 break;
 
-            case MapModelChange::TOGGLE_ELEVATION_LAYER:
-                toggleElevationLayer( change.getElevationLayer() );
+            case MapModelChange::TOGGLE_LAYER:
+            {
+                ElevationLayer* elevationLayer = change.getElevationLayer();
+                if (elevationLayer)
+                {
+                    toggleElevationLayer(elevationLayer);
+                }
                 break;
+            }
 
-            default: 
+            default:
                 break;
             }
         }
@@ -936,7 +928,7 @@ MPTerrainEngineNode::removeElevationLayer( ElevationLayer* layerRemoved )
 
 void
 MPTerrainEngineNode::moveElevationLayer(ElevationLayer* layer)
-{    
+{
     // only need to refresh the terrain if the layer is visible
     if (layer->getVisible())
     {
@@ -969,13 +961,13 @@ MPTerrainEngineNode::updateState()
         osg::StateSet* terrainStateSet = getTerrainStateSet();
         if ( !terrainStateSet )
             return;
-        
+
         // required for multipass tile rendering to work
         terrainStateSet->setAttributeAndModes(
             new osg::Depth(osg::Depth::LEQUAL, 0, 1, true) );
 
         // activate standard mix blending.
-        terrainStateSet->setAttributeAndModes( 
+        terrainStateSet->setAttributeAndModes(
             new osg::BlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA),
             osg::StateAttribute::ON );
 
@@ -990,13 +982,13 @@ MPTerrainEngineNode::updateState()
 
             package.replace( "$MP_PRIMARY_UNIT",   Stringify() << _primaryUnit );
             package.replace( "$MP_SECONDARY_UNIT", Stringify() << (_secondaryUnit>=0?_secondaryUnit:0) );
-                
+
             package.define( "MP_USE_BLENDING", (_terrainOptions.enableBlending() == true) );
 
             package.load( vp, package.EngineVertexModel );
             package.load( vp, package.EngineVertexView );
             package.load( vp, package.EngineFragment );
-            
+
             if ( this->normalTexturesRequired() )
             {
                 package.load( vp, package.NormalMapVertex );
@@ -1086,7 +1078,7 @@ MPTerrainEngineNode::updateState()
             }
 
             // binding for the terrain texture
-            terrainStateSet->getOrCreateUniform( 
+            terrainStateSet->getOrCreateUniform(
                 "oe_layer_tex", osg::Uniform::SAMPLER_2D )->set( _primaryUnit );
 
             // binding for the secondary texture (for LOD blending)
@@ -1128,7 +1120,7 @@ MPTerrainEngineNode::updateState()
             terrainStateSet->addUniform( new osg::Uniform("oe_layer_minRange", 0.0f) );
             terrainStateSet->addUniform( new osg::Uniform("oe_layer_maxRange", FLT_MAX) );
             terrainStateSet->addUniform( new osg::Uniform("oe_layer_attenuationRange", _terrainOptions.attenuationDistance().get()) );
-            
+
             terrainStateSet->getOrCreateUniform(
                 "oe_min_tile_range_factor",
                 osg::Uniform::FLOAT)->set( *_terrainOptions.minTileRangeFactor() );
@@ -1152,7 +1144,7 @@ MPTerrainEngineNode::updateState()
                         terrainStateSet->addUniform( new osg::Uniform(
                             layer->shareTexUniformName()->c_str(),
                             layer->shareImageUnit().get() ) );
-                        
+
                     }
                 }
             }
