@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
-* Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+* Copyright 2018 Pelican Mapping
 * http://osgearth.org
 *
 * osgEarth is free software; you can redistribute it and/or modify
@@ -260,7 +260,7 @@ namespace
             {
                 osg::ref_ptr<MPGeometry> stitchGeom = new MPGeometry( d.model->_tileKey, d.frame, d.textureImageUnit );
                 stitchGeom->setName("stitchGeom");
-                d.maskRecords.push_back( MaskRecord(boundary, min_ndc, max_ndc, stitchGeom) );
+                d.maskRecords.push_back( MaskRecord(boundary, min_ndc, max_ndc, stitchGeom.get()) );
             }
         }
     }
@@ -354,28 +354,25 @@ namespace
         // allocate and assign vertices
         d.surfaceVerts = new osg::Vec3Array();
         d.surfaceVerts->reserve( d.numVerticesInSurface );
-        d.surface->setVertexArray( d.surfaceVerts );
+        d.surface->setVertexArray( d.surfaceVerts.get() );
 
         // allocate and assign normals
-        d.normals = new osg::Vec3Array();
+        d.normals = new osg::Vec3Array(osg::Array::BIND_PER_VERTEX);
         d.normals->reserve( d.numVerticesInSurface );
-        d.surface->setNormalArray( d.normals );
-        d.surface->setNormalBinding( osg::Geometry::BIND_PER_VERTEX );
+        d.surface->setNormalArray( d.normals.get() );
 
         // vertex attribution
         // for each vertex, a vec4 containing a unit extrusion vector in [0..2] and the raw elevation in [3]
-        d.surfaceAttribs = new osg::Vec4Array();
+        d.surfaceAttribs = new osg::Vec4Array(osg::Array::BIND_PER_VERTEX);
+        d.surfaceAttribs->setNormalize(false);
         d.surfaceAttribs->reserve( d.numVerticesInSurface );
-        d.surface->setVertexAttribArray( osg::Drawable::ATTRIBUTE_6, d.surfaceAttribs );
-        d.surface->setVertexAttribBinding( osg::Drawable::ATTRIBUTE_6, osg::Geometry::BIND_PER_VERTEX );
-        d.surface->setVertexAttribNormalize( osg::Drawable::ATTRIBUTE_6, false );
+        d.surface->setVertexAttribArray( osg::Drawable::ATTRIBUTE_6, d.surfaceAttribs.get() );
 
         // for each vertex, index 0 holds the interpolated elevation from the lower lod (for morphing)
-        d.surfaceAttribs2 = new osg::Vec4Array();
+        d.surfaceAttribs2 = new osg::Vec4Array(osg::Array::BIND_PER_VERTEX);
+        d.surfaceAttribs2->setNormalize(false);
         d.surfaceAttribs2->reserve( d.numVerticesInSurface );
-        d.surface->setVertexAttribArray( osg::Drawable::ATTRIBUTE_7, d.surfaceAttribs2 );
-        d.surface->setVertexAttribBinding( osg::Drawable::ATTRIBUTE_7, osg::Geometry::BIND_PER_VERTEX );
-        d.surface->setVertexAttribNormalize( osg::Drawable::ATTRIBUTE_7, false );
+        d.surface->setVertexAttribArray( osg::Drawable::ATTRIBUTE_7, d.surfaceAttribs2.get() );
         
         // temporary data structures for triangulation support
         d.elevations = new osg::FloatArray();
@@ -425,6 +422,9 @@ namespace
 
 #else // not USE_TEXCOORD_CACHE
         d.renderTileCoords = new osg::Vec2Array();
+#if OSG_MIN_VERSION_REQUIRED(3,5,6)
+        if(osg::DisplaySettings::instance()->getVertexBufferHint() == osg::DisplaySettings::VERTEX_ARRAY_OBJECT) d.renderTileCoords->setVertexBufferObject( new osg::VertexBufferObject() );
+#endif
         d.renderTileCoords->reserve( d.numVerticesInSurface );
         d.ownsTileCoords = true;
 #endif
@@ -470,6 +470,9 @@ namespace
 
 #else // not USE_TEXCOORD_CACHE
                     r._texCoords = new osg::Vec2Array();
+#if OSG_MIN_VERSION_REQUIRED(3,5,6)
+                    if(osg::DisplaySettings::instance()->getVertexBufferHint() == osg::DisplaySettings::VERTEX_ARRAY_OBJECT) r._texCoords->setVertexBufferObject( new osg::VertexBufferObject() );
+#endif
                     r._texCoords->reserve( d.numVerticesInSurface );
                     r._ownsTexCoords = true;
 #endif
@@ -581,7 +584,7 @@ namespace
 
                 if ( hf )
                 {
-                    validValue = d.model->_elevationData.getHeight( ndc, d.model->_tileLocator, heightValue, INTERP_TRIANGULATE );
+                    validValue = d.model->_elevationData.getHeight( ndc, d.model->_tileLocator.get(), heightValue, INTERP_TRIANGULATE );
                 }
 
                 ndc.z() = heightValue * d.heightScale + d.heightOffset;
@@ -718,7 +721,7 @@ namespace
 
     /**
      * If there are masking records, calculate the vertices to bound the masked area
-     * and the internal verticies to populate it. Then build a triangulation of the
+     * and the internal vertices to populate it. Then build a triangulation of the
      * area inside the masking bounding box and add this to the surface geode.
      */
     void createMaskGeometry( Data& d )
@@ -951,7 +954,7 @@ namespace
                 for (osg::Vec3Array::iterator it = maskConstraint->begin(); it != maskConstraint->end(); ++it)
                 {
                     //If the z-value was set from a mask vertex there is no need to change it.  If
-                    //it was set from a vertex from the stitching polygon it may need overriden if
+                    //it was set from a vertex from the stitching polygon it may need overridden if
                     //the vertex lies along a mask edge.  Or if it is unset, it will need to be set.
                     //if (isZSet[count] < 2)
                     if (!isZSet[count])
@@ -1055,27 +1058,24 @@ namespace
 
             osg::ref_ptr<osg::Vec3Array> stitch_verts = new osg::Vec3Array();
             stitch_verts->reserve(trig->getInputPointArray()->size());
-            stitch_geom->setVertexArray(stitch_verts);
+            stitch_geom->setVertexArray(stitch_verts.get());
 
-            osg::ref_ptr<osg::Vec3Array> stitch_norms = new osg::Vec3Array(trig->getInputPointArray()->size());
-            stitch_geom->setNormalArray( stitch_norms );
-            stitch_geom->setNormalBinding( osg::Geometry::BIND_PER_VERTEX );
+            osg::ref_ptr<osg::Vec3Array> stitch_norms = new osg::Vec3Array(osg::Array::BIND_PER_VERTEX, trig->getInputPointArray()->size());
+            stitch_geom->setNormalArray( stitch_norms.get() );
 
 
             // vertex attribution
             // for each vertex, a vec4 containing a unit extrusion vector in [0..2] and the raw elevation in [3]
-            osg::ref_ptr<osg::Vec4Array> surfaceAttribs = new osg::Vec4Array();
+            osg::ref_ptr<osg::Vec4Array> surfaceAttribs = new osg::Vec4Array(osg::Array::BIND_PER_VERTEX);
+            surfaceAttribs->setNormalize(false);
             surfaceAttribs->reserve( trig->getInputPointArray()->size() );
-            stitch_geom->setVertexAttribArray( osg::Drawable::ATTRIBUTE_6, surfaceAttribs );
-            stitch_geom->setVertexAttribBinding( osg::Drawable::ATTRIBUTE_6, osg::Geometry::BIND_PER_VERTEX );
-            stitch_geom->setVertexAttribNormalize( osg::Drawable::ATTRIBUTE_6, false );
+            stitch_geom->setVertexAttribArray( osg::Drawable::ATTRIBUTE_6, surfaceAttribs.get() );
 
             // for each vertex, index 0 holds the interpolated elevation from the lower lod (for morphing)
-            osg::ref_ptr<osg::Vec4Array> surfaceAttribs2 = new osg::Vec4Array();
+            osg::ref_ptr<osg::Vec4Array> surfaceAttribs2 = new osg::Vec4Array(osg::Array::BIND_PER_VERTEX);
+            surfaceAttribs2->setNormalize(false);
             surfaceAttribs2->reserve( trig->getInputPointArray()->size() );
-            stitch_geom->setVertexAttribArray( osg::Drawable::ATTRIBUTE_7, surfaceAttribs2 );
-            stitch_geom->setVertexAttribBinding( osg::Drawable::ATTRIBUTE_7, osg::Geometry::BIND_PER_VERTEX );
-            stitch_geom->setVertexAttribNormalize( osg::Drawable::ATTRIBUTE_7, false );
+            stitch_geom->setVertexAttribArray( osg::Drawable::ATTRIBUTE_7, surfaceAttribs2.get() );
 
 
             //Initialize tex coords
@@ -1120,7 +1120,7 @@ namespace
                 // Calculate and store the "old height", i.e the height value from
                 // the parent LOD.
                 float heightValue = 0.0;
-                d.model->_elevationData.getHeight( local_zero, d.model->_tileLocator, heightValue, INTERP_TRIANGULATE );
+                d.model->_elevationData.getHeight( local_zero, d.model->_tileLocator.get(), heightValue, INTERP_TRIANGULATE );
 
                 float     oldHeightValue = heightValue;
                 osg::Vec3 oldNormal;
@@ -1173,11 +1173,11 @@ namespace
             }
 
 
-            // Get triangles from triangulator and add as primative set to the geometry
+            // Get triangles from triangulator and add as primitive set to the geometry
             osg::ref_ptr<osg::DrawElementsUInt> tris = trig->getTriangles();
             if ( tris && tris->getNumIndices() >= 3 )
             {
-                stitch_geom->addPrimitiveSet(tris);
+                stitch_geom->addPrimitiveSet(tris.get());
             }
 
             // Finally, add it to the geode.
@@ -1604,10 +1604,10 @@ namespace
 
             // Utility arrays:
             std::vector<osg::Vec3> boundaryVerts;
-            boundaryVerts.reserve( 2 * std::max(d.numRows, d.numCols) );
+            boundaryVerts.reserve( 2 * osg::maximum(d.numRows, d.numCols) );
 
             std::vector< float > boundaryElevations;
-            boundaryElevations.reserve( 2 * std::max(d.numRows, d.numCols) );
+            boundaryElevations.reserve( 2 * osg::maximum(d.numRows, d.numCols) );
 
             //Recalculate the west side
             if (w_neighbor && w_neighbor->getNumColumns() == d.originalNumCols && w_neighbor->getNumRows() == d.originalNumRows)
@@ -1969,7 +1969,7 @@ namespace
         // in the case of full-masking, this will be empty
         if ( elements->getNumIndices() > 0 )
         {
-            d.surface->insertPrimitiveSet(0, elements); // because we always want this first.
+            d.surface->insertPrimitiveSet(0, elements.get()); // because we always want this first.
         }
     }
 
@@ -1987,6 +1987,13 @@ namespace
         if ( d.renderTileCoords.valid() )
             d.surface->_tileCoords = d.renderTileCoords;
 
+        // install the tile coordinates in the geometry.
+        if ( d.surface->_tileCoords.valid() )
+        {
+            int index = d.surface->getTexCoordArrayList().size();
+            d.surface->setTexCoordArray( index, d.surface->_tileCoords.get() );
+        }
+
         // install the render data for each layer:
         for( RenderLayerVector::const_iterator r = d.renderLayers.begin(); r != d.renderLayers.end(); ++r )
         {
@@ -2059,13 +2066,6 @@ namespace
             }
         }
 
-        // install the tile coordinates in the geometry.
-        if ( d.surface->_tileCoords.valid() )
-        {
-            int index = d.surface->getTexCoordArrayList().size();
-            d.surface->setTexCoordArray( index, d.surface->_tileCoords.get() );
-        }
-
         // elevation texture.
         d.surface->_elevTex = d.model->_elevationTexture.get();
     }
@@ -2104,10 +2104,9 @@ namespace
             de->push_back(2); de->push_back(6);
             geom->addPrimitiveSet(de);
 
-            osg::Vec4Array* c= new osg::Vec4Array();
+            osg::Vec4Array* c= new osg::Vec4Array(osg::Array::BIND_OVERALL);
             c->push_back(osg::Vec4(0,1,1,1));
             geom->setColorArray(c);
-            geom->setColorBinding(geom->BIND_OVERALL);
 
             geode->addDrawable(geom);
 
@@ -2129,7 +2128,7 @@ namespace
         geode->addDrawable(t);
 
         geode->getOrCreateStateSet()->setAttributeAndModes(new osg::Program(),0);
-        geode->getOrCreateStateSet()->setMode(GL_LIGHTING,0);
+        geode->getOrCreateStateSet()->setMode(GL_LIGHTING,0); // ok .. FFP debugging code
 
         return geode;
     }
@@ -2218,7 +2217,7 @@ TileModelCompiler::compile(TileModel*        model,
     {
         d.surfaceGeode->addDrawable( d.surface );
 
-        // tesselate the surface verts into triangles.
+        // tessellate the surface verts into triangles.
         tessellateSurfaceGeometry( d, _optimizeTriOrientation, *_options.normalizeEdges() );
 
         // build the skirts.
@@ -2243,8 +2242,13 @@ TileModelCompiler::compile(TileModel*        model,
     // camera matricies when DRAW overlaps the next frame's CULL. Please see my comments
     // in DrapingTechnique.cpp for more information.
     // NOTE: cannot set this until optimizations (above) are complete
+
+    // Using VAOs on MPGeometry doesn't work if the data variance is set to DYNAMIC,
+    // so for now, don't run with for VAO testing
+//#ifndef USE_VAO
     SetDataVarianceVisitor sdv( osg::Object::DYNAMIC );
     tile->accept( sdv );
+//#endif
 
     osg::ComputeBoundsVisitor cbv;
     d.surfaceGeode->accept(cbv);
