@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
-* Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+* Copyright 2018 Pelican Mapping
 * http://osgearth.org
 *
 * osgEarth is free software; you can redistribute it and/or modify
@@ -84,17 +84,7 @@ namespace
             const Profile* layerProfile = _layer->getProfile();
 
             //Only try to get data from the source if it actually intersects the key extent
-            bool hasDataInExtent = _layer->mayHaveDataInExtent(_key.getExtent());
-            //bool hasDataInExtent = true;
-            //if (tileSource && layerProfile)
-            //{
-            //    GeoExtent ext = _key.getExtent();
-            //    if (!layerProfile->getSRS()->isEquivalentTo( ext.getSRS()))
-            //    {
-            //        ext = layerProfile->clampAndTransformExtent( ext );
-            //    }
-            //    hasDataInExtent = tileSource->hasDataInExtent( ext );
-            //}
+            bool hasDataInExtent = _layer->mayHaveData(_key);
             
             // fetch the image from the layer.
             if (hasDataInExtent && _layer->isKeyInLegalRange(_key))
@@ -515,4 +505,9 @@ TileModelFactory::createTileModel(const TileKey&           key,
     }
 
     out_model = model.release();
+
+    if (progress && progress->isCanceled())
+    {
+        out_model = 0;
+    }
 }
