@@ -1,5 +1,5 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
+/* osgEarth - Geospatial SDK for OpenSceneGraph
 * Copyright 2008-2014 Pelican Mapping
 * http://osgearth.org
 *
@@ -61,17 +61,12 @@ _tilesLastCull(0)
     _mainThreadId = Threading::getCurrentThreadId();
 }
 
-const Map*
+osg::ref_ptr<const Map>
 EngineContext::getMap() const
 {
-#if 0
-    // debugging check
-    if (Threading::getCurrentThreadId() != _mainThreadId)
-    {
-        OE_WARN << LC << "Illegal - do not call getMap from outside the main thread\n";
-    }
-#endif
-    return _map;
+    osg::ref_ptr<const Map> map;
+    _map.lock(map);
+    return map;
 }
 
 void
