@@ -1,5 +1,5 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
+/* osgEarth - Geospatial SDK for OpenSceneGraph
 * Copyright 2008-2014 Pelican Mapping
 * http://osgearth.org
 *
@@ -175,10 +175,12 @@ namespace
     verts->push_back( (*verts)[INDEX] ); \
     normals->push_back( (*normals)[INDEX] ); \
     texCoords->push_back( (*texCoords)[INDEX] ); \
+    texCoords->back().z() = (float)((int)texCoords->back().z() | VERTEX_MARKER_SKIRT); \
     if ( neighbors ) neighbors->push_back( (*neighbors)[INDEX] ); \
     verts->push_back( (*verts)[INDEX] - ((*normals)[INDEX])*(HEIGHT) ); \
     normals->push_back( (*normals)[INDEX] ); \
     texCoords->push_back( (*texCoords)[INDEX] ); \
+    texCoords->back().z() = (float)((int)texCoords->back().z() | VERTEX_MARKER_SKIRT); \
     if ( neighbors ) neighbors->push_back( (*neighbors)[INDEX] - ((*normals)[INDEX])*(HEIGHT) ); \
 }
 
@@ -316,8 +318,8 @@ GeometryPool::createGeometry(const TileKey& tileKey,
 
             if ( populateTexCoords )
             {
-                // if masked then set textCoord z-value to 0.0
-                float marker = maskSet ? maskSet->getMarker(nx, ny) : MASK_MARKER_NORMAL;
+                // Use the Z coord as a type marker
+                float marker = maskSet ? maskSet->getMarker(nx, ny) : VERTEX_MARKER_GRID;
                 texCoords->push_back( osg::Vec3f(nx, ny, marker) );
             }
 
@@ -401,7 +403,7 @@ GeometryPool::createGeometry(const TileKey& tileKey,
         {
             maskSet = 0L;
             for (osg::Vec3Array::iterator i = texCoords->begin(); i != texCoords->end(); ++i)
-                i->z() = MASK_MARKER_NORMAL;
+                i->z() = VERTEX_MARKER_GRID;
         }
 
         // If the boundary contains the entire tile, draw nothing!
@@ -598,6 +600,8 @@ SharedGeometry::SharedGeometry()
 {
     setSupportsDisplayList(false);
     _supportsVertexBufferObjects = true;
+    _ptype.resize(64u);
+    _ptype.setAllElementsTo(GL_TRIANGLES);
 }
 
 SharedGeometry::SharedGeometry(const SharedGeometry& rhs,const osg::CopyOp& copyop):
@@ -625,7 +629,6 @@ SharedGeometry::empty() const
         (_maskElements.valid() == false || _maskElements->getNumIndices() == 0);
 }
 
-
 #ifdef SUPPORTS_VAO
 #if OSG_MIN_VERSION_REQUIRED(3,5,9)
 osg::VertexArrayState* SharedGeometry::createVertexArrayStateImplementation(osg::RenderInfo& renderInfo) const
@@ -650,6 +653,7 @@ osg::VertexArrayState* SharedGeometry::createVertexArrayState(osg::RenderInfo& r
     }
     if (texUnits)
         vas->assignTexCoordArrayDispatcher(texUnits);
+
     if (state.useVertexArrayObject(_useVertexArrayObject))
     {
         vas->generateVertexArrayObject();
@@ -659,62 +663,6 @@ osg::VertexArrayState* SharedGeometry::createVertexArrayState(osg::RenderInfo& r
 }
 #endif
 
-void SharedGeometry::compileGLObjects(osg::RenderInfo& renderInfo) const
-{
-    if (!_vertexArray)
-        return;
-
-    if (_vertexArray->getVertexBufferObject())
-    {
-        osg::State& state = *renderInfo.getState();
-        unsigned int contextID = state.getContextID();
-        osg::GLExtensions* extensions = state.get<osg::GLExtensions>();
-        if (!extensions) return;
-
-        osg::BufferObject* vbo = _vertexArray->getVertexBufferObject();
-        osg::GLBufferObject* vbo_glBufferObject = vbo->getOrCreateGLBufferObject(contextID);
-        if (vbo_glBufferObject && vbo_glBufferObject->isDirty())
-        {
-            // OSG_NOTICE<<"Compile buffer "<<glBufferObject<<std::endl;
-            vbo_glBufferObject->compileBuffer();
-            extensions->glBindBuffer(GL_ARRAY_BUFFER_ARB,0);
-        }
-
-        osg::BufferObject* ebo = _drawElements->getElementBufferObject();
-        osg::GLBufferObject* ebo_glBufferObject = ebo->getOrCreateGLBufferObject(contextID);
-        if (ebo_glBufferObject && vbo_glBufferObject->isDirty())
-        {
-            // OSG_NOTICE<<"Compile buffer "<<glBufferObject<<std::endl;
-            ebo_glBufferObject->compileBuffer();
-            extensions->glBindBuffer(GL_ELEMENT_ARRAY_BUFFER_ARB,0);
-        }
-
-#ifdef SUPPORTS_VAO
-        if (state.useVertexArrayObject(_useVertexArrayObject))
-        {
-            osg::VertexArrayState* vas = 0;
-
-            _vertexArrayStateList[contextID] = vas = createVertexArrayState(renderInfo);
-
-            osg::State::SetCurrentVertexArrayStateProxy setVASProxy(state, vas);
-
-#if OSG_MIN_VERSION_REQUIRED(3,5,6)
-            state.bindVertexArrayObject(vas);
-#else
-            vas->bindVertexArrayObject();
-#endif
-
-            if (vbo_glBufferObject) vas->bindVertexBufferObject(vbo_glBufferObject);
-            if (ebo_glBufferObject) vas->bindElementBufferObject(ebo_glBufferObject);
-        }
-#endif
-    }
-    else
-    {
-        Drawable::compileGLObjects(renderInfo);
-    }
-}
-
 void SharedGeometry::resizeGLObjectBuffers(unsigned int maxSize)
 {
     osg::Drawable::resizeGLObjectBuffers(maxSize);
@@ -738,7 +686,7 @@ void SharedGeometry::releaseGLObjects(osg::State* state) const
 }
 
 // called from DrawTileCommand
-void SharedGeometry::render(GLenum primitiveType, osg::RenderInfo& renderInfo) const
+void SharedGeometry::drawImplementation(osg::RenderInfo& renderInfo) const
 {
     osg::State& state = *renderInfo.getState();
     
@@ -754,10 +702,9 @@ void SharedGeometry::render(GLenum primitiveType, osg::RenderInfo& renderInfo) c
 
 #ifdef SUPPORTS_VAO
     osg::VertexArrayState* vas = state.getCurrentVertexArrayState();
+    
     if (!state.useVertexArrayObject(_useVertexArrayObject) || vas->getRequiresSetArrays())
     {
-        // OSG_NOTICE<<"   sending vertex arrays vas->getRequiresSetArrays()="<<vas->getRequiresSetArrays()<<std::endl;
-
         vas->lazyDisablingOfVertexAttributes();
 
         // set up arrays
@@ -806,15 +753,20 @@ void SharedGeometry::render(GLenum primitiveType, osg::RenderInfo& renderInfo) c
     bool request_bind_unbind = true;
 #endif
 
+    GLenum primitiveType = _ptype[state.getContextID()];
+
     osg::GLBufferObject* ebo = _drawElements->getOrCreateGLBufferObject(state.getContextID());
 
     if (ebo)
     {
         /*if (request_bind_unbind)*/ state.bindElementBufferObject(ebo);
 
-        glDrawElements(primitiveType, _drawElements->getNumIndices(), _drawElements->getDataType(), (const GLvoid *)(ebo->getOffset(_drawElements->getBufferIndex())));
+        if (_drawElements->getNumIndices() > 0u)
+        {
+            glDrawElements(primitiveType, _drawElements->getNumIndices(), _drawElements->getDataType(), (const GLvoid *)(ebo->getOffset(_drawElements->getBufferIndex())));
+        }
 
-        if (_maskElements.valid())
+        if (_maskElements.valid() && _maskElements->getNumIndices() > 0u)
         {
             glDrawElements(primitiveType, _maskElements->getNumIndices(), _maskElements->getDataType(), (const GLvoid *)(ebo->getOffset(_maskElements->getBufferIndex())));
         }
@@ -823,9 +775,12 @@ void SharedGeometry::render(GLenum primitiveType, osg::RenderInfo& renderInfo) c
     }
     else
     {
-        glDrawElements(primitiveType, _drawElements->getNumIndices(), _drawElements->getDataType(), _drawElements->getDataPointer());
+        if (_drawElements->getNumIndices() > 0u)
+        {
+            glDrawElements(primitiveType, _drawElements->getNumIndices(), _drawElements->getDataType(), _drawElements->getDataPointer());
+        }
 
-        if (_maskElements.valid())
+        if (_maskElements.valid() && _maskElements->getNumIndices() > 0u)
         {
             glDrawElements(primitiveType, _maskElements->getNumIndices(), _maskElements->getDataType(), _maskElements->getDataPointer());
         }
