@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
-* Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+* Copyright 2018 Pelican Mapping
 * http://osgearth.org
 *
 * osgEarth is free software; you can redistribute it and/or modify
@@ -36,7 +36,9 @@
 #include <osgEarth/ShaderFactory>
 #include <osgEarth/ShaderGenerator>
 #include <osgEarth/Shaders>
+#include <osgEarth/GLUtils>
 #include <osgEarth/Lighting>
+#include <osgEarth/PointDrawable>
 
 #include <osg/MatrixTransform>
 #include <osg/ShapeDrawable>
@@ -101,10 +103,9 @@ namespace
             texCoords->reserve( latSegments * lonSegments );
             geom->setTexCoordArray( 0, texCoords );
 
-            normals = new osg::Vec3Array();
+            normals = new osg::Vec3Array(osg::Array::BIND_PER_VERTEX);
             normals->reserve( latSegments * lonSegments );
             geom->setNormalArray( normals );
-            geom->setNormalBinding(osg::Geometry::BIND_PER_VERTEX );
         }
 
         osg::DrawElementsUShort* el = new osg::DrawElementsUShort( GL_TRIANGLES );
@@ -193,22 +194,15 @@ namespace
 
 //---------------------------------------------------------------------------
 
-SimpleSkyNode::SimpleSkyNode(const SpatialReference* srs) :
-SkyNode()
-{
-    initialize(srs);
-}
-
-SimpleSkyNode::SimpleSkyNode(const SpatialReference* srs,
-                             const SimpleSkyOptions& options) :
+SimpleSkyNode::SimpleSkyNode(const SimpleSkyOptions& options) :
 SkyNode ( options ),
 _options( options )
 {
-    initialize(srs);
+    construct();
 }
 
 void
-SimpleSkyNode::initialize(const SpatialReference* srs)
+SimpleSkyNode::construct()
 {
     // protect us from the ShaderGenerator.
     ShaderGenerator::setIgnoreHint(this, true);
@@ -235,9 +229,9 @@ SimpleSkyNode::initialize(const SpatialReference* srs)
     }
 
     // only supports geocentric for now.
-    if ( srs && !srs->isGeographic() )
+    if (getReferencePoint().isValid())
     {
-        OE_WARN << LC << "Sorry, SimpleSky only supports geocentric maps." << std::endl;
+        OE_WARN << LC << "Found an ephemeris reference point, but SimpleSky does not support projected maps" << std::endl;
         return;
     }
 
@@ -245,12 +239,15 @@ SimpleSkyNode::initialize(const SpatialReference* srs)
     _cullContainer = new osg::Group();
     
     // set up the astronomical parameters:
-    _ellipsoidModel = srs ? srs->getEllipsoid() : new osg::EllipsoidModel();
+    osg::ref_ptr<const SpatialReference> wgs84 = SpatialReference::get("wgs84");
+    _ellipsoidModel = wgs84->getEllipsoid();
     _innerRadius = osg::minimum(
         _ellipsoidModel->getRadiusPolar(),
         _ellipsoidModel->getRadiusEquator() );
     _outerRadius = _innerRadius * 1.025f;
-    _sunDistance = _innerRadius * 12000.0f;
+
+    CelestialBody sun = getEphemeris()->getSunPosition(DateTime());
+    _sunDistance = sun.altitude.as(Units::METERS); //149600000000.0; //_innerRadius * 12000.0f;
     
     if ( Registry::capabilities().supportsGLSL() )
     {
@@ -260,10 +257,6 @@ SimpleSkyNode::initialize(const SpatialReference* srs)
         _lightPosUniform->set( lightPos / lightPos.length() );
         stateset->addUniform( _lightPosUniform.get() );
 
-        // default GL_LIGHTING uniform setting
-        //stateset->addUniform(
-        //    Registry::shaderFactory()->createUniformForGLMode(GL_LIGHTING, 1) );
-
         stateset->setDefine(OE_LIGHTING_DEFINE, osg::StateAttribute::ON);
 
         // make the uniforms and the terrain lighting shaders.
@@ -277,6 +270,12 @@ SimpleSkyNode::initialize(const SpatialReference* srs)
         makeMoon();
 
         makeStars();
+
+        // Decorations are shown by default, so hide them as needed
+        if (_options.sunVisible() == false) setSunVisible(false);
+        if (_options.moonVisible() == false) setMoonVisible(false);
+        if (_options.starsVisible() == false) setStarsVisible(false);
+        if (_options.atmosphereVisible() == false) setAtmosphereVisible(false);
     }
 
     // Update everything based on the date/time.
@@ -322,7 +321,23 @@ SimpleSkyNode::traverse( osg::NodeVisitor& nv )
     } 
 
     SkyNode::traverse( nv ); 
-} 
+}
+
+void
+SimpleSkyNode::releaseGLObjects(osg::State* state) const
+{
+    SkyNode::releaseGLObjects(state);
+    if (_cullContainer.valid())
+        _cullContainer->releaseGLObjects(state);
+}
+
+void
+SimpleSkyNode::resizeGLObjectBuffers(unsigned maxSize)
+{
+    SkyNode::resizeGLObjectBuffers(maxSize);
+    if (_cullContainer.valid())
+        _cullContainer->resizeGLObjectBuffers(maxSize);
+}
 
 void
 SimpleSkyNode::onSetEphemeris()
@@ -339,12 +354,11 @@ SimpleSkyNode::onSetDateTime()
         osg::View* view = 0L;
         const DateTime& dt = getDateTime();
 
-        osg::Vec3d sunPos = getEphemeris()->getSunPositionECEF( dt );
-        osg::Vec3d moonPos = getEphemeris()->getMoonPositionECEF( dt );
+        CelestialBody sun = getEphemeris()->getSunPosition(dt);
+        setSunPosition( sun.geocentric );
 
-        sunPos.normalize();
-        setSunPosition( sunPos );
-        setMoonPosition( moonPos );
+        CelestialBody moon = getEphemeris()->getMoonPosition(dt);
+        setMoonPosition( moon.geocentric );
 
         // position the stars:
         double time_r = dt.hours()/24.0; // 0..1
@@ -377,9 +391,9 @@ SimpleSkyNode::attach( osg::View* view, int lightNum )
 }
 
 void
-SimpleSkyNode::setSunPosition(const osg::Vec3& pos)
+SimpleSkyNode::setSunPosition(const osg::Vec3d& pos)
 {
-    osg::Vec3 npos = pos;
+    osg::Vec3d npos = pos;
     npos.normalize();
     _light->setPosition( osg::Vec4(npos, 0.0f) ); // directional light
 
@@ -387,23 +401,22 @@ SimpleSkyNode::setSunPosition(const osg::Vec3& pos)
     
     if ( _lightPosUniform.valid() )
     {
-        _lightPosUniform->set( pos/pos.length() );
+        _lightPosUniform->set( osg::Vec3(npos) );
     }
 
     if ( _sunXform.valid() )
     {
-        _sunXform->setMatrix( osg::Matrix::translate( 
-            _sunDistance * pos.x(), 
-            _sunDistance * pos.y(),
-            _sunDistance * pos.z() ) );
+        _sunXform->setMatrix( osg::Matrix::translate(pos) );
     }
 }
 
 void
 SimpleSkyNode::setMoonPosition(const osg::Vec3d& pos)
 {
-    if ( _moonXform.valid() )
-        _moonXform->setMatrix( osg::Matrixd::translate(pos.x(), pos.y(), pos.z()) );
+    if (_moonXform.valid())
+    {
+        _moonXform->setMatrix(osg::Matrixd::translate(pos));
+    }
 }
 
 void
@@ -520,7 +533,7 @@ SimpleSkyNode::makeAtmosphere(const osg::EllipsoidModel* em)
     
     // configure the state set:
     osg::StateSet* atmosSet = drawable->getOrCreateStateSet();
-    atmosSet->setMode( GL_LIGHTING, osg::StateAttribute::OFF );
+    GLUtils::setLighting(atmosSet, osg::StateAttribute::OFF);
     atmosSet->setAttributeAndModes( new osg::CullFace(osg::CullFace::BACK), osg::StateAttribute::ON );
     atmosSet->setAttributeAndModes( new osg::Depth( osg::Depth::LESS, 0, 1, false ) ); // no depth write
     atmosSet->setAttributeAndModes( new osg::Depth(osg::Depth::ALWAYS, 0, 1, false) ); // no zbuffer
@@ -558,9 +571,12 @@ SimpleSkyNode::makeSun()
     sun->setMode( osg::Billboard::POINT_ROT_EYE );
     sun->setNormal( osg::Vec3(0, 0, 1) );
 
-    float sunRadius = _innerRadius * 100.0f;
+    //float sunRadius = _innerRadius * 100.0f;
+    //sun->addDrawable( s_makeDiscGeometry( sunRadius*80.0f ) ); 
 
-    sun->addDrawable( s_makeDiscGeometry( sunRadius*80.0f ) ); 
+    const double zoomFactor = 80.0; // to account for the solare glare
+    const double sunRadius = 695700000.0;
+    sun->addDrawable(s_makeDiscGeometry(sunRadius * zoomFactor));
 
     osg::StateSet* set = sun->getOrCreateStateSet();
     set->setMode( GL_BLEND, 1 );
@@ -595,7 +611,6 @@ SimpleSkyNode::makeSun()
     _sun = cam;
 
     // make the sun's transform:
-    // todo: move this?
     _sunXform = new osg::MatrixTransform();
     _sunXform->setMatrix( osg::Matrix::translate( 
         _sunDistance * _light->getPosition().x(),
@@ -610,35 +625,45 @@ void
 SimpleSkyNode::makeMoon()
 {
     osg::ref_ptr< osg::EllipsoidModel > em = new osg::EllipsoidModel( 1738140.0, 1735970.0 );   
-    osg::Geode* moon = new osg::Geode;
-    moon->getOrCreateStateSet()->setAttributeAndModes( new osg::Program(), osg::StateAttribute::OFF | osg::StateAttribute::PROTECTED );
-    osg::Geometry* geom = s_makeEllipsoidGeometry( em.get(), em->getRadiusEquator(), true );    
+    
+    osg::Geometry* moonDrawable = s_makeEllipsoidGeometry( em.get(), em->getRadiusEquator(), true );    
+    osg::StateSet* stateSet = moonDrawable->getOrCreateStateSet();
+
     //TODO:  Embed this texture in code or provide a way to have a default resource directory for osgEarth.
     //       Right now just need to have this file somewhere in your OSG_FILE_PATH
+    stateSet->setAttributeAndModes( new osg::Program(), osg::StateAttribute::OFF | osg::StateAttribute::PROTECTED );
     osg::ref_ptr<osg::Image> image = osgDB::readRefImageFile( "moon_1024x512.jpg" );
+
     osg::Texture2D * texture = new osg::Texture2D( image );
     texture->setFilter(osg::Texture::MIN_FILTER,osg::Texture::LINEAR);
     texture->setFilter(osg::Texture::MAG_FILTER,osg::Texture::LINEAR);
     texture->setResizeNonPowerOfTwoHint(false);
-    geom->getOrCreateStateSet()->setTextureAttributeAndModes( 0, texture, osg::StateAttribute::ON | osg::StateAttribute::PROTECTED);
+    stateSet->setTextureAttributeAndModes( 0, texture, osg::StateAttribute::ON | osg::StateAttribute::PROTECTED);
+#ifdef OSG_GL3_AVAILABLE
+    // Adjust for loss of GL_LUMINANCE in glTexture2D's format parameter.  OSG handles the texture's internal format,
+    // but the format parameter comes from the image's pixel format field.
+    if (image.valid() && image->getPixelFormat() == GL_LUMINANCE)
+    {
+      image->setPixelFormat(GL_RED);
+      // Swizzle the RGB all to RED in order to match previous GL_LUMINANCE behavior
+      texture->setSwizzle(osg::Vec4i(GL_RED, GL_RED, GL_RED, GL_ONE));
+    }
+#endif
 
-    osg::Vec4Array* colors = new osg::Vec4Array(1);    
-    geom->setColorArray( colors );
-    geom->setColorBinding(osg::Geometry::BIND_OVERALL);
+    osg::Vec4Array* colors = new osg::Vec4Array(osg::Array::BIND_OVERALL, 1);    
+    moonDrawable->setColorArray( colors );
     (*colors)[0] = osg::Vec4(1, 1, 1, 1 );
-    moon->addDrawable( geom  ); 
 
-    osg::StateSet* set = moon->getOrCreateStateSet();
     // configure the stateset
-    set->setMode( GL_LIGHTING, osg::StateAttribute::ON );
-    set->setAttributeAndModes( new osg::CullFace( osg::CullFace::BACK ), osg::StateAttribute::ON);
-    set->setRenderBinDetails( BIN_MOON, "RenderBin" );
-    set->setAttributeAndModes( new osg::Depth(osg::Depth::ALWAYS, 0, 1, false), osg::StateAttribute::ON );
-    set->setAttributeAndModes( new osg::BlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA), osg::StateAttribute::ON );
+    GLUtils::setLighting(stateSet, osg::StateAttribute::ON );
+    stateSet->setAttributeAndModes( new osg::CullFace( osg::CullFace::BACK ), osg::StateAttribute::ON);
+    stateSet->setRenderBinDetails( BIN_MOON, "RenderBin" );
+    stateSet->setAttributeAndModes( new osg::Depth(osg::Depth::ALWAYS, 0, 1, false), osg::StateAttribute::ON );
+    stateSet->setAttributeAndModes( new osg::BlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA), osg::StateAttribute::ON );
     
 #if 1 //defined(OSG_GLES2_AVAILABLE) || defined(OSG_GLES3_AVAILABLE)
 
-    set->addUniform(new osg::Uniform("moonTex", 0));
+    stateSet->addUniform(new osg::Uniform("moonTex", 0));
 
     // create shaders
     Shaders pkg;
@@ -651,7 +676,7 @@ SimpleSkyNode::makeMoon()
         osg::Shader::FRAGMENT,
         ShaderLoader::load(pkg.Moon_Frag, pkg) );
     program->addShader( fs );
-    set->setAttributeAndModes( program, osg::StateAttribute::ON | osg::StateAttribute::PROTECTED );
+    stateSet->setAttributeAndModes( program, osg::StateAttribute::ON | osg::StateAttribute::PROTECTED );
 #endif
 
     // A nested camera isolates the projection matrix calculations so the node won't 
@@ -660,14 +685,15 @@ SimpleSkyNode::makeMoon()
     cam->getOrCreateStateSet()->setRenderBinDetails( BIN_MOON, "RenderBin" );
     cam->setRenderOrder( osg::Camera::NESTED_RENDER );
     cam->setComputeNearFarMode( osg::CullSettings::COMPUTE_NEAR_FAR_USING_BOUNDING_VOLUMES );
-    cam->addChild( moon );
+    cam->addChild( moonDrawable );
 
     _moon = cam;
 
     // make the moon's transform:
-    _moonXform = new osg::MatrixTransform();    
-    osg::Vec3d moonPosECEF = getEphemeris()->getMoonPositionECEF( getDateTime() );
-    _moonXform->setMatrix( osg::Matrix::translate( moonPosECEF ) ); 
+    CelestialBody moon = getEphemeris()->getMoonPosition(getDateTime());
+
+    _moonXform = new osg::MatrixTransform();   
+    _moonXform->setMatrix( osg::Matrix::translate( moon.geocentric ) ); 
     _moonXform->addChild( _moon.get() );
 
     _cullContainer->addChild( _moonXform.get() );
@@ -738,77 +764,55 @@ SimpleSkyNode::buildStarGeometry(const std::vector<StarData>& stars)
 {
     double minMag = DBL_MAX, maxMag = DBL_MIN;
 
-    osg::Vec3Array* coords = new osg::Vec3Array();
-    std::vector<StarData>::const_iterator p;
-    for( p = stars.begin(); p != stars.end(); p++ )
+    PointDrawable* drawable = new PointDrawable();
+    drawable->setPointSize(_options.starSize().get());
+    drawable->allocate(stars.size());
+
+    for(unsigned p=0; p<stars.size(); ++p)
     {
+        const StarData& star = stars[p];
+
         osg::Vec3d v = getEphemeris()->getECEFfromRADecl(
-            p->right_ascension, 
-            p->declination, 
+            star.right_ascension, 
+            star.declination, 
             _starRadius );
 
-        coords->push_back( v );
+        drawable->setVertex(p, v);
 
-        if ( p->magnitude < minMag ) minMag = p->magnitude;
-        if ( p->magnitude > maxMag ) maxMag = p->magnitude;
+        if ( star.magnitude < minMag ) minMag = star.magnitude;
+        if ( star.magnitude > maxMag ) maxMag = star.magnitude;
     }
 
-    osg::Vec4Array* colors = new osg::Vec4Array();
-    for( p = stars.begin(); p != stars.end(); p++ )
+    for(unsigned p=0; p<stars.size(); ++p)
     {
-        float c = ( (p->magnitude-minMag) / (maxMag-minMag) );
-        colors->push_back( osg::Vec4(c,c,c,1.0f) );
+        const StarData& star = stars[p];
+        float c = (star.magnitude-minMag) / (maxMag-minMag);
+        drawable->setColor(p, osg::Vec4(c,c,c,1.0f));
     }
 
-    osg::Geometry* geometry = new osg::Geometry;
-    geometry->setUseVertexBufferObjects(true);
+    drawable->finish();
 
-    geometry->setVertexArray( coords );
-    geometry->setColorArray( colors );
-    geometry->setColorBinding(osg::Geometry::BIND_PER_VERTEX);
-    geometry->addPrimitiveSet( new osg::DrawArrays(osg::PrimitiveSet::POINTS, 0, coords->size()));
+    osg::StateSet* sset = drawable->getOrCreateStateSet();
 
-    osg::StateSet* sset = geometry->getOrCreateStateSet();
+    const osgEarth::Capabilities& caps = osgEarth::Registry::capabilities();
 
-#if !defined(OSG_GL3_AVAILABLE)
-    // In GL3, PointSprite is no longer available, and is always on.
-    sset->setTextureAttributeAndModes( 0, new osg::PointSprite(), osg::StateAttribute::ON );
-#endif
-    sset->setMode( GL_VERTEX_PROGRAM_POINT_SIZE, osg::StateAttribute::ON );
-
-    Shaders pkg;
-    std::string starVertSource, starFragSource;
-    if ( Registry::capabilities().isGLES() )
-    {
-        starVertSource = ShaderLoader::load(pkg.Stars_GLES_Vert, pkg);
-        starFragSource = ShaderLoader::load(pkg.Stars_GLES_Frag, pkg);
-    }
-    else
-    {
-        starVertSource = ShaderLoader::load(pkg.Stars_Vert, pkg);
-        starFragSource = ShaderLoader::load(pkg.Stars_Frag, pkg);
-    }
-
-    osg::Program* program = new osg::Program;
-    program->addShader( new osg::Shader(osg::Shader::VERTEX, starVertSource) );
-    program->addShader( new osg::Shader(osg::Shader::FRAGMENT, starFragSource) );
-    sset->setAttributeAndModes( program, osg::StateAttribute::ON | osg::StateAttribute::PROTECTED );
+    VirtualProgram* vp = VirtualProgram::getOrCreate(drawable->getOrCreateStateSet());
+    vp->setName("SimpleSky Stars");
+    Shaders shaders;
+    shaders.load(vp, shaders.Stars_Vert);
+    shaders.load(vp, shaders.Stars_Frag);
+    vp->setInheritShaders(false);
 
     sset->setRenderBinDetails( BIN_STARS, "RenderBin");
     sset->setAttributeAndModes( new osg::Depth(osg::Depth::ALWAYS, 0, 1, false), osg::StateAttribute::ON );
     sset->setMode(GL_BLEND, 1);
 
-    osg::Geode* starGeode = new osg::Geode;
-    starGeode->addDrawable( geometry );
-
     // A separate camera isolates the projection matrix calculations.
     osg::Camera* cam = new osg::Camera();
     cam->getOrCreateStateSet()->setRenderBinDetails( BIN_STARS, "RenderBin" );
     cam->setRenderOrder( osg::Camera::NESTED_RENDER );
-    cam->setNearFarRatio(1e-9);
-    cam->setComputeNearFarMode( osg::CullSettings::COMPUTE_NEAR_FAR_USING_BOUNDING_VOLUMES );
-    cam->addChild( starGeode );
 
+    cam->addChild( drawable );
     return cam;
 }
 
