@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
-* Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+* Copyright 2018 Pelican Mapping
 * http://osgearth.org
 *
 * osgEarth is free software; you can redistribute it and/or modify
@@ -119,7 +119,7 @@ TMSTileSource::initialize(const osgDB::Options* dbOptions)
 
         if (!_tileMap.valid())
         {
-            return Status::Error( Status::ResourceUnavailable, Stringify() << "Failed to read tilemap from " << tmsURI.full() );
+            return Status::Error( Status::ResourceUnavailable, Stringify() << "Failed to read configuration from " << tmsURI.full() );
         }
 
         OE_INFO << LC
@@ -155,27 +155,7 @@ TMSTileSource::initialize(const osgDB::Options* dbOptions)
             {
                 this->getDataExtents().push_back(*itr);
             }
-        }
-        else
-        {
-            
-            GeoExtent extent;
-
-            // Push back a single area that covers the "bounds" present in the TMS dataset.
-            if (_tileMap.valid())
-            {
-                double minX, minY, maxX, maxY;
-                _tileMap->getExtents(minX, minY, maxX, maxY);
-                extent = GeoExtent(profile->getSRS(), minX, minY, maxX, maxY);
-            }
-
-            // If the extent isn't valid, use the profile's extent.
-            if (!extent.isValid() || extent.width() <= 0.0 || extent.height() <= 0.0 )
-            {
-                extent = profile->getExtent();
-            }
-            this->getDataExtents().push_back(DataExtent(extent, 0, _tileMap->getMaxLevel()));
-        }
+        }        
     }
     return STATUS_OK;
 }
@@ -218,8 +198,9 @@ TMSTileSource::createImage(const TileKey&    key,
 
         osg::ref_ptr<osg::Image> image;
         if (!image_url.empty())
-        {
-            image = URI(image_url).readImage( _dbOptions.get(), progress ).getImage();
+        {     
+            URI uri(image_url, _options.url()->context());
+            image = uri.readImage( _dbOptions.get(), progress ).getImage();
         }
 
         if (!image.valid())
@@ -238,7 +219,7 @@ TMSTileSource::createImage(const TileKey&    key,
         
         if (image.valid() && _options.coverage() == true)
         {
-            image->setInternalTextureFormat(GL_LUMINANCE32F_ARB);
+            image->setInternalTextureFormat(GL_R16F);
             ImageUtils::markAsUnNormalized(image.get(), true);
         }
 
