@@ -1,6 +1,6 @@
 /* -*_maxPolyTilingAngle_deg-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
- * Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+ * Copyright 2018 Pelican Mapping
  * http://osgearth.org
  *
  * osgEarth is free software; you can redistribute it and/or modify
@@ -20,20 +20,21 @@
 #include <osgEarthFeatures/Session>
 #include <osgEarthFeatures/FeatureSourceIndexNode>
 #include <osgEarthFeatures/PolygonizeLines>
-#include <osgEarthFeatures/GPULines>
 #include <osgEarthSymbology/TextSymbol>
 #include <osgEarthSymbology/PointSymbol>
 #include <osgEarthSymbology/LineSymbol>
 #include <osgEarthSymbology/PolygonSymbol>
 #include <osgEarthSymbology/MeshSubdivider>
 #include <osgEarthSymbology/ResourceCache>
-#include <osgEarthSymbology/MeshConsolidator>
 #include <osgEarth/Tessellator>
 #include <osgEarth/Utils>
 #include <osgEarth/Clamping>
+#include <osgEarth/LineDrawable>
+#include <osgEarth/PointDrawable>
+#include <osgEarth/StateSetCache>
+#include <osgEarth/ShaderGenerator>
 #include <osg/Geode>
 #include <osg/Geometry>
-#include <osg/LineWidth>
 #include <osg/LineStipple>
 #include <osg/Point>
 #include <osg/MatrixTransform>
@@ -98,7 +99,8 @@ _maxAngle_deg ( 180.0 ),
 _geoInterp    ( GEOINTERP_RHUMB_LINE ),
 _maxPolyTilingAngle_deg( 45.0f ),
 _optimizeVertexOrdering( false ),
-_maximumCreaseAngle( 0.0f )
+_maximumCreaseAngle( 0.0f ),
+_shaderPolicy(SHADERPOLICY_GENERATE)
 {
     //nop
 }
@@ -174,10 +176,8 @@ BuildGeometryFilter::processPolygons(FeatureList& features, FilterContext& conte
             osg::Matrixd w2l, l2w;
             if (makeECEF)
             {
-                osgEarth::GeoExtent featureExtent(featureSRS);
-                featureExtent.expandToInclude(part->getBounds());
-
-                computeLocalizers(context, featureExtent, w2l, l2w);
+                osgEarth::GeoExtent partExtent(featureSRS, part->getBounds());
+                computeLocalizers(context, partExtent, w2l, l2w);
             }
             else
             {
@@ -226,10 +226,9 @@ BuildGeometryFilter::processPolygons(FeatureList& features, FilterContext& conte
                 // assign the primary color array. PER_VERTEX required in order to support
                 // vertex optimization later
                 unsigned count = osgGeom->getVertexArray()->getNumElements();
-                osg::Vec4Array* colors = new osg::Vec4Array;
+                osg::Vec4Array* colors = new osg::Vec4Array(osg::Array::BIND_PER_VERTEX);
                 colors->assign( count, primaryColor );
                 osgGeom->setColorArray( colors );
-                osgGeom->setColorBinding( osg::Geometry::BIND_PER_VERTEX );
 
                 geode->addDrawable( osgGeom );
 
@@ -280,7 +279,7 @@ BuildGeometryFilter::processPolygonizedLines(FeatureList&   features,
                                              bool           twosided,
                                              FilterContext& context)
 {
-    osg::Group* group = new osg::Group;    
+    osg::Group* group = new osg::Group;
 
     // establish some referencing
     bool                    makeECEF   = false;
@@ -296,7 +295,7 @@ BuildGeometryFilter::processPolygonizedLines(FeatureList&   features,
 
     // We need to create a different geode for each texture that is used so they can share statesets.
     typedef std::map< std::string, osg::ref_ptr< osg::Geode > > TextureToGeodeMap;
-    TextureToGeodeMap geodes;    
+    TextureToGeodeMap geodes;
 
     // iterate over all features.
     for( FeatureList::iterator i = features.begin(); i != features.end(); ++i )
@@ -316,7 +315,7 @@ BuildGeometryFilter::processPolygonizedLines(FeatureList&   features,
         if (line->imageURI().isSet() && context.getSession() && context.getSession()->getResourceCache())
         {
             StringExpression temp( *line->imageURI() );
-            imageURI = input->eval( temp, context.getSession());            
+            imageURI = URI(input->eval(temp, context.getSession()), temp.uriContext()).full();
         }
 
         // Try to find the existing geode, otherwise create one.
@@ -410,7 +409,7 @@ BuildGeometryFilter::processPolygonizedLines(FeatureList&   features,
                 Clamping::applyDefaultClampingAttrs( geom, input->getDouble("__oe_verticalOffset", 0.0) );
                 Clamping::setHeights( geom, copyHeights._newHeights.get() );
                 //OE_WARN << "heights = " << hats->size() << ", new hats = " << copyHeights._newHeights->size() << ", verts=" << geom->getVertexArray()->getNumElements() << std::endl;
-            }            
+            }
         }
         polygonizer.installShaders( geode.get() );
     }
@@ -441,11 +440,12 @@ BuildGeometryFilter::processPolygonizedLines(FeatureList&   features,
 }
 
 
-osg::Geode*
+osg::Group*
 BuildGeometryFilter::processLines(FeatureList& features, FilterContext& context)
 {
-    osg::Geode* geode = new osg::Geode();
-    
+    // Group to contain all the lines we create here
+    LineGroup* drawables = new LineGroup();
+
     bool makeECEF = false;
     const SpatialReference* featureSRS = 0L;
     const SpatialReference* outputSRS = 0L;
@@ -458,9 +458,13 @@ BuildGeometryFilter::processLines(FeatureList& features, FilterContext& context)
         makeECEF = outputSRS->isGeographic();
     }
 
-    optional<Stroke> masterStroke;
-    
-    for( FeatureList::iterator f = features.begin(); f != features.end(); ++f )
+    // Need to know if we are GPU clamping so we can add more attribs
+    bool doGpuClamping =
+        _style.has<AltitudeSymbol>() &&
+        _style.get<AltitudeSymbol>()->technique() == AltitudeSymbol::TECHNIQUE_GPU;
+
+    // For each input feature:
+    for (FeatureList::iterator f = features.begin(); f != features.end(); ++f)
     {
         Feature* input = f->get();
 
@@ -473,12 +477,6 @@ BuildGeometryFilter::processLines(FeatureList& features, FilterContext& context)
         if ( !line )
             continue;
 
-        // save the first stroke to use for shader generation of GPU lines if used
-        if (!masterStroke.isSet())
-        {
-            masterStroke = line->stroke().get();
-        }
-
         // run a symbol script if present.
         if ( line->script().isSet() )
         {
@@ -486,9 +484,6 @@ BuildGeometryFilter::processLines(FeatureList& features, FilterContext& context)
             input->eval( temp, &context );
         }
 
-        // GPU line generator (if used)
-        GPULinesOperator gpuLines(line->stroke().get());
-
         GeometryIterator parts( input->getGeometry(), true );
         while( parts.hasMore() )
         {
@@ -498,115 +493,93 @@ BuildGeometryFilter::processLines(FeatureList& features, FilterContext& context)
             if ( part->size() < 2 )
                 continue;
 
-            // collect all the pre-transformation HAT (Z) values.
-            // (Note: GPU Lines have doubled-up vertices, so for that case we 
-            // have to add 2 hats for every input point.)
-            osg::ref_ptr<osg::FloatArray> hats = new osg::FloatArray();
-
-            if (_useGPULines == true)
-                hats->reserve(part->size() * 2);
-            else
-                hats->reserve(part->size());
-
-            for(Geometry::const_iterator i = part->begin(); i != part->end(); ++i )
-            {
-                hats->push_back( i->z() );
-                if (_useGPULines == true)
-                    hats->push_back(i->z());
-            }
-
             // if the underlying geometry is a ring (or a polygon), use a line loop; otherwise
             // use a line strip.
             bool isRing = (dynamic_cast<Ring*>(part) != 0L);
-            
+
             // resolve the color:
             osg::Vec4f primaryColor = line->stroke()->color();
 
-            // generate the geometry:
-            osg::Vec3Array* allPoints = new osg::Vec3Array();
+            // generate the geometry and localize to the local tangent plane
+            osg::ref_ptr< osg::Vec3Array > allPoints = new osg::Vec3Array();
+            transformAndLocalize( part->asVector(), featureSRS, allPoints.get(), outputSRS, _world2local, makeECEF );
 
-            transformAndLocalize( part->asVector(), featureSRS, allPoints, outputSRS, _world2local, makeECEF );
+            // construct a drawable for the lines
+            LineDrawable* drawable = new LineDrawable(isRing? GL_LINE_LOOP : GL_LINE_STRIP);
 
-            osg::ref_ptr<osg::Geometry> osgGeom;
+            drawable->importVertexArray(allPoints.get());
 
-            if (_useGPULines == true)
+            if (line->stroke().isSet())
             {
-                // Lines tessellated on the GPU - replacement for deprecated glLineWidth
-                osgGeom = gpuLines(allPoints, isRing);
-            }
-            else
-            {
-                // normal GL lines
-                osgGeom = new osg::Geometry();
-                osgGeom->setUseVertexBufferObjects(true);
-                osgGeom->setUseDisplayList(false);
-                GLenum primMode = isRing ? GL_LINE_LOOP : GL_LINE_STRIP;
-                osgGeom->addPrimitiveSet(new osg::DrawArrays(primMode, 0, allPoints->getNumElements()));
-                osgGeom->setVertexArray(allPoints);
+                if (line->stroke()->width().isSet())
+                    drawable->setLineWidth(line->stroke()->width().get());
+
+                if (line->stroke()->stipplePattern().isSet())
+                    drawable->setStipplePattern(line->stroke()->stipplePattern().get());
+
+                if (line->stroke()->stippleFactor().isSet())
+                    drawable->setStippleFactor(line->stroke()->stippleFactor().get());
+
+                if (line->stroke()->smooth().isSet())
+                    drawable->setLineSmooth(line->stroke()->smooth().get());
             }
 
-            if (osgGeom.valid())
+            // For GPU clamping, we need an attribute array with Heights above Terrain in it.
+            if (doGpuClamping)
             {
-                // embed the feature name if requested. Warning: blocks geometry merge optimization!
-                if ( _featureNameExpr.isSet() )
+                osg::FloatArray* hats = new osg::FloatArray();
+                hats->setBinding(osg::Array::BIND_PER_VERTEX);
+                hats->setNormalize(false);
+                drawable->setVertexAttribArray(Clamping::HeightsAttrLocation, hats);
+                for (Geometry::const_iterator i = part->begin(); i != part->end(); ++i)
                 {
-                    const std::string& name = input->eval( _featureNameExpr.mutable_value(), &context );
-                    osgGeom->setName( name );
-                }
-
-                // subdivide the mesh if necessary to conform to an ECEF globe;
-                // but if the tessellation is set to zero, or if the style specifies a
-                // tessellation size, skip this step.
-                if ( makeECEF && !line->tessellation().isSetTo(0) && !line->tessellationSize().isSet() )
-                {
-                    double threshold = osg::DegreesToRadians( *_maxAngle_deg );
-                    OE_DEBUG << "Running mesh subdivider with threshold " << *_maxAngle_deg << std::endl;
-
-                    MeshSubdivider ms( _world2local, _local2world );
-                    //ms.setMaxElementsPerEBO( INT_MAX );
-                    if ( input->geoInterp().isSet() )
-                        ms.run( *osgGeom, threshold, *input->geoInterp() );
-                    else
-                        ms.run( *osgGeom, threshold, *_geoInterp );
-                }
-
-                // assign the primary color (PER_VERTEX required for later optimization)
-                osg::Vec4Array* colors = new osg::Vec4Array;
-                colors->assign( osgGeom->getVertexArray()->getNumElements(), primaryColor );
-                osgGeom->setColorArray( colors );
-                osgGeom->setColorBinding( osg::Geometry::BIND_PER_VERTEX );
-
-                geode->addDrawable( osgGeom );
-
-                // record the geometry's primitive set(s) in the index:
-                if ( context.featureIndex() )
-                    context.featureIndex()->tagDrawable( osgGeom.get(), input );
-
-                // install clamping attributes if necessary
-                if (_style.has<AltitudeSymbol>() &&
-                    _style.get<AltitudeSymbol>()->technique() == AltitudeSymbol::TECHNIQUE_GPU)
-                {
-                    Clamping::applyDefaultClampingAttrs( osgGeom.get(), input->getDouble("__oe_verticalOffset", 0.0) );
-                    Clamping::setHeights( osgGeom.get(), hats.get() );
+                    drawable->pushVertexAttrib(hats, i->z());
                 }
             }
+
+            // assign the color:
+            drawable->setColor(primaryColor);
+
+            // embed the feature name if requested. Warning: blocks geometry merge optimization!
+            if ( _featureNameExpr.isSet() )
+            {
+                const std::string& name = input->eval( _featureNameExpr.mutable_value(), &context );
+                drawable->setName( name );
+            }
+
+            // record the geometry's primitive set(s) in the index:
+            if ( context.featureIndex() )
+            {
+                context.featureIndex()->tagDrawable( drawable, input );
+            }
+
+            // install clamping attributes if necessary
+            if (doGpuClamping)
+            {
+                Clamping::applyDefaultClampingAttrs( drawable, input->getDouble("__oe_verticalOffset", 0.0) );
+            }
+
+            // finalize the drawable and generate primitive sets
+            drawable->dirty();
+
+            drawables->addChild(drawable);
         }
     }
 
-    if (_useGPULines == true)
+    // Finally, optimize the finished group for rendering.
+    if (drawables)
     {
-        GPULinesOperator op(masterStroke.get());
-        op.installShaders(geode);
+        drawables->optimize();
     }
 
-    return geode;
+    return drawables;
 }
 
 
 osg::Geode*
 BuildGeometryFilter::processPoints(FeatureList& features, FilterContext& context)
 {
-    osg::Geode* geode = new osg::Geode();
+    PointGroup* drawables = new PointGroup();
 
     bool makeECEF = false;
     const SpatialReference* featureSRS = 0L;
@@ -615,12 +588,16 @@ BuildGeometryFilter::processPoints(FeatureList& features, FilterContext& context
     // set up referencing information:
     if ( context.isGeoreferenced() )
     {
-        //makeECEF   = context.getSession()->getMapInfo().isGeocentric();
         featureSRS = context.extent()->getSRS();
         outputSRS  = context.getOutputSRS();
         makeECEF = outputSRS->isGeographic();
     }
 
+    // Need to know if we are GPU clamping so we can add more attribs
+    bool doGpuClamping =
+        _style.has<AltitudeSymbol>() &&
+        _style.get<AltitudeSymbol>()->technique() == AltitudeSymbol::TECHNIQUE_GPU;
+
     for( FeatureList::iterator f = features.begin(); f != features.end(); ++f )
     {
         Feature* input = f->get();
@@ -647,65 +624,82 @@ BuildGeometryFilter::processPoints(FeatureList& features, FilterContext& context
             // resolve the color:
             osg::Vec4f primaryColor = point->fill()->color();
 
-            osg::ref_ptr<osg::Geometry> osgGeom = new osg::Geometry();
-            osgGeom->setUseVertexBufferObjects( true );
-            osgGeom->setUseDisplayList( false );
+            // build the geometry:
+            osg::ref_ptr<osg::Vec3Array> allPoints = new osg::Vec3Array();
+            transformAndLocalize( part->asVector(), featureSRS, allPoints.get(), outputSRS, _world2local, makeECEF );
+
+            PointDrawable* drawable = new PointDrawable();
+
+            drawable->importVertexArray(allPoints.get());
+            
+            if (point->size().isSet())
+                drawable->setPointSize(point->size().get());
+
+            if (point->smooth().isSet())
+                drawable->setPointSmooth(point->smooth().get());
+
+            // For GPU clamping, we need an attribute array with Heights above Terrain in it.
+            if (doGpuClamping)
+            {
+                osg::FloatArray* hats = new osg::FloatArray();
+                hats->setBinding(osg::Array::BIND_PER_VERTEX);
+                hats->setNormalize(false);
+                drawable->setVertexAttribArray(Clamping::HeightsAttrLocation, hats);
+                for (Geometry::const_iterator i = part->begin(); i != part->end(); ++i)
+                {
+                    drawable->pushVertexAttrib(hats, i->z());
+                }
+            }
+
+            // assign the color:
+            drawable->setColor(primaryColor);            
 
             // embed the feature name if requested. Warning: blocks geometry merge optimization!
             if ( _featureNameExpr.isSet() )
             {
                 const std::string& name = input->eval( _featureNameExpr.mutable_value(), &context );
-                osgGeom->setName( name );
+                drawable->setName( name );
             }
 
-            // build the geometry:
-            osg::Vec3Array* allPoints = new osg::Vec3Array();
-
-            transformAndLocalize( part->asVector(), featureSRS, allPoints, outputSRS, _world2local, makeECEF );
-
-            osgGeom->addPrimitiveSet( new osg::DrawArrays(GL_POINTS, 0, allPoints->getNumElements()) );
-            osgGeom->setVertexArray( allPoints );
-
-            if ( input->style().isSet() )
-            {
-                //TODO: re-evaluate this. does it hinder geometry merging?
-                applyPointSymbology( osgGeom->getOrCreateStateSet(), point );
-            }
-
-            // assign the primary color (PER_VERTEX required for later optimization)
-            osg::Vec4Array* colors = new osg::Vec4Array;
-            colors->assign( osgGeom->getVertexArray()->getNumElements(), primaryColor );
-            osgGeom->setColorArray( colors );
-            osgGeom->setColorBinding( osg::Geometry::BIND_PER_VERTEX );
-
-            geode->addDrawable( osgGeom.get() );
-
             // record the geometry's primitive set(s) in the index:
             if ( context.featureIndex() )
-                context.featureIndex()->tagDrawable( osgGeom.get(), input );
+            {
+                context.featureIndex()->tagDrawable( drawable, input );
+            }
 
             // install clamping attributes if necessary
-            if (_style.has<AltitudeSymbol>() &&
-                _style.get<AltitudeSymbol>()->technique() == AltitudeSymbol::TECHNIQUE_GPU)
+            if (doGpuClamping)
             {
-                Clamping::applyDefaultClampingAttrs( osgGeom.get(), input->getDouble("__oe_verticalOffset", 0.0) );
-                Clamping::setHeights( osgGeom.get(), hats.get() );
+                Clamping::applyDefaultClampingAttrs( drawable, input->getDouble("__oe_verticalOffset", 0.0) );
             }
+
+            drawable->dirty();
+
+            drawables->addChild(drawable);
         }
     }
 
-    return geode;
+    // Finally, optimize the finished group for rendering.
+    if (drawables)
+    {
+        drawables->optimize();
+    }
+
+    return drawables;
 }
 
 // Borrowed from MeshConsolidator.cpp
-template<typename FROM, typename TO>
-osg::PrimitiveSet* copy( FROM* src, unsigned offset )
+namespace
 {
-    TO* newDE = new TO( src->getMode() );
-    newDE->reserve( src->size() );
-    for( typename FROM::const_iterator i = src->begin(); i != src->end(); ++i )
-        newDE->push_back( (*i) + offset );
-    return newDE;
+    template<typename FROM, typename TO>
+    osg::PrimitiveSet* copy( FROM* src, unsigned offset )
+    {
+        TO* newDE = new TO( src->getMode() );
+        newDE->reserve( src->size() );
+        for( typename FROM::const_iterator i = src->begin(); i != src->end(); ++i )
+            newDE->push_back( (*i) + offset );
+        return newDE;
+    }
 }
 
 
@@ -1231,7 +1225,7 @@ namespace
             // calc orientation of triangle.
             osg::Vec3 normal = (v2 - v1) ^ (v3 - v1);
             normal.normalize();
-            
+
             (*_normals)[i1] += normal;
             (*_normals)[i2] += normal;
             (*_normals)[i3] += normal;
@@ -1258,12 +1252,12 @@ namespace
         {
             osg::Geometry* geom = drawable.asGeometry();
             if (geom)
-            {                
+            {
                 osg::Vec3Array* verts = dynamic_cast<osg::Vec3Array*>(geom->getVertexArray());
 
                 osg::Vec3Array* normals = new osg::Vec3Array(verts->size());
                 normals->setBinding(normals->BIND_PER_VERTEX);
-                
+
                 osg::TriangleIndexFunctor<GenerateNormalFunctor> f;
                 f.set(verts, normals);
                 geom->accept(f);
@@ -1440,39 +1434,28 @@ BuildGeometryFilter::push( FeatureList& input, FilterContext& context )
     if ( lines.size() > 0 )
     {
         OE_TEST << LC << "Building " << lines.size() << " lines." << std::endl;
-        
-        osg::ref_ptr<osg::Geode> geode = processLines(lines, context);
-        if ( geode->getNumDrawables() > 0 )
+
+        osg::ref_ptr<osg::Group> group = processLines(lines, context);
+
+        if ( group->getNumChildren() > 0 )
         {
-            osgUtil::Optimizer::MergeGeometryVisitor mg;
-            mg.setTargetMaximumNumberOfVertices(65536);
-            geode->accept(mg);
-
-            if (_useGPULines == false)
-            {
-                applyLineSymbology( geode->getOrCreateStateSet(), line );
-            }
-
-            result->addChild( geode.get() );
+            result->addChild(group.get());
         }
     }
 
     if ( points.size() > 0 )
     {
         OE_TEST << LC << "Building " << points.size() << " points." << std::endl;
-        osg::ref_ptr<osg::Geode> geode = processPoints(points, context);
-        if ( geode->getNumDrawables() > 0 )
-        {
-            osgUtil::Optimizer::MergeGeometryVisitor mg;
-            mg.setTargetMaximumNumberOfVertices(65536);
-            geode->accept(mg);
 
-            applyPointSymbology( geode->getOrCreateStateSet(), point );
-            result->addChild( geode.get() );
+        osg::ref_ptr<osg::Group> group = processPoints(points, context);
+
+        if ( group->getNumChildren() > 0 )
+        {
+            result->addChild(group.get());
         }
     }
 
-    // indicate that geometry contains clamping attributes
+    //// indicate that geometry contains clamping attributes
     if (_style.has<AltitudeSymbol>() &&
         _style.get<AltitudeSymbol>()->technique() == AltitudeSymbol::TECHNIQUE_GPU)
     {
