@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
- * Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+ * Copyright 2018 Pelican Mapping
  * http://osgearth.org
  *
  * osgEarth is free software; you can redistribute it and/or modify
@@ -95,6 +95,11 @@ FeatureProfile::setProfile( const osgEarth::Profile* profile )
 std::string
 AttributeValue::getString() const
 {
+    if (!second.set)
+    {
+        return "";
+    }
+
     switch( first ) {
         case ATTRTYPE_STRING: return second.stringValue;
         case ATTRTYPE_DOUBLE: return osgEarth::toString(second.doubleValue);
@@ -108,6 +113,11 @@ AttributeValue::getString() const
 double
 AttributeValue::getDouble( double defaultValue ) const 
 {
+    if (!second.set)
+    {
+        return defaultValue;
+    }
+
     switch( first ) {
         case ATTRTYPE_STRING: return osgEarth::as<double>(second.stringValue, defaultValue);
         case ATTRTYPE_DOUBLE: return second.doubleValue;
@@ -121,6 +131,11 @@ AttributeValue::getDouble( double defaultValue ) const
 int
 AttributeValue::getInt( int defaultValue ) const 
 {
+    if (!second.set)
+    {
+        return defaultValue;
+    }
+
     switch( first ) {
         case ATTRTYPE_STRING: return osgEarth::as<int>(second.stringValue, defaultValue);
         case ATTRTYPE_DOUBLE: return (int)second.doubleValue;
@@ -134,6 +149,11 @@ AttributeValue::getInt( int defaultValue ) const
 bool
 AttributeValue::getBool( bool defaultValue ) const 
 {
+    if (!second.set)
+    {
+        return defaultValue;
+    }
+
     switch( first ) {
         case ATTRTYPE_STRING: return osgEarth::as<bool>(second.stringValue, defaultValue);
         case ATTRTYPE_DOUBLE: return second.doubleValue != 0.0;
@@ -531,13 +551,13 @@ bool Feature::getWorldBoundingPolytope( const osg::BoundingSphered& bs, const Sp
 
         // for a projected feature, we're done. For a geocentric one, transform the polytope
         // into world (ECEF) space.
-        if ( srs->isGeographic() && !srs->isPlateCarre() )
+        if ( srs->isGeographic() )
         {
             const osg::EllipsoidModel* e = srs->getEllipsoid();
 
             // add a bottom cap, unless the bounds are sufficiently large.
-            double minRad = std::min(e->getRadiusPolar(), e->getRadiusEquator());
-            double maxRad = std::max(e->getRadiusPolar(), e->getRadiusEquator());
+            double minRad = osg::minimum(e->getRadiusPolar(), e->getRadiusEquator());
+            double maxRad = osg::maximum(e->getRadiusPolar(), e->getRadiusEquator());
             double zeroOffset = bs.center().length();
             if ( zeroOffset > minRad * 0.1 )
             {
