@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
- * Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+ * Copyright 2018 Pelican Mapping
  * http://osgearth.org
  *
  * osgEarth is free software; you can redistribute it and/or modify
@@ -36,12 +36,26 @@
 
 using namespace osgEarth::SilverLining;
 
-SilverLiningNode::SilverLiningNode(const osgEarth::SpatialReference*    mapSRS,
+SilverLiningNode::SilverLiningNode(const SilverLiningOptions& options,
+                                   Callback*                  callback) :
+_options(options),
+_callback(callback)
+{
+    construct();
+}
+
+// DEPRECATED
+SilverLiningNode::SilverLiningNode(const osgEarth::SpatialReference* mapSRS,
                                    const SilverLiningOptions& options,
                                    Callback*                  callback) :
 _options(options),
-_mapSRS(mapSRS),
 _callback(callback)
+{
+    construct();
+}
+
+void
+SilverLiningNode::construct()
 {
     // Create a new Light for the Sun.
     _light = new LightGL3(0);
@@ -66,7 +80,6 @@ _callback(callback)
     ADJUST_UPDATE_TRAV_COUNT(this, +1);
 }
 
-
 SilverLiningNode::~SilverLiningNode()
 {
     if ( _lighting.valid() )
@@ -172,7 +185,7 @@ SilverLiningNode::traverse(osg::NodeVisitor& nv)
             {
                 for (CameraSet::const_iterator i = _camerasToAdd.begin(); i != _camerasToAdd.end(); ++i)
                 {
-                    SilverLiningContextNode* newNode = new SilverLiningContextNode(this, i->get(), _light.get(), _mapSRS, _options, _callback.get());
+                    SilverLiningContextNode* newNode = new SilverLiningContextNode(this, i->get(), _light.get(), _options, _callback.get());
                     _contexts[i->get()] = newNode;
                     _contextList.push_back(newNode);
                 }
