@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
-* Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+* Copyright 2018 Pelican Mapping
 * http://osgearth.org
 *
 * osgEarth is free software; you can redistribute it and/or modify
@@ -37,7 +37,7 @@ using namespace geos;
 using namespace geos::operation;
 #endif
 
-#define GEOS_OUT OE_INFO
+#define GEOS_OUT OE_DEBUG
 
 #define LC "[Geometry] "
 
@@ -587,7 +587,7 @@ Geometry::getOrientation() const
 {
     // adjust for a non-open ring:
     int n = size();
-    while( n > 0 && front() == back() )
+    if ( n > 0 && front() == back() )
         n--;
 
     if ( n < 3 )
