@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
- * Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+ * Copyright 2018 Pelican Mapping
  * http://osgearth.org
  *
  * osgEarth is free software; you can redistribute it and/or modify
@@ -55,7 +55,7 @@ ResourceLibrary::mergeConfig( const Config& conf )
     if (_name.empty())
         _name = conf.value( "name" );
 
-    conf.getIfSet( "url", _uri );
+    conf.get( "url", _uri );
 
     for( ConfigSet::const_iterator i = conf.children().begin(); i != conf.children().end(); ++i )
     {
@@ -64,11 +64,6 @@ ResourceLibrary::mergeConfig( const Config& conf )
         {
             addResource( new SkinResource(child) );
         }
-        else if ( child.key() == "marker" )
-        {
-            // to be decrepated
-            addResource( new MarkerResource(child) );
-        }
         else if ( child.key() == "model" )
         {
             addResource( new ModelResource(child) );
@@ -94,7 +89,7 @@ ResourceLibrary::getConfig() const
 
         if ( _uri.isSet() )
         {
-            conf.addIfSet( "url", _uri );
+            conf.set( "url", _uri );
         }
         else
         {
@@ -104,12 +99,6 @@ ResourceLibrary::getConfig() const
                 conf.add( res->getConfig() );
             }
 
-            for( ResourceMap<MarkerResource>::const_iterator i = _markers.begin(); i != _markers.end(); ++i )
-            {
-                MarkerResource* res = i->second.get();
-                conf.add( res->getConfig() );
-            }
-
             for( ResourceMap<InstanceResource>::const_iterator i = _instances.begin(); i != _instances.end(); ++i )
             {
                 InstanceResource* res = i->second.get();
@@ -128,11 +117,6 @@ ResourceLibrary::addResource( Resource* resource )
         Threading::ScopedWriteLock exclusive(_mutex);
         _skins[resource->name()] = static_cast<SkinResource*>(resource);
     }
-    else if ( dynamic_cast<MarkerResource*>(resource) )
-    {
-        Threading::ScopedWriteLock exclusive(_mutex);
-        _markers[resource->name()] = static_cast<MarkerResource*>(resource);
-    }
     else if ( dynamic_cast<InstanceResource*>(resource) )
     {
         Threading::ScopedWriteLock exclusive(_mutex);
@@ -152,11 +136,6 @@ ResourceLibrary::removeResource( Resource* resource )
         Threading::ScopedWriteLock exclusive(_mutex);
         _skins.erase( resource->name() );
     }
-    else if ( dynamic_cast<MarkerResource*>( resource ) )
-    {
-        Threading::ScopedWriteLock exclusive(_mutex);
-        _markers.erase( resource->name() );
-    }
     else if ( dynamic_cast<InstanceResource*>( resource ) )
     {
         Threading::ScopedWriteLock exclusive(_mutex);
@@ -325,28 +304,6 @@ ResourceLibrary::matches( const SkinSymbol* q, SkinResource* s ) const
     return true;
 }
 
-MarkerResource*
-ResourceLibrary::getMarker( const std::string& name, const osgDB::Options* dbOptions ) const
-{
-    const_cast<ResourceLibrary*>(this)->initialize( dbOptions );
-    Threading::ScopedReadLock shared( _mutex );
-    ResourceMap<MarkerResource>::const_iterator i = _markers.find( name );
-    return i != _markers.end() ? i->second.get() : 0L;
-}
-
-void
-ResourceLibrary::getMarkers( MarkerResourceVector& output, const osgDB::Options* dbOptions ) const
-{
-    const_cast<ResourceLibrary*>(this)->initialize( dbOptions );
-    Threading::ScopedReadLock shared( _mutex );
-
-    output.clear();
-    output.reserve( _markers.size() );
-
-    for( ResourceMap<MarkerResource>::const_iterator i = _markers.begin(); i != _markers.end(); ++i )
-        output.push_back( i->second.get() );
-}
-
 InstanceResource*
 ResourceLibrary::getInstance( const std::string& name, const osgDB::Options* dbOptions ) const
 {
