@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
-* Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+* Copyright 2018 Pelican Mapping
 * http://osgearth.org
 *
 * osgEarth is free software; you can redistribute it and/or modify
@@ -21,39 +21,24 @@
 */
 
 #include <osgEarthUtil/ExampleResources>
-#include <osgEarthUtil/EarthManipulator>
 #include <osgEarthUtil/LatLongFormatter>
 #include <osgEarthUtil/MGRSFormatter>
 #include <osgEarthUtil/MouseCoordsTool>
-#include <osgEarthUtil/AutoClipPlaneHandler>
-#include <osgEarthUtil/DataScanner>
-#include <osgEarthUtil/Sky>
-#include <osgEarthUtil/Ocean>
 #include <osgEarthUtil/Shadowing>
 #include <osgEarthUtil/ActivityMonitorTool>
 #include <osgEarthUtil/LogarithmicDepthBuffer>
-#include <osgEarthUtil/ContourMap>
-
-#include <osgEarthUtil/LODBlending>
-#include <osgEarthUtil/VerticalScale>
+#include <osgEarthUtil/SimpleOceanLayer>
 
 #include <osgEarthAnnotation/AnnotationData>
-#include <osgEarthAnnotation/AnnotationRegistry>
-#include <osgEarth/ScreenSpaceLayout>
 #include <osgEarth/TerrainEngineNode>
 #include <osgEarth/NodeUtils>
-#include <osgEarth/FileUtils>
+#include <osgEarth/GLUtils>
 
-#include <osgEarth/XmlUtils>
-#include <osgEarth/StringUtils>
 
 #include <osgEarthDrivers/kml/KML>
 
-#include <osgDB/FileNameUtils>
 #include <osgDB/WriteFile>
 #include <osgGA/StateSetManipulator>
-#include <osgGA/AnimationPathManipulator>
-#include <osgViewer/View>
 #include <osgViewer/ViewerEventHandlers>
 
 #define KML_PUSHPIN_URL "../data/placemark32.png"
@@ -88,7 +73,7 @@ namespace
     // flies to a viewpoint in response to control event (click)
     struct ClickViewpointHandler : public ControlEventHandler
     {
-        ClickViewpointHandler( const Viewpoint& vp, osgGA::CameraManipulator* manip ) 
+        ClickViewpointHandler( const Viewpoint& vp, osgGA::CameraManipulator* manip )
             : _vp(vp), _manip( dynamic_cast<EarthManipulator*>(manip) ) { }
 
         Viewpoint         _vp;
@@ -153,8 +138,22 @@ namespace
     {
         osg::ref_ptr<osg::Uniform> _u;
         ApplyValueUniform(osg::Uniform* u) :_u(u) { }
-        void onValueChanged(Control* c, double value) {
+        void onValueChanged(Control* c, double value)
+        {
             _u->set( float(value) );
+            osgEarth::Registry::instance()->dataStore().store(
+                osgEarth::Registry::instance(), _u->getName(), _u.get());
+        }
+    };
+
+    struct ToggleDefine : public ControlEventHandler
+    {
+        osg::ref_ptr<osg::StateSet> _ss;
+        std::string _name;
+        ToggleDefine(osg::StateSet* ss, const std::string& name) : _ss(ss), _name(name) { }
+        void onValueChanged(Control* c, bool value) {
+            if (value) _ss->setDefine(_name, osg::StateAttribute::ON | osg::StateAttribute::OVERRIDE);
+            else _ss->setDefine(_name, osg::StateAttribute::OFF | osg::StateAttribute::OVERRIDE); //(_name);
         }
     };
 }
@@ -166,7 +165,7 @@ Control*
 MouseCoordsControlFactory::create(MapNode*         mapNode,
                                   osgViewer::View* view     ) const
 {
-    // readout for coordinates under the mouse   
+    // readout for coordinates under the mouse
     LabelControl* readout = new LabelControl();
     readout->setHorizAlign( Control::ALIGN_RIGHT );
     readout->setVertAlign( Control::ALIGN_BOTTOM );
@@ -245,21 +244,68 @@ AnnotationGraphControlFactory::create(osg::Node*       graph,
 
 //------------------------------------------------------------------------
 
+std::string getAttributionString(Map* map)
+{
+    StringSet attributions;
+    map->getAttributions(attributions);
+    std::stringstream buf;
+    for (StringSet::iterator itr = attributions.begin(); itr != attributions.end(); ++itr)
+    {
+        buf << *itr << std::endl;
+    }
+
+    return buf.str();
+}
+
+class AttributeCallback : public osgEarth::MapCallback
+{
+public:
+    AttributeCallback(LabelControl* label, MapNode* mapNode) :
+        _label(label),
+        _mapNode(mapNode)
+    {
+    }
+
+    virtual void onMapModelChanged(const MapModelChange& change)
+    {
+        osg::ref_ptr< LabelControl > label;
+        osg::ref_ptr< MapNode > mapNode;
+
+        StringVector attributions;
+        if (_mapNode.lock(mapNode) && _label.lock(label))
+        {
+            label->setText(getAttributionString(_mapNode->getMap()));
+        }
+    }
+
+    osg::observer_ptr< LabelControl > _label;
+    osg::observer_ptr< MapNode > _mapNode;
+};
+
+Control* AttributionControlFactory::create(MapNode* mapNode) const
+{
+    // readout for coordinates under the mouse
+    LabelControl* credits = new LabelControl();
+    credits->setHorizAlign(Control::ALIGN_CENTER);
+    credits->setVertAlign(Control::ALIGN_BOTTOM);
+    credits->setText(getAttributionString(mapNode->getMap()));
+    mapNode->getMap()->addMapCallback(new AttributeCallback(credits, mapNode));
+    return credits;
+}
+
+//------------------------------------------------------------------------
+
 #undef  LC
 #define LC "[MapNodeHelper] "
 
 osg::Group*
-MapNodeHelper::load(osg::ArgumentParser&  args,
-                    osgViewer::View*      view,
-                    Container*            userContainer,
-                    const osgDB::Options* readOptions) const
+MapNodeHelper::load(osg::ArgumentParser&   args,
+                    osgViewer::ViewerBase* viewer,
+                    Container*             userContainer,
+                    const osgDB::Options*  readOptions) const
 {
-    // do this first before scanning for an earth file
-    std::string outEarth;
-    args.read( "--out-earth", outEarth );
-
     osg::ref_ptr<osgDB::Options> myReadOptions = Registry::cloneOrCreateOptions(readOptions);
-    
+
     Config c;
     c.add("elevation_smoothing", false);
     TerrainOptions to(c);
@@ -272,33 +318,24 @@ MapNodeHelper::load(osg::ArgumentParser&  args,
     // read in the Earth file:
     osg::ref_ptr<osg::Node> node = osgDB::readNodeFiles(args, myReadOptions.get());
 
-    osg::ref_ptr<MapNode> mapNode;
-    if ( !node )
+    osg::ref_ptr<MapNode> mapNode = MapNode::get(node.get());
+    if ( !mapNode.valid() )
     {
-        if ( args.find("--images") < 0 )
-        {
-            OE_WARN << LC << "No earth file." << std::endl;
-            return 0L;
-        }
-        else
-        {
-            mapNode = new MapNode();
-        }
+        OE_WARN << LC << "Loaded scene graph does not contain a MapNode - aborting" << std::endl;
+        return 0L;
     }
-    else
+
+    // collect the views
+    osgViewer::Viewer::Views views;
+    if (viewer)
     {
-        mapNode = MapNode::get(node.get());
-        if ( !mapNode.valid() )
-        {
-            OE_WARN << LC << "Loaded scene graph does not contain a MapNode - aborting" << std::endl;
-            return 0L;
-        }
+        viewer->getViews(views);
     }
 
     // warn about not having an earth manip
-    if ( view )
+    for (osgViewer::Viewer::Views::iterator view = views.begin(); view != views.end(); ++view)
     {
-        EarthManipulator* manip = dynamic_cast<EarthManipulator*>(view->getCameraManipulator());
+        EarthManipulator* manip = dynamic_cast<EarthManipulator*>((*view)->getCameraManipulator());
         if ( manip == 0L )
         {
             OE_WARN << LC << "Helper used before installing an EarthManipulator" << std::endl;
@@ -307,28 +344,28 @@ MapNodeHelper::load(osg::ArgumentParser&  args,
 
     // a root node to hold everything:
     osg::Group* root = new osg::Group();
-    
+
     root->addChild( node );
 
-    // parses common cmdline arguments.
-    if ( view )
+    // parses common cmdline arguments and apply to the first view:
+    if ( !views.empty() )
     {
-        parse( mapNode.get(), args, view, root, userContainer );
+        parse( mapNode.get(), args, views.front(), root, userContainer );
     }
 
-    // Dump out an earth file if so directed.
-    if ( !outEarth.empty() )
+    // configures each view with some stock goodies
+    for (osgViewer::Viewer::Views::iterator view = views.begin(); view != views.end(); ++view)
     {
-        OE_NOTICE << LC << "Writing earth file: " << outEarth << std::endl;
-        osgDB::writeNodeFile( *mapNode, outEarth );
+        configureView( *view );
     }
 
-    // configures the viewer with some stock goodies
-    if ( view )
+#ifdef OSG_GL3_AVAILABLE
+    if (viewer)
     {
-        configureView( view );
+        viewer->setRealizeOperation(new GL3RealizeOperation());
     }
-    
+#endif
+
     return root;
 }
 
@@ -364,12 +401,7 @@ MapNodeHelper::parse(MapNode*             mapNode,
     osg::ref_ptr<osgDB::Options> dbOptions = Registry::instance()->cloneOrCreateOptions();
 
     // parse out custom example arguments first:
-    bool useMGRS       = args.read("--mgrs");
-    bool useDMS        = args.read("--dms");
-    bool useDD         = args.read("--dd");
-    bool useCoords     = args.read("--coords") || useMGRS || useDMS || useDD;
-
-    bool useAutoClip   = args.read("--autoclip");
+    bool useCoords     = args.read("--coords");
     bool showActivity  = args.read("--activity");
     bool useLogDepth   = args.read("--logdepth");
     bool useLogDepth2  = args.read("--logdepth2");
@@ -378,12 +410,6 @@ MapNodeHelper::parse(MapNode*             mapNode,
     std::string kmlFile;
     args.read( "--kml", kmlFile );
 
-    std::string imageFolder;
-    args.read( "--images", imageFolder );
-
-    std::string imageExtensions;
-    args.read("--image-extensions", imageExtensions);
-    
     // animation path:
     std::string animpath;
     if ( args.read("--path", animpath) )
@@ -391,6 +417,15 @@ MapNodeHelper::parse(MapNode*             mapNode,
         view->setCameraManipulator( new osgGA::AnimationPathManipulator(animpath) );
     }
 
+    // vertical field of view:
+    float vfov = -1.0f;
+    if (args.read("--vfov", vfov) && vfov > 0.0f)
+    {
+        double fov, ar, n, f;
+        view->getCamera()->getProjectionMatrixAsPerspective(fov, ar, n, f);
+        view->getCamera()->setProjectionMatrixAsPerspective(vfov, ar, n, f);
+    }
+
     // Install a new Canvas for our UI controls, or use one that already exists.
     ControlCanvas* canvas = ControlCanvas::getOrCreate( view );
 
@@ -402,7 +437,7 @@ MapNodeHelper::parse(MapNode*             mapNode,
     else
     {
         mainContainer = new VBox();
-        mainContainer->setAbsorbEvents( true );
+        mainContainer->setAbsorbEvents( false );
         mainContainer->setBackColor( Color(Color::Black, 0.8) );
         mainContainer->setHorizAlign( Control::ALIGN_LEFT );
         mainContainer->setVertAlign( Control::ALIGN_BOTTOM );
@@ -415,10 +450,6 @@ MapNodeHelper::parse(MapNode*             mapNode,
     // look for external data in the map node:
     const Config& externals = mapNode->externalConfig();
 
-    // some terrain effects.
-    // TODO: Most of these are likely to move into extensions.
-    const Config& vertScaleConf   = externals.child("vertical_scale");
-
     // Loading KML from the command line:
     if ( !kmlFile.empty() )
     {
@@ -443,10 +474,10 @@ MapNodeHelper::parse(MapNode*             mapNode,
                 if ( c )
                 {
                     c->setVertAlign( Control::ALIGN_TOP );
-                    canvas->addControl( c );
+                    mainContainer->addControl( c );
                 }
             }
-            root->addChild( kml );
+            mapNode->addChild( kml );
         }
         else
         {
@@ -454,33 +485,25 @@ MapNodeHelper::parse(MapNode*             mapNode,
         }
     }
 
-    //// Configure the de-cluttering engine for labels and annotations:
-    //if ( !screenSpaceLayoutConf.empty() )
-    //{
-    //    ScreenSpaceLayout::setOptions( ScreenSpaceLayoutOptions(screenSpaceLayoutConf) );
-    //}
-
     // Configure the mouse coordinate readout:
     if ( useCoords )
-    { 
+    {
         LabelControl* readout = new LabelControl();
         readout->setBackColor( Color(Color::Black, 0.8) );
         readout->setHorizAlign( Control::ALIGN_RIGHT );
         readout->setVertAlign( Control::ALIGN_BOTTOM );
 
-        Formatter* formatter = 
-            useMGRS ? (Formatter*)new MGRSFormatter(MGRSFormatter::PRECISION_1M, 0L, MGRSFormatter::USE_SPACES) :
-            useDMS  ? (Formatter*)new LatLongFormatter(LatLongFormatter::FORMAT_DEGREES_MINUTES_SECONDS) :
-            useDD   ? (Formatter*)new LatLongFormatter(LatLongFormatter::FORMAT_DECIMAL_DEGREES) :
-            0L;
-
         MouseCoordsTool* mcTool = new MouseCoordsTool( mapNode );
-        mcTool->addCallback( new MouseCoordsLabelCallback(readout, formatter) );
+        mcTool->addCallback( new MouseCoordsLabelCallback(readout) );
         view->addEventHandler( mcTool );
 
         canvas->addControl( readout );
     }
 
+
+    // Add the credits display
+    canvas->addControl(AttributionControlFactory().create(mapNode));
+
     // Configure for an ortho camera:
     if ( args.read("--ortho") )
     {
@@ -506,12 +529,6 @@ MapNodeHelper::parse(MapNode*             mapNode,
         canvas->addControl( vbox );
     }
 
-    // Install an auto clip plane clamper
-    if ( useAutoClip )
-    {
-        mapNode->addCullCallback( new AutoClipPlaneCullCallback(mapNode) );
-    }
-
     // Install logarithmic depth buffer on main camera
     if ( useLogDepth )
     {
@@ -529,39 +546,6 @@ MapNodeHelper::parse(MapNode*             mapNode,
         logDepth.install( view->getCamera() );
     }
 
-    // Scan for images if necessary.
-    if ( !imageFolder.empty() )
-    {
-        std::vector<std::string> extensions;
-        if ( !imageExtensions.empty() )
-            StringTokenizer( imageExtensions, extensions, ",;", "", false, true );
-        if ( extensions.empty() )
-            extensions.push_back( "tif" );
-
-        OE_INFO << LC << "Loading images from " << imageFolder << "..." << std::endl;
-        ImageLayerVector imageLayers;
-        DataScanner scanner;
-        scanner.findImageLayers( imageFolder, extensions, imageLayers );
-
-        if ( imageLayers.size() > 0 )
-        {
-            mapNode->getMap()->beginUpdate();
-            for( ImageLayerVector::iterator i = imageLayers.begin(); i != imageLayers.end(); ++i )
-            {
-                mapNode->getMap()->addLayer( i->get() );
-            }
-            mapNode->getMap()->endUpdate();
-        }
-        OE_INFO << LC << "...found " << imageLayers.size() << " image layers." << std::endl;
-    }
-
-    // Install vertical scaler.
-    // TODO: deprecate this, or move it to an extension.
-    if ( !vertScaleConf.empty() )
-    {
-        mapNode->getTerrainEngine()->addEffect( new VerticalScale(vertScaleConf) );
-    }
-
     // Generic named value uniform with min/max.
     VBox* uniformBox = 0L;
     while( args.find( "--uniform" ) >= 0 )
@@ -575,7 +559,7 @@ MapNodeHelper::parse(MapNode*             mapNode,
                 uniformBox = new VBox();
                 uniformBox->setBackColor(0,0,0,0.5);
                 uniformBox->setAbsorbEvents( true );
-                canvas->addControl( uniformBox );
+                mainContainer->addControl( uniformBox );
             }
             osg::Uniform* uniform = new osg::Uniform(osg::Uniform::FLOAT, name);
             uniform->set( minval );
@@ -590,6 +574,26 @@ MapNodeHelper::parse(MapNode*             mapNode,
         }
     }
 
+    while (args.find("--define") >= 0)
+    {
+        std::string name;
+        if (args.read("--define", name))
+        {
+            if ( uniformBox == 0L )
+            {
+                uniformBox = new VBox();
+                uniformBox->setBackColor(0,0,0,0.5);
+                uniformBox->setAbsorbEvents( true );
+                mainContainer->addControl( uniformBox );
+            }
+            
+            HBox* box = new HBox();
+            box->addControl(new CheckBoxControl(false, new ToggleDefine(mapNode->getOrCreateStateSet(), name)));
+            box->addControl(new LabelControl(name));
+            uniformBox->addControl(box);
+        }
+    }
+
     // Map inspector:
     if (args.read("--inspect"))
     {
@@ -612,7 +616,11 @@ MapNodeHelper::parse(MapNode*             mapNode,
     // Simple ocean model:
     if (args.read("--ocean"))
     {
-        mapNode->addExtension(Extension::create("ocean_simple", ConfigOptions()));
+        //mapNode->addExtension(Extension::create("ocean_simple", ConfigOptions()));
+        SimpleOceanLayer* layer = new SimpleOceanLayer();
+        mapNode->getMap()->addLayer(layer);
+        Control* ui = OceanControlFactory::create(layer);
+        mainContainer->addControl(ui);
     }
 
     // Arbitrary extension:
@@ -623,7 +631,7 @@ MapNodeHelper::parse(MapNode*             mapNode,
         if (ext)
             mapNode->addExtension(ext);
     }
-    
+
 
     // Hook up the extensions!
     for(std::vector<osg::ref_ptr<Extension> >::const_iterator eiter = mapNode->getExtensions().begin();
@@ -654,7 +662,7 @@ MapNodeHelper::parse(MapNode*             mapNode,
             ShadowCaster* caster = new ShadowCaster();
             caster->setTextureImageUnit( unit );
             caster->setLight( view->getLight() );
-            caster->getShadowCastingGroup()->addChild( mapNode->getModelLayerGroup() );
+            caster->getShadowCastingGroup()->addChild( mapNode->getLayerNodeGroup() );
             caster->getShadowCastingGroup()->addChild(mapNode->getTerrainEngine());
             if ( mapNode->getNumParents() > 0 )
             {
@@ -675,6 +683,9 @@ MapNodeHelper::parse(MapNode*             mapNode,
 void
 MapNodeHelper::configureView( osgViewer::View* view ) const
 {
+    // default uniform values:
+    GLUtils::setGlobalDefaults(view->getCamera()->getOrCreateStateSet());
+
     // add some stock OSG handlers:
     view->addEventHandler(new osgViewer::StatsHandler());
     view->addEventHandler(new osgViewer::WindowSizeHandler());
@@ -694,19 +705,16 @@ MapNodeHelper::usage() const
         << "  --kml <file.kml>              : load a KML or KMZ file\n"
         << "  --kmlui                       : display a UI for toggling nodes loaded with --kml\n"
         << "  --coords                      : display map coords under mouse\n"
-        << "  --dms                         : dispay deg/min/sec coords under mouse\n"
-        << "  --dd                          : display decimal degrees coords under mouse\n"
-        << "  --mgrs                        : show MGRS coords under mouse\n"
         << "  --ortho                       : use an orthographic camera\n"
         << "  --logdepth                    : activates the logarithmic depth buffer\n"
         << "  --logdepth2                   : activates logarithmic depth buffer with per-fragment interpolation\n"
         << "  --shadows                     : activates model layer shadows\n"
-        << "  --images [path]               : finds and loads image layers from folder [path]\n"
-        << "  --image-extensions [ext,...]  : with --images, extensions to use\n"
         << "  --out-earth [file]            : write the loaded map to an earth file\n"
         << "  --uniform [name] [min] [max]  : create a uniform controller with min/max values\n"
+        << "  --define [name]               : install a shader #define\n"
         << "  --path [file]                 : load and playback an animation path\n"
-        << "  --extension [name]            : loads a named extension\n";
+        << "  --extension [name]            : loads a named extension\n"
+        << "  --ocean                       : add a simple ocean model (requires bathymetry)\n";
 }
 
 
@@ -725,7 +733,7 @@ namespace
             _sky->setDateTime(DateTime(d.year(), d.month(), d.day(), value));
         }
     };
-    
+
     static std::string s_month[12] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
 
     struct SkyMonthSlider : public ui::ControlEventHandler
@@ -735,8 +743,8 @@ namespace
         ui::LabelControl* _label;
         void onValueChanged(ui::Control* control, float value )
         {
-            int m = std::min((int)value, 11);
-            DateTime d = _sky->getDateTime();            
+            int m = osg::minimum((int)value, 11);
+            DateTime d = _sky->getDateTime();
             _sky->setDateTime(DateTime(d.year(), m, d.day(), d.hours()));
             _label->setText(s_month[m]);
         }
@@ -749,7 +757,7 @@ namespace
         ui::LabelControl* _label;
         void onValueChanged(ui::Control* control, float value )
         {
-            DateTime d = _sky->getDateTime();            
+            DateTime d = _sky->getDateTime();
             _sky->setDateTime(DateTime((int)value, d.month(), d.day(), d.hours()));
             _label->setText(Stringify() << (int)value);
         }
@@ -763,7 +771,8 @@ namespace
 
         void onValueChanged(ui::Control* control, float value )
         {
-            _sky->setMinimumAmbient(osg::Vec4(value,value,value,1));
+            if (_sky->getSunLight())
+                _sky->getSunLight()->setAmbient(osg::Vec4(value,value,value,1));
         }
     };
 }
@@ -771,9 +780,9 @@ namespace
 ui::Control* SkyControlFactory::create(SkyNode* sky)
 {
     ui::Grid* grid = new ui::Grid();
+    grid->setBackColor(0,0,0,.1);
     grid->setChildVertAlign( ui::Control::ALIGN_CENTER );
     grid->setChildSpacing( 10 );
-    grid->setHorizFill( true );
 
     if (sky)
     {
@@ -785,14 +794,14 @@ ui::Control* SkyControlFactory::create(SkyNode* sky)
         skyHoursSlider->setHorizFill( true, 250 );
         skyHoursSlider->addEventHandler( new SkyHoursSlider(sky) );
         grid->setControl(2, r, new ui::LabelControl(skyHoursSlider) );
-    
+
         ++r;
         grid->setControl( 0, r, new ui::LabelControl("Month: ", 16) );
         ui::HSliderControl* skyMonthSlider = grid->setControl(1, r, new ui::HSliderControl( 0.0f, 12.0f, dt.month() ));
         skyMonthSlider->setHorizFill( true, 250 );
-        ui::LabelControl* monthLabel = grid->setControl(2, r, new ui::LabelControl(s_month[dt.month()]));
+        ui::LabelControl* monthLabel = grid->setControl(2, r, new ui::LabelControl(s_month[dt.month()-1]));
         skyMonthSlider->addEventHandler( new SkyMonthSlider(sky, monthLabel) );
-    
+
         ++r;
         grid->setControl( 0, r, new ui::LabelControl("Year: ", 16) );
         ui::HSliderControl* skyYearSlider = grid->setControl(1, r, new ui::HSliderControl( 1970.0f, 2061.0f, dt.year() ));
@@ -809,3 +818,40 @@ ui::Control* SkyControlFactory::create(SkyNode* sky)
 
     return grid;
 }
+
+//........................................................................
+
+
+namespace
+{
+    struct OceanSeaLevel : public ui::ControlEventHandler
+    {
+        OceanSeaLevel(SimpleOceanLayer* ocean) : _ocean(ocean) { }
+        SimpleOceanLayer* _ocean;
+        void onValueChanged(ui::Control* control, float value )
+        {
+            _ocean->setSeaLevel(value);
+        }
+    };
+}
+
+ui::Control*
+OceanControlFactory::create(SimpleOceanLayer* ocean)
+{
+    ui::Grid* grid = new ui::Grid();
+    grid->setBackColor(0,0,0,.1);
+    grid->setChildVertAlign( ui::Control::ALIGN_CENTER );
+    grid->setChildSpacing( 10 );
+
+    if (ocean)
+    {
+        int r=0;
+
+        grid->setControl( 0, r, new ui::LabelControl("Sea Level: ", 16) );
+        ui::HSliderControl* seaLevel = grid->setControl(1, r, new ui::HSliderControl(-250.0f, 250.0f, 0.0f, new OceanSeaLevel(ocean)));
+        seaLevel->setHorizFill( true, 250 );
+        grid->setControl(2, r, new ui::LabelControl(seaLevel) );
+    }
+
+    return grid;
+}
