@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
- * Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+ * Copyright 2018 Pelican Mapping
  * http://osgearth.org
  *
  * osgEarth is free software; you can redistribute it and/or modify
@@ -19,13 +19,7 @@
 #include <osgEarthUtil/FlatteningLayer>
 #include <osgEarth/Registry>
 #include <osgEarth/HeightFieldUtils>
-#include <osgEarth/Map>
-#include <osgEarth/Progress>
-#include <osgEarth/Utils>
 #include <osgEarthFeatures/FeatureCursor>
-#include <osgEarthFeatures/GeometryUtils>
-#include <osgEarthFeatures/FilterContext>
-#include <osgEarthSymbology/Query>
 
 using namespace osgEarth;
 using namespace osgEarth::Util;
@@ -61,7 +55,7 @@ namespace
     // clamp "a" to [lo..hi].
     double inline clamp(double a, double lo, double hi)
     {
-        return std::max(std::min(a, hi), lo);
+        return osg::maximum(osg::minimum(a, hi), lo);
     }
 
     typedef osg::Vec3d POINT;
@@ -90,7 +84,7 @@ namespace
     // This will not always work with polygons that contain holes,
     // so we need to come up with a different algorithm if this becomes a problem.
     // Maybe try a random point generator and profile it.
-    osg::Vec3d inline getInternalPoint(const Polygon* p)
+    osg::Vec3d inline getInternalPoint(const Symbology::Polygon* p)
     {
         // Simple test: if the centroid is in the polygon, use it.
         osg::Vec3d centroid = p->getBounds().center();
@@ -158,7 +152,7 @@ namespace
         return p->getBounds().center();
     }
 
-    double getDistanceSquaredToClosestEdge(const osg::Vec3d& P, const Polygon* poly)
+    double getDistanceSquaredToClosestEdge(const osg::Vec3d& P, const Symbology::Polygon* poly)
     {        
         double Dmin = DBL_MAX;
         ConstSegmentIterator segIter(poly, true);
@@ -233,7 +227,7 @@ namespace
                 double minD2 = DBL_MAX;//bufferWidth * bufferWidth; // minimum distance(squared) to closest polygon edge
                 double bufferWidth = 0.0;
 
-                const Polygon* bestPoly = 0L;
+                const Symbology::Polygon* bestPoly = 0L;
 
                 for (unsigned int geomIndex = 0; geomIndex < geom->getNumComponents(); geomIndex++)
                 {
@@ -242,7 +236,7 @@ namespace
                     ConstGeometryIterator giter(component, false);
                     while (giter.hasMore() && !done)
                     {
-                        const Polygon* polygon = dynamic_cast<const Polygon*>(giter.next());
+                        const Symbology::Polygon* polygon = dynamic_cast<const Symbology::Polygon*>(giter.next());
                         if (polygon)
                         {
                             // Does the point P fall within the polygon?
@@ -867,7 +861,7 @@ FlatteningLayer::createImplementation(const TileKey& key,
             Query query;        
             query.tileKey() = *i;
 
-            osg::ref_ptr<FeatureCursor> cursor = _featureSource->createFeatureCursor(query);
+            osg::ref_ptr<FeatureCursor> cursor = _featureSource->createFeatureCursor(query, progress);
             while (cursor.valid() && cursor->hasMore())
             {
                 Feature* feature = cursor->nextFeature();
@@ -915,7 +909,7 @@ FlatteningLayer::createImplementation(const TileKey& key,
         query.bounds() = queryExtent.bounds();
 
         // Run the query and fill the list.
-        osg::ref_ptr<FeatureCursor> cursor = _featureSource->createFeatureCursor(query);
+        osg::ref_ptr<FeatureCursor> cursor = _featureSource->createFeatureCursor(query, progress);
         while (cursor.valid() && cursor->hasMore())
         {
             Feature* feature = cursor->nextFeature();
