@@ -1,6 +1,6 @@
 /* -*-c++-*- */
-/* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
- * Copyright 2016 Pelican Mapping
+/* osgEarth - Geospatial SDK for OpenSceneGraph
+ * Copyright 2018 Pelican Mapping
  * http://osgearth.org
  *
  * osgEarth is free software; you can redistribute it and/or modify
@@ -17,37 +17,25 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>
  */
 #include <osgEarthUtil/MGRSGraticule>
-#include <osgEarthUtil/MGRSFormatter>
 #include <osgEarthUtil/UTMLabelingEngine>
 
-#include <osgEarthFeatures/GeometryCompiler>
 #include <osgEarthFeatures/TextSymbolizer>
-#include <osgEarthFeatures/FeatureSource>
 #include <osgEarthFeatures/TessellateOperator>
 
 #include <osgEarthAnnotation/FeatureNode>
 
 #include <osgEarth/Registry>
-#include <osgEarth/CullingUtils>
-#include <osgEarth/Utils>
 #include <osgEarth/PagedNode>
-#include <osgEarth/ShaderUtils>
 #include <osgEarth/Endian>
+#include <osgEarth/LineDrawable>
+#include <osgEarth/GLUtils>
+#include <osgEarth/Text>
 
-#include <osg/BlendFunc>
-#include <osg/PagedLOD>
 #include <osg/Depth>
-#include <osg/LogicOp>
-#include <osg/MatrixTransform>
-#include <osg/ClipNode>
-#include <osgDB/FileNameUtils>
-#include <osgDB/ReaderWriter>
 //#include <osgDB/WriteFile>
 
 #include <osgEarthDrivers/feature_ogr/OGRFeatureOptions>
 
-#include <fstream>
-#include <sstream>
 
 #define LC "[MGRSGraticule] "
 
@@ -59,12 +47,21 @@ using namespace osgEarth::Annotation;
 
 REGISTER_OSGEARTH_LAYER(mgrs_graticule, MGRSGraticule);
 
+#ifndef GL_CLIP_DISTANCE0
+#define GL_CLIP_DISTANCE0 0x3000
+#endif
+
+// whether to use SCREEN_COORDS to auto-size the text
+#define USE_SCREEN_COORDS
+
+
 //#define DEBUG_MODE
 
 //---------------------------------------------------------------------------
 
 MGRSGraticuleOptions::MGRSGraticuleOptions(const ConfigOptions& conf) :
-VisibleLayerOptions(conf)
+VisibleLayerOptions(conf),
+_useDefaultStyles(true)
 {
     _sqidURI.init(URI("../data/mgrs_sqid.bin", conf.referrer()));
     _styleSheet = new StyleSheet();
@@ -75,9 +72,8 @@ Config
 MGRSGraticuleOptions::getConfig() const
 {
     Config conf = VisibleLayerOptions::getConfig();
-    conf.key() = "mgrs_graticule";
     conf.set("sqid_data", _sqidURI);
-    conf.setObj("styles", _styleSheet);
+    conf.set("styles", _styleSheet);
     conf.set("use_default_styles", _useDefaultStyles);
     return conf;
 }
@@ -85,9 +81,9 @@ MGRSGraticuleOptions::getConfig() const
 void
 MGRSGraticuleOptions::fromConfig(const Config& conf)
 {
-    conf.getIfSet("sqid_data", _sqidURI);
-    conf.getObjIfSet("styles", _styleSheet);
-    conf.getIfSet("use_default_styles", _useDefaultStyles);
+    conf.get("sqid_data", _sqidURI);
+    conf.get("styles", _styleSheet);
+    conf.get("use_default_styles", _useDefaultStyles);
 }
 
 //---------------------------------------------------------------------------
@@ -125,7 +121,7 @@ namespace
         {
             // Read source data into an array:
             FeatureList sqids;
-            osg::ref_ptr<FeatureCursor> sqid_cursor = sqid_fs->createFeatureCursor();
+            osg::ref_ptr<FeatureCursor> sqid_cursor = sqid_fs->createFeatureCursor(0L);
             if (sqid_cursor.valid() && sqid_cursor->hasMore())
                 sqid_cursor->fill(sqids);
 
@@ -136,8 +132,8 @@ namespace
             // We will need a local XY SRS for geometry simplification:
             const SpatialReference* xysrs = SpatialReference::get("spherical-mercator");
 
-            u_long count = OE_ENCODE_LONG(sqids.size());
-            out.write(reinterpret_cast<const char*>(&count), sizeof(u_long));
+            u_int count = OE_ENCODE_LONG(sqids.size());
+            out.write(reinterpret_cast<const char*>(&count), sizeof(u_int));
 
             for (FeatureList::iterator i = sqids.begin(); i != sqids.end(); ++i)
             {
@@ -198,13 +194,13 @@ namespace
         if (fin.eof() || fin.is_open() == false)
             return false;
 
-        u_long count;
-        fin.read(reinterpret_cast<char*>(&count), sizeof(u_long));
+        u_int count;
+        fin.read(reinterpret_cast<char*>(&count), sizeof(u_int));
         count = OE_DECODE_LONG(count);
 
         const SpatialReference* wgs84 = SpatialReference::get("wgs84");
 
-        for (u_long i = 0; i < count; ++i)
+        for (u_int i = 0; i < count; ++i)
         {
             char gzd[4]; gzd[3] = 0;
             fin.read(gzd, 3);
@@ -223,7 +219,7 @@ namespace
             if (numPoints > 16384)
             {
                 OE_WARN << LC << "sqid bin file is corrupt.. abort!" << std::endl;
-                exit(-1);
+                return false;
             }
 
             osgEarth::Symbology::Ring* line = new osgEarth::Symbology::Ring();
@@ -326,16 +322,32 @@ MGRSGraticule::init()
 
     osg::StateSet* ss = this->getOrCreateStateSet();
 
-    // make the shared depth attr:
+    // disable the depth buffer
     ss->setAttributeAndModes(
         new osg::Depth(osg::Depth::ALWAYS, 0.f, 1.f, false),
         osg::StateAttribute::ON);
+    
+    // activate horizon clipping
+    ss->setMode(GL_CLIP_DISTANCE0, 1);
 
-    ss->setMode( GL_LIGHTING, 0 );
-    ss->setMode( GL_BLEND, 1 );
+    // blending on to support transculency
+    ss->setMode(GL_BLEND, 1);
 
     // force it to render after the terrain.
     ss->setRenderBinDetails(1, "RenderBin");
+
+    _root = new LocalRoot();
+
+    GLUtils::setLighting(_root->getOrCreateStateSet(), osg::StateAttribute::OFF);
+
+    // install the range callback for clip plane activation
+    _root->addCullCallback( new RangeUniformCullCallback() );
+
+    if (getEnabled() == true)
+    {
+        rebuild();
+    }
+
 }
 
 void
@@ -352,18 +364,8 @@ MGRSGraticule::removedFromMap(const Map* map)
 }
 
 osg::Node*
-MGRSGraticule::getOrCreateNode()
+MGRSGraticule::getNode() const
 {
-    if (_root.valid() == false)
-    {
-        _root = new LocalRoot();
-
-        // install the range callback for clip plane activation
-        _root->addCullCallback( new RangeUniformCullCallback() );
-
-        rebuild();
-    }
-
     return _root.get();
 }
 
@@ -711,7 +713,7 @@ namespace
         {
             GeometryCompilerOptions gco;
             gco.shaderPolicy() = SHADERPOLICY_INHERIT;
-            return new FeatureNode(0L, _sqidFeatures, _style, gco);
+            return new FeatureNode(_sqidFeatures, _style, gco);
         }
     };
 
@@ -778,6 +780,9 @@ namespace
             // make sure we get sufficient tessellation:
             compiler.options().maxGranularity() = 1.0;
 
+            // line shaders are at the root of the geometry tree, so don't install any
+            compiler.options().shaderPolicy() = SHADERPOLICY_INHERIT;
+
             FeatureList features;
 
             // longitudinal line:
@@ -815,13 +820,11 @@ namespace
             osg::Vec3d tileCenter;
             extent.getCentroid( tileCenter.x(), tileCenter.y() );
 
-            const SpatialReference* ecefSRS = extent.getSRS()->getECEF();
+            const SpatialReference* ecefSRS = extent.getSRS()->getGeocentricSRS();
     
             osg::Vec3d centerECEF;
             extent.getSRS()->transform( tileCenter, ecefSRS, centerECEF );
 
-            Registry::shaderGenerator().run(group, Registry::stateSetCache());
-    
             return ClusterCullingFactory::createAndInstall(group, centerECEF);
         }
         
@@ -863,13 +866,19 @@ namespace
             {
                 const Feature* feature = f->get();
                 std::string sqid = feature->getString("sqid");
-                osgText::Text* drawable = symbolizer.create(sqid);
-                drawable->setCharacterSizeMode(drawable->SCREEN_COORDS);
+                osgText::Text* drawable = new osgEarth::Text(sqid);
+                symbolizer.apply(drawable);
+#ifdef USE_SCREEN_COORDS
+                drawable->setCharacterSizeMode(osgText::Text::SCREEN_COORDS);
+#else
+                drawable->setCharacterSizeMode(osgText::Text::OBJECT_COORDS);
+                drawable->setCharacterSize(13000);
+#endif
                 drawable->getOrCreateStateSet()->setRenderBinToInherit();
             
                 GeoExtent extent(feature->getSRS(), feature->getGeometry()->getBounds());
 
-                const SpatialReference* ecef = feature->getSRS()->getECEF();
+                const SpatialReference* ecef = feature->getSRS()->getGeocentricSRS();
 
                 osg::Vec3d LL;
                 findPointClosestTo(feature, osg::Vec3d(extent.xMin(), extent.yMin(), 0), LL);
@@ -888,7 +897,7 @@ namespace
 
             OE_DEBUG << LC << "Created " << features.size() << " text elements for " << getName() << std::endl;
             
-            Registry::shaderGenerator().run(this, Registry::stateSetCache());
+            //Registry::shaderGenerator().run(this, Registry::stateSetCache());
         }
         
 #ifdef DEBUG_MODE
@@ -956,11 +965,20 @@ namespace
                 textSym->alignment() = textSym->ALIGN_LEFT_BASE_LINE;
         
             TextSymbolizer symbolizer( textSym.get() );
-            osgText::Text* drawable = symbolizer.create(getName());
+            osgText::Text* drawable = new osgEarth::Text(getName());
+            symbolizer.apply(drawable);
+
+#ifdef USE_SCREEN_COORDS
             drawable->setCharacterSizeMode(osgText::Text::SCREEN_COORDS);
+            //drawable->setPosition(osg::Vec3(0,0,0));
+            //drawable->setCharacterSize(36);
+#else
+            drawable->setCharacterSizeMode(osgText::Text::OBJECT_COORDS);
+            drawable->setCharacterSize(130000);
+#endif
             drawable->getOrCreateStateSet()->setRenderBinToInherit();
 
-            const SpatialReference* ecef = f->getSRS()->getECEF();
+            const SpatialReference* ecef = f->getSRS()->getGeocentricSRS();
             osg::Vec3d positionECEF;
             extent.getSRS()->transform( osg::Vec3d(extent.xMin(),extent.yMin(),0), ecef, positionECEF );
         
@@ -969,8 +987,6 @@ namespace
             osg::MatrixTransform* mt = new osg::MatrixTransform(L2W);
             mt->addChild(drawable); 
 
-            Registry::shaderGenerator().run(drawable, Registry::stateSetCache());
-
             return ClusterCullingFactory::createAndInstall(mt, positionECEF);
         }
 
@@ -1028,19 +1044,6 @@ MGRSGraticule::rebuild()
     // rebuild the graph:
     osg::Group* top = _root.get();
 
-    // Horizon clipping plane.
-    osg::ClipPlane* cp = _clipPlane.get();
-    if ( cp == 0L )
-    {
-        osg::ClipNode* clipNode = new osg::ClipNode();
-        osgEarth::Registry::shaderGenerator().run( clipNode );
-        cp = new osg::ClipPlane( 0 );
-        clipNode->addClipPlane( cp );
-        _root->addChild(clipNode);
-        top = clipNode;
-    }
-    top->addCullCallback( new ClipToGeocentricHorizon(_profile->getSRS(), cp) );
-
 #if 0
     // Uncomment to write out a SQID data file
     writeSQIDfile(options().sqidData().get());
@@ -1057,10 +1060,14 @@ MGRSGraticule::rebuild()
             table[i->get()->getString("gzd")].push_back(i->get());
         }
 
-        osg::Group* geomTop = new osg::Group();
+        // Root of the geometry tree
+        osg::Group* geomTop = new LineGroup(); //osg::Group();
         top->addChild(geomTop);
 
+        // Root of the text tree
         osg::Group* textTop = new osg::Group();
+        osg::StateSet* textSS = textTop->getOrCreateStateSet();
+        //TextSymbolizer::insta(textSS);
         top->addChild(textTop);
 
         // build the GZD feature set
@@ -1108,6 +1115,12 @@ MGRSGraticule::rebuild()
         if (ss->getStyle("1", false)) maxRes = 1.0;
         labeler->setMaxResolution(maxRes);
 
+        // Set labeler styles
+        const Style* xEdgeStyle = ss->getStyle("xedge", false);
+        const Style* yEdgeStyle = ss->getStyle("yedge", false);
+        if (xEdgeStyle && yEdgeStyle)
+            labeler->setStyles(*xEdgeStyle, *yEdgeStyle);
+
         osg::ref_ptr<StateSetCache> sscache = new StateSetCache();
         sscache->optimize(geomTop);
         sscache->optimize(textTop);
@@ -1221,7 +1234,7 @@ MGRSGraticule::setUpDefaultStyles()
             line->stroke()->width() = 4.0;
             line->tessellation() = 20;
             TextSymbol* text = style.getOrCreate<TextSymbol>();
-            text->fill()->color() = Color::Gray;
+            text->fill()->color() = Color::White;
             text->halo()->color() = Color::Black;
             text->alignment() = TextSymbol::ALIGN_LEFT_BOTTOM;
             styles->addStyle(style);
@@ -1243,7 +1256,7 @@ MGRSGraticule::setUpDefaultStyles()
             line->stroke()->color().set(1,1,0,alpha);
             line->stroke()->width() = 3;
             TextSymbol* text = style.getOrCreate<TextSymbol>();
-            text->fill()->color() = Color::Gray;
+            text->fill()->color() = Color::White;
             text->halo()->color() = Color::Black;
             text->alignment() = TextSymbol::ALIGN_LEFT_BOTTOM;
             styles->addStyle(style);
@@ -1300,502 +1313,3 @@ MGRSGraticule::setUpDefaultStyles()
         }
     }
 }
-
-// ------ OLD CODE --------------------------------------------------------
-
-
-#if 0
-
-osg::Node*
-MGRSGraticule::buildSQIDTiles( const std::string& gzd )
-{
-    const GeoExtent& extent = _utmData.sectorTable()[gzd];
-
-    // parse the GZD into its components:
-    unsigned zone;
-    char letter;
-    sscanf( gzd.c_str(), "%u%c", &zone, &letter );
-    
-    const TextSymbol* textSymFromOptions = options().sqidStyle()->get<TextSymbol>();
-    if ( !textSymFromOptions )
-        textSymFromOptions = options().sqidStyle()->get<TextSymbol>();
-
-    // copy it since we intend to alter it
-    osg::ref_ptr<TextSymbol> textSym = 
-        textSymFromOptions ? new TextSymbol(*textSymFromOptions) :
-        new TextSymbol();
-
-    double h = 0.0;
-
-    TextSymbolizer ts( textSym );
-    MGRSFormatter mgrs(MGRSFormatter::PRECISION_100000M);
-    osg::Geode* textGeode = new osg::Geode();
-
-    const SpatialReference* ecefSRS = extent.getSRS()->getECEF();
-    osg::Vec3d centerMap, centerECEF;
-    extent.getCentroid(centerMap.x(), centerMap.y());
-    extent.getSRS()->transform(centerMap, ecefSRS, centerECEF);
-
-    osg::Matrix local2world;
-    ecefSRS->createLocalToWorld( centerECEF, local2world );
-    osg::Matrix world2local;
-    world2local.invert(local2world);
-
-    FeatureList features;
-
-    std::vector<GeoExtent> sqidExtents;
-
-    // UTM:
-    if ( letter > 'B' && letter < 'Y' )
-    {
-        // grab the SRS for the current UTM zone:
-        // TODO: AL/AA designation??
-        const SpatialReference* utm = SpatialReference::create(
-            Stringify() << "+proj=utm +zone=" << zone << " +north +units=m" );
-
-        // transform the four corners of the tile to UTM.
-        osg::Vec3d gzdUtmSW, gzdUtmSE, gzdUtmNW, gzdUtmNE;
-        extent.getSRS()->transform( osg::Vec3d(extent.xMin(),extent.yMin(),h), utm, gzdUtmSW );
-        extent.getSRS()->transform( osg::Vec3d(extent.xMin(),extent.yMax(),h), utm, gzdUtmNW );
-        extent.getSRS()->transform( osg::Vec3d(extent.xMax(),extent.yMin(),h), utm, gzdUtmSE );
-        extent.getSRS()->transform( osg::Vec3d(extent.xMax(),extent.yMax(),h), utm, gzdUtmNE );
-
-        // find the southern boundary of the first full SQID tile in the GZD tile.
-        double southSQIDBoundary = gzdUtmSW.y(); //extentUTM.yMin();
-        double remainder = fmod( southSQIDBoundary, 100000.0 );
-        if ( remainder > 0.0 )
-            southSQIDBoundary += (100000.0 - remainder);
-
-        // find the min/max X for this cell in UTM:
-        double xmin = extent.yMin() >= 0.0 ? gzdUtmSW.x() : gzdUtmNW.x();
-        double xmax = extent.yMin() >= 0.0 ? gzdUtmSE.x() : gzdUtmNE.x();
-
-        // Record the UTM extent of each SQID cell in this tile.
-        // Go from the south boundary northwards:
-        for( double y = southSQIDBoundary; y < gzdUtmNW.y(); y += 100000.0 )
-        {
-            // start at the central meridian (500K) and go west:
-            for( double x = 500000.0; x > xmin; x -= 100000.0 )
-            {
-                sqidExtents.push_back( GeoExtent(utm, x-100000.0, y, x, y+100000.0) );
-            }
-
-            // then start at the central meridian and go east:
-            for( double x = 500000.0; x < xmax; x += 100000.0 )
-            {
-                sqidExtents.push_back( GeoExtent(utm, x, y, x+100000.0, y+100000.0) );
-            }
-        }
-
-        for( std::vector<GeoExtent>::iterator i = sqidExtents.begin(); i != sqidExtents.end(); ++i )
-        {
-            GeoExtent utmEx = *i;
-
-            // now, clamp each of the points in the UTM SQID extent to the map-space
-            // boundaries of the GZD tile. (We only need to clamp in the X dimension,
-            // Y geometry is allowed to overflow.) Also, skip NE, we don't need it.
-            double r, xlimit;
-
-            osg::Vec3d sw(utmEx.xMin(), utmEx.yMin(), 0);
-            r = (sw.y()-gzdUtmSW.y())/(gzdUtmNW.y()-gzdUtmSW.y());
-            xlimit = gzdUtmSW.x() + r * (gzdUtmNW.x() - gzdUtmSW.x());
-            if ( sw.x() < xlimit ) sw.x() = xlimit;
-
-            osg::Vec3d nw(utmEx.xMin(), utmEx.yMax(), 0);
-            r = (nw.y()-gzdUtmSW.y())/(gzdUtmNW.y()-gzdUtmSW.y());
-            xlimit = gzdUtmSW.x() + r * (gzdUtmNW.x() - gzdUtmSW.x());
-            if ( nw.x() < xlimit ) nw.x() = xlimit;
-            
-            osg::Vec3d se(utmEx.xMax(), utmEx.yMin(), 0);
-            r = (se.y()-gzdUtmSE.y())/(gzdUtmNE.y()-gzdUtmSE.y());
-            xlimit = gzdUtmSE.x() + r * (gzdUtmNE.x() - gzdUtmSE.x());
-            if ( se.x() > xlimit ) se.x() = xlimit;
-
-            // at the northernmost GZD (lateral band X), clamp the northernmost SQIDs to the upper latitude.
-            if ( letter == 'X' && nw.y() > gzdUtmNW.y() ) 
-                nw.y() = gzdUtmNW.y();
-
-            // need this in order to calculate the font size:
-            double utmWidth = se.x() - sw.x();
-
-            // now transform the corner points back into the map SRS:
-            utm->transform( sw, extent.getSRS(), sw );
-            utm->transform( nw, extent.getSRS(), nw );
-            utm->transform( se, extent.getSRS(), se );
-
-            // and draw valid sqid geometry.
-            if ( sw.x() < se.x() )
-            {
-                Feature* lat = new Feature(new LineString(2), extent.getSRS());
-                lat->geoInterp() = GEOINTERP_RHUMB_LINE;
-                lat->getGeometry()->push_back( sw );
-                lat->getGeometry()->push_back( se );
-                features.push_back(lat);
-
-                Feature* lon = new Feature(new LineString(2), extent.getSRS());
-                lon->geoInterp() = GEOINTERP_GREAT_CIRCLE;
-                lon->getGeometry()->push_back( sw );
-                lon->getGeometry()->push_back( nw );
-                features.push_back(lon);
-
-                // and the text label:
-                osg::Vec3d sqidTextMap = (nw + se) * 0.5;
-                sqidTextMap.z() += 1000.0;
-                osg::Vec3d sqidTextECEF;
-                extent.getSRS()->transform(sqidTextMap, ecefSRS, sqidTextECEF);
-                osg::Vec3d sqidLocal;
-                sqidLocal = sqidTextECEF * world2local;
-
-                MGRSCoord mgrsCoord;
-                if ( mgrs.transform( GeoPoint(extent.getSRS(),sqidTextMap,ALTMODE_ABSOLUTE), mgrsCoord) )
-                {
-                    textSym->size() = utmWidth/3.0;        
-                    osgText::Text* d = ts.create( mgrsCoord.sqid );
-
-                    osg::Matrixd textLocal2World;
-                    ecefSRS->createLocalToWorld( sqidTextECEF, textLocal2World );
-
-                    d->setPosition( sqidLocal );
-                    textGeode->addDrawable( d );
-                }
-            }
-        }
-    }
-
-    else if ( letter == 'A' || letter == 'B' )
-    {
-        // SRS for south polar region UPS projection. This projection has (0,0) at the
-        // south pole, with +X extending towards 90 degrees E longitude and +Y towards
-        // 0 degrees longitude.
-        const SpatialReference* ups = SpatialReference::create(
-            "+proj=stere +lat_ts=-90 +lat_0=-90 +lon_0=0 +k_0=1 +x_0=0 +y_0=0");
-
-        osg::Vec3d gtemp;
-        double r = GeoMath::distance(-osg::PI_2, 0.0, -1.3962634, 0.0); // -90 => -80 latitude
-        double r2 = r*r;
-
-        if ( letter == 'A' )
-        {
-            for( double x = 0.0; x < 1200000.0; x += 100000.0 )
-            {
-                double yminmax = sqrt( r2 - x*x );
-                Feature* f = new Feature( new LineString(2), extent.getSRS() );
-                f->geoInterp() = GEOINTERP_GREAT_CIRCLE;
-                osg::Vec3d p0, p1;
-                ups->transform( osg::Vec3d(-x, -yminmax, 0), extent.getSRS(), p0 );
-                ups->transform( osg::Vec3d(-x,  yminmax, 0), extent.getSRS(), p1 );
-                f->getGeometry()->push_back( p0 );
-                f->getGeometry()->push_back( p1 );
-                features.push_back( f );
-            }
-
-            for( double y = -1100000.0; y < 1200000.0; y += 100000.0 )
-            {
-                double xmax = sqrt( r2 - y*y );
-                Feature* f = new Feature( new LineString(2), extent.getSRS() );
-                f->geoInterp() = GEOINTERP_GREAT_CIRCLE;
-                osg::Vec3d p0, p1;
-                ups->transform( osg::Vec3d(-xmax, y, 0), extent.getSRS(), p0 );
-                ups->transform( osg::Vec3d(    0, y, 0), extent.getSRS(), p1 );
-                f->getGeometry()->push_back( p0 );
-                f->getGeometry()->push_back( p1 );
-                features.push_back( f );
-            }
-
-            for( double x = -1200000.0; x < 0.0; x += 100000.0 )
-            {
-                for( double y = -1200000.0; y < 1200000.0; y += 100000.0 )
-                {
-                    osg::Vec3d sqidTextMap;
-                    ups->transform( osg::Vec3d(x+50000.0, y+50000.0, 0), extent.getSRS(), sqidTextMap);
-                    if ( sqidTextMap.y() < -80.0 )
-                    {
-                        sqidTextMap.z() += 1000.0;
-                        osg::Vec3d sqidTextECEF;
-                        extent.getSRS()->transform(sqidTextMap, ecefSRS, sqidTextECEF);
-                        osg::Vec3d sqidLocal = sqidTextECEF * world2local;
-
-                        MGRSCoord mgrsCoord;
-                        if ( mgrs.transform( GeoPoint(extent.getSRS(),sqidTextMap,ALTMODE_ABSOLUTE), mgrsCoord) )
-                        {
-                            textSym->size() = 33000.0;
-                            osgText::Text* d = ts.create( mgrsCoord.sqid );
-                            osg::Matrixd textLocal2World;
-                            ecefSRS->createLocalToWorld( sqidTextECEF, textLocal2World );
-                            d->setPosition( sqidLocal );
-                            textGeode->addDrawable( d );
-                        }
-                    }
-                }
-            }
-        }
-
-        else if ( letter == 'B' )
-        {
-            for( double x = 100000.0; x < 1200000.0; x += 100000.0 )
-            {
-                double yminmax = sqrt( r2 - x*x );
-                Feature* f = new Feature( new LineString(2), extent.getSRS() );
-                f->geoInterp() = GEOINTERP_GREAT_CIRCLE;
-                osg::Vec3d p0, p1;
-                ups->transform( osg::Vec3d(x, -yminmax, 0), extent.getSRS(), p0 );
-                ups->transform( osg::Vec3d(x,  yminmax, 0), extent.getSRS(), p1 );
-                f->getGeometry()->push_back( p0 );
-                f->getGeometry()->push_back( p1 );
-                features.push_back( f );
-            }
-
-            for( double y = -1100000.0; y < 1200000.0; y += 100000.0 )
-            {
-                double xmax = sqrt( r2 - y*y );
-                Feature* f = new Feature( new LineString(2), extent.getSRS() );
-                f->geoInterp() = GEOINTERP_GREAT_CIRCLE;
-                osg::Vec3d p0, p1;
-                ups->transform( osg::Vec3d(    0, y, 0), extent.getSRS(), p0 );
-                ups->transform( osg::Vec3d( xmax, y, 0), extent.getSRS(), p1 );
-                f->getGeometry()->push_back( p0 );
-                f->getGeometry()->push_back( p1 );
-                features.push_back( f );
-            }
-
-            for( double x = 0.0; x < 1200000.0; x += 100000.0 )
-            {
-                for( double y = -1200000.0; y < 1200000.0; y += 100000.0 )
-                {
-                    osg::Vec3d sqidTextMap;
-                    ups->transform( osg::Vec3d(x+50000.0, y+50000.0, 0), extent.getSRS(), sqidTextMap);
-                    if ( sqidTextMap.y() < -80.0 )
-                    {
-                        sqidTextMap.z() += 1000.0;
-                        osg::Vec3d sqidTextECEF;
-                        extent.getSRS()->transform(sqidTextMap, ecefSRS, sqidTextECEF);
-                        //extent.getSRS()->transformToECEF(sqidTextMap, sqidTextECEF);
-                        osg::Vec3d sqidLocal = sqidTextECEF * world2local;
-
-                        MGRSCoord mgrsCoord;
-                        if ( mgrs.transform( GeoPoint(extent.getSRS(),sqidTextMap,ALTMODE_ABSOLUTE), mgrsCoord) )
-                        {
-                            textSym->size() = 33000.0;
-                            osgText::Text* d = ts.create( mgrsCoord.sqid );
-                            osg::Matrixd textLocal2World;
-                            ecefSRS->createLocalToWorld( sqidTextECEF, textLocal2World );
-                            d->setPosition( sqidLocal );
-                            textGeode->addDrawable( d );
-                        }
-                    }
-                }
-            }
-        }
-    }
-
-    else if ( letter == 'Y' || letter == 'Z' )
-    {
-        // SRS for north polar region UPS projection. This projection has (0,0) at the
-        // south pole, with +X extending towards 90 degrees E longitude and +Y towards
-        // 180 degrees longitude.
-        const SpatialReference* ups = SpatialReference::create(
-            "+proj=stere +lat_ts=90 +lat_0=90 +lon_0=0 +k_0=1 +x_0=0 +y_0=0");
-
-        osg::Vec3d gtemp;
-        double r = GeoMath::distance(osg::PI_2, 0.0, 1.46607657, 0.0); // 90 -> 84 latitude
-        double r2 = r*r;
-
-        if ( letter == 'Y' )
-        {
-            for( double x = 0.0; x < 700000.0; x += 100000.0 )
-            {
-                double yminmax = sqrt( r2 - x*x );
-                Feature* f = new Feature( new LineString(2), extent.getSRS() );
-                f->geoInterp() = GEOINTERP_GREAT_CIRCLE;
-                osg::Vec3d p0, p1;
-                ups->transform( osg::Vec3d(-x, -yminmax, 0), extent.getSRS(), p0 );
-                ups->transform( osg::Vec3d(-x,  yminmax, 0), extent.getSRS(), p1 );
-                f->getGeometry()->push_back( p0 );
-                f->getGeometry()->push_back( p1 );
-                features.push_back( f );
-            }
-
-            for( double y = -600000.0; y < 700000.0; y += 100000.0 )
-            {
-                double xmax = sqrt( r2 - y*y );
-                Feature* f = new Feature( new LineString(2), extent.getSRS() );
-                f->geoInterp() = GEOINTERP_GREAT_CIRCLE;
-                osg::Vec3d p0, p1;
-                ups->transform( osg::Vec3d(-xmax, y, 0), extent.getSRS(), p0 );
-                ups->transform( osg::Vec3d(    0, y, 0), extent.getSRS(), p1 );
-                f->getGeometry()->push_back( p0 );
-                f->getGeometry()->push_back( p1 );
-                features.push_back( f );
-            }
-
-            for( double x = -700000.0; x < 0.0; x += 100000.0 )
-            {
-                for( double y = -700000.0; y < 700000.0; y += 100000.0 )
-                {
-                    osg::Vec3d sqidTextMap;
-                    ups->transform( osg::Vec3d(x+50000.0, y+50000.0, 0), extent.getSRS(), sqidTextMap);
-                    if ( sqidTextMap.y() > 84.0 )
-                    {
-                        sqidTextMap.z() += 1000.0;
-                        osg::Vec3d sqidTextECEF;
-                        extent.getSRS()->transform(sqidTextMap, ecefSRS, sqidTextECEF);
-                        //extent.getSRS()->transformToECEF(sqidTextMap, sqidTextECEF);
-                        osg::Vec3d sqidLocal = sqidTextECEF * world2local;
-
-                        MGRSCoord mgrsCoord;
-                        if ( mgrs.transform( GeoPoint(extent.getSRS(),sqidTextMap,ALTMODE_ABSOLUTE), mgrsCoord) )
-                        {
-                            textSym->size() = 33000.0;
-                            osgText::Text* d = ts.create( mgrsCoord.sqid );
-                            osg::Matrixd textLocal2World;
-                            ecefSRS->createLocalToWorld( sqidTextECEF, textLocal2World );
-                            d->setPosition( sqidLocal );
-                            textGeode->addDrawable( d );
-                        }
-                    }
-                }
-            }
-        }
-
-        else if ( letter == 'Z' )
-        {
-            for( double x = 100000.0; x < 700000.0; x += 100000.0 )
-            {
-                double yminmax = sqrt( r2 - x*x );
-                Feature* f = new Feature( new LineString(2), extent.getSRS() );
-                f->geoInterp() = GEOINTERP_GREAT_CIRCLE;
-                osg::Vec3d p0, p1;
-                ups->transform( osg::Vec3d(x, -yminmax, 0), extent.getSRS(), p0 );
-                ups->transform( osg::Vec3d(x,  yminmax, 0), extent.getSRS(), p1 );
-                f->getGeometry()->push_back( p0 );
-                f->getGeometry()->push_back( p1 );
-                features.push_back( f );
-            }
-
-            for( double y = -600000.0; y < 700000.0; y += 100000.0 )
-            {
-                double xmax = sqrt( r2 - y*y );
-                Feature* f = new Feature( new LineString(2), extent.getSRS() );
-                f->geoInterp() = GEOINTERP_GREAT_CIRCLE;
-                osg::Vec3d p0, p1;
-                ups->transform( osg::Vec3d(    0, y, 0), extent.getSRS(), p0 );
-                ups->transform( osg::Vec3d( xmax, y, 0), extent.getSRS(), p1 );
-                f->getGeometry()->push_back( p0 );
-                f->getGeometry()->push_back( p1 );
-                features.push_back( f );
-            }
-
-            for( double x = 0.0; x < 700000.0; x += 100000.0 )
-            {
-                for( double y = -700000.0; y < 700000.0; y += 100000.0 )
-                {
-                    osg::Vec3d sqidTextMap;
-                    ups->transform( osg::Vec3d(x+50000.0, y+50000.0, 0), extent.getSRS(), sqidTextMap);
-                    if ( sqidTextMap.y() > 84.0 )
-                    {
-                        sqidTextMap.z() += 1000.0;
-                        osg::Vec3d sqidTextECEF;
-                        extent.getSRS()->transform(sqidTextMap, ecefSRS, sqidTextECEF);
-                        //extent.getSRS()->transformToECEF(sqidTextMap, sqidTextECEF);
-                        osg::Vec3d sqidLocal = sqidTextECEF * world2local;
-
-                        MGRSCoord mgrsCoord;
-                        if ( mgrs.transform( GeoPoint(extent.getSRS(),sqidTextMap,ALTMODE_ABSOLUTE), mgrsCoord) )
-                        {
-                            textSym->size() = 33000.0;
-                            osgText::Text* d = ts.create( mgrsCoord.sqid );
-                            osg::Matrixd textLocal2World;
-                            ecefSRS->createLocalToWorld( sqidTextECEF, textLocal2World );
-                            d->setPosition( sqidLocal );
-                            textGeode->addDrawable( d );
-                        }
-                    }
-                }
-            }
-        }
-    }
-
-    osg::Group* group = new osg::Group();
-
-    osg::ref_ptr<const Map> map;
-    if (_map.lock(map))
-    {
-        Style lineStyle;
-        lineStyle.add( options().sqidStyle()->get<LineSymbol>() );
-
-        GeometryCompiler compiler;
-        osg::ref_ptr<Session> session = new Session(map.get());
-        FilterContext context( session.get(), _featureProfile.get(), extent );
-
-        // make sure we get sufficient tessellation:
-        compiler.options().maxGranularity() = 0.25;
-
-        osg::Node* geomNode = compiler.compile(features, lineStyle, context);
-        if ( geomNode ) 
-            group->addChild( geomNode );
-
-        osg::MatrixTransform* mt = new osg::MatrixTransform(local2world);
-        mt->addChild(textGeode);
-        group->addChild( mt );
-
-        Registry::shaderGenerator().run(textGeode, Registry::stateSetCache());
-    }
-
-    return group;
-}
-
-//---------------------------------------------------------------------------
-
-namespace osgEarth { namespace Util
-{
-    // OSG Plugin for loading subsequent graticule levels
-    class MGRSGraticulePseudoLoader : public osgDB::ReaderWriter
-    {
-    public:
-        MGRSGraticulePseudoLoader()
-        {
-            supportsExtension( MGRS_GRATICULE_PSEUDOLOADER_EXTENSION, "osgEarth MGRS graticule" );
-        }
-
-        const char* className() const
-        {
-            return "osgEarth MGRS graticule LOD loader";
-        }
-
-        bool acceptsExtension(const std::string& extension) const
-        {
-            return osgDB::equalCaseInsensitive(extension, MGRS_GRATICULE_PSEUDOLOADER_EXTENSION);
-        }
-
-        ReadResult readNode(const std::string& uri, const Options* options) const
-        {        
-            std::string ext = osgDB::getFileExtension( uri );
-            if ( !acceptsExtension( ext ) )
-                return ReadResult::FILE_NOT_HANDLED;
-
-            if ( !options )
-            {
-                OE_WARN << LC << "INTERNAL ERROR: MGRSGraticule object not present in Options (1)\n";
-                return ReadResult::ERROR_IN_READING_FILE;
-            }
-
-            osg::ref_ptr<MGRSGraticule> graticule;
-            if (!OptionsData<MGRSGraticule>::lock(options, "osgEarth.MGRSGraticule", graticule))
-            {
-                OE_WARN << LC << "INTERNAL ERROR: MGRSGraticule object not present in Options (2)\n";
-                return ReadResult::ERROR_IN_READING_FILE;
-            }
-
-            std::string def = osgDB::getNameLessExtension(uri);
-            std::string gzd = osgDB::getNameLessExtension(def);
-            
-            osg::Node* result = graticule->buildSQIDTiles( gzd );
-
-            return result ? ReadResult(result) : ReadResult::ERROR_IN_READING_FILE;
-        }
-    };
-    REGISTER_OSGPLUGIN(MGRS_GRATICULE_PSEUDOLOADER_EXTENSION, MGRSGraticulePseudoLoader);
-
-} } // namespace osgEarth::Util
-
-#endif
\ No newline at end of file
